@@ -31,6 +31,9 @@
 /* define this if you have zlib compression available */
 #undef HAVE_ZLIB
 
+/* define this if you have OpenLDAP client available */
+#undef HAVE_LDAP
+
 /* define this if you have the libical calendaring library available */
 #undef HAVE_LIBICAL
 
