@@ -1,5 +1,5 @@
 /*
- * $Id: citadel.c,v 613.0 2003/12/15 16:37:02 ajc Exp $
+ * $Id: citadel.c,v 620.0 2004/03/27 02:33:28 ajc Exp $
  *
  * Main source module for the client program.
  */
@@ -68,7 +68,6 @@ char editor_paths[MAX_EDITORS][SIZ];	/* paths to external editors */
 char printcmd[SIZ];		/* print command */
 int editor_pid = (-1);
 char fullname[USERNAME_SIZE];
-struct CtdlServInfo serv_info;	/* Info on the server connected */
 int screenwidth;
 int screenheight;
 unsigned room_flags;
@@ -379,7 +378,8 @@ void dotgoto(CtdlIPC *ipc, char *towhere, int display_name, int fromungoto)
 	 * If a match is not found, try a partial match.
 	 * Partial matches anywhere in the string carry a weight of 1,
 	 * left-aligned matches carry a weight of 2.  Pick the room that
-	 * has the highest-weighted match.
+	 * has the highest-weighted match.  Do not match on forgotten
+	 * rooms.
 	 */
 	if (r / 100 != 2) {
 		struct march *march = NULL;
@@ -387,7 +387,7 @@ void dotgoto(CtdlIPC *ipc, char *towhere, int display_name, int fromungoto)
 		best_match = 0;
 		strcpy(bbb, "");
 
-		r = CtdlIPCKnownRooms(ipc, AllAccessibleRooms, AllFloors, &march, aaa);
+		r = CtdlIPCKnownRooms(ipc, SubscribedRooms, AllFloors, &march, aaa);
 		if (r / 100 == 1) {
 			/* Run the roomlist; free the data as we go */
 			struct march *mp = march;	/* Current */
@@ -479,7 +479,7 @@ void dotgoto(CtdlIPC *ipc, char *towhere, int display_name, int fromungoto)
 			system(rc_gotmail_cmd);
 		}
 	}
-	status_line(serv_info.serv_humannode, serv_info.serv_bbs_city,
+	status_line(ipc->ServInfo.humannode, ipc->ServInfo.bbs_city,
 			room_name, secure, newmailcount);
 }
 
@@ -626,6 +626,30 @@ void gotofloor(CtdlIPC *ipc, char *towhere, int mode)
 		return;
 	}
 
+	/* Find first known room on the floor */
+
+	strcpy(targ, "");
+	mptr = NULL;
+	r = CtdlIPCKnownRooms(ipc, SubscribedRooms, tofloor, &mptr, buf);
+	if (r / 100 == 1) {
+		struct march *tmp = mptr;
+
+		/* TODO: room order is being ignored? */
+		if (mptr)
+			strncpy(targ, mptr->march_name, ROOMNAMELEN);
+		while (mptr) {
+			tmp = mptr->next;
+			free(mptr);
+			mptr = tmp;
+		}
+	}
+	if (strlen(targ) > 0) {
+		gf_toroom(ipc, targ, mode);
+		return;
+	}
+
+	/* No known rooms on the floor; unzap the first one then */
+
 	strcpy(targ, "");
 	mptr = NULL;
 	r = CtdlIPCKnownRooms(ipc, AllAccessibleRooms, tofloor, &mptr, buf);
@@ -699,9 +723,9 @@ void check_screen_dims(void)
 /*
  * set floor mode depending on client, server, and user settings
  */
-void set_floor_mode(void)
+void set_floor_mode(CtdlIPC* ipc)
 {
-	if (serv_info.serv_ok_floors == 0) {
+	if (ipc->ServInfo.ok_floors == 0) {
 		floor_mode = 0;	/* Don't use floors if the server */
 	}
 	/* doesn't support them!          */
@@ -757,14 +781,14 @@ void get_serv_info(CtdlIPC *ipc, char *supplied_hostname)
 {
 	char buf[SIZ];
 
-	CtdlIPCServerInfo(ipc, &serv_info, buf);
+	CtdlIPCServerInfo(ipc, buf);
 
 	/* be nice and identify ourself to the server */
 	CtdlIPCIdentifySoftware(ipc, SERVER_TYPE, 0, REV_LEVEL,
 		 (ipc->isLocal ? "local" : CITADEL),
 		 (supplied_hostname) ? supplied_hostname : 
 		 /* Look up the , in the bible if you're confused */
-		 (locate_host(buf), buf), buf);
+		 (locate_host(ipc, buf), buf), buf);
 
 	/* Tell the server what our preferred content formats are */
 	if ((CtdlIPCSpecifyPreferredFormats(ipc, buf, "text/html|text/plain") / 100 )== 2) {
@@ -1023,6 +1047,7 @@ int main(int argc, char **argv)
 	load_command_set();	/* parse the citadel.rc file */
 	sttybbs(SB_NO_INTR);	/* Install the new ones */
 	signal(SIGHUP, dropcarr);	/* Cleanup gracefully if carrier is dropped */
+	signal(SIGPIPE, dropcarr);	/* Cleanup gracefully if local conn. dropped */
 	signal(SIGTERM, dropcarr);	/* Cleanup gracefully if terminated */
 	signal(SIGCONT, catch_sigcont);	/* Catch SIGCONT so we can reset terminal */
 #ifdef SIGWINCH
@@ -1126,7 +1151,7 @@ int main(int argc, char **argv)
 #endif
 	ipc_for_signal_handlers = ipc;	/* KLUDGE cover your eyes */
 
-	CtdlIPC_getline(ipc, aaa);
+	CtdlIPC_chat_recv(ipc, aaa);
 	if (aaa[0] != '2') {
 		scr_printf("%s\n", &aaa[4]);
 		logoff(ipc, atoi(aaa));
@@ -1166,11 +1191,11 @@ int main(int argc, char **argv)
 #endif
 
 	get_serv_info(ipc, telnet_client_host);
-	scr_printf("%-24s\n%s\n%s\n", serv_info.serv_software, serv_info.serv_humannode,
-		   serv_info.serv_bbs_city);
+	scr_printf("%-24s\n%s\n%s\n", ipc->ServInfo.software, ipc->ServInfo.humannode,
+		   ipc->ServInfo.bbs_city);
 	scr_flush();
 
-	status_line(serv_info.serv_humannode, serv_info.serv_bbs_city, NULL,
+	status_line(ipc->ServInfo.humannode, ipc->ServInfo.bbs_city, NULL,
 		    secure, -1);
 
 	screenwidth = 80;	/* default screen dimensions */
@@ -1257,7 +1282,8 @@ int main(int argc, char **argv)
 
 NEWUSR:	if (strlen(rc_password) == 0) {
 		scr_printf("'%s' not found.\n"
-			"Do you want to create a new account with this name? ");
+			"Do you want to create a new account with this name? ",
+			fullname);
 		if (yesno() == 0)
 			goto GSTA;
 	}
@@ -1346,7 +1372,7 @@ NEWUSR:	if (strlen(rc_password) == 0) {
 	check_screen_dims();
 #endif
 
-	set_floor_mode();
+	set_floor_mode(ipc);
 
 	/* Enter the lobby */
 	dotgoto(ipc, "_BASEROOM_", 1, 0);
@@ -1382,14 +1408,16 @@ NEWUSR:	if (strlen(rc_password) == 0) {
 				entmsg(ipc, 0, 2);
 				break;
 			case 78:
-				newprompt("What do you want your username to be? ", aaa, 32);
-				snprintf(bbb, sizeof bbb, "ENT0 2|0|0|0|%s", aaa);
-				CtdlIPC_putline(ipc, bbb);
-				CtdlIPC_getline(ipc, aaa);
-				if (strncmp("200", aaa, 3))
-					scr_printf("\n%s\n", aaa);
-				else
-					entmsg(ipc, 0, 0);
+				{
+					/* Only m.author is used */
+					struct ctdlipcmessage m;
+					newprompt("What do you want your username to be? ", m.author, USERNAME_SIZE - 1);
+					r = CtdlIPCPostMessage(ipc, 2, &m, aaa);
+					if (r / 100 != 2)
+						scr_printf("%s\n", aaa);
+					else
+						entmsg(ipc, 0, 0);
+				}
 				break;
 			case 5:				/* <G>oto */
 				updatels(ipc);
@@ -1545,19 +1573,19 @@ NEWUSR:	if (strlen(rc_password) == 0) {
 				enternew(ipc, "roomname", aaa, 20);
 				r = CtdlIPCChangeRoomname(ipc, aaa, bbb);
 				if (r / 100 != 2)
-					scr_printf("\n%s\n", aaa);
+					scr_printf("\n%s\n", bbb);
 				break;
 			case 76:
 				enternew(ipc, "hostname", aaa, 25);
 				r = CtdlIPCChangeHostname(ipc, aaa, bbb);
 				if (r / 100 != 2)
-					scr_printf("\n%s\n", aaa);
+					scr_printf("\n%s\n", bbb);
 				break;
 			case 77:
 				enternew(ipc, "username", aaa, 32);
 				r = CtdlIPCChangeUsername(ipc, aaa, bbb);
 				if (r / 100 != 2)
-					scr_printf("\n%s\n", aaa);
+					scr_printf("\n%s\n", bbb);
 				break;
 
 			case 35:
@@ -1706,12 +1734,12 @@ NEWUSR:	if (strlen(rc_password) == 0) {
 
 			case 37:
 				enter_config(ipc, 0);
-				set_floor_mode();
+				set_floor_mode(ipc);
 				break;
 
 			case 59:
 				enter_config(ipc, 3);
-				set_floor_mode();
+				set_floor_mode(ipc);
 				break;
 
 			case 60:
