@@ -1,4 +1,4 @@
-/* $Id: citadel_ipc.c,v 613.0 2003/12/15 16:37:02 ajc Exp $ */
+/* $Id: citadel_ipc.c,v 620.0 2004/03/27 02:33:30 ajc Exp $ */
 
 #include "sysdep.h"
 #if TIME_WITH_SYS_TIME
@@ -77,6 +77,8 @@ static void endtls(SSL *ssl);
 static unsigned long id_callback(void);
 #endif /* THREADED_CLIENT */
 #endif /* HAVE_OPENSSL */
+static void CtdlIPC_getline(CtdlIPC* ipc, char *buf);
+static void CtdlIPC_putline(CtdlIPC *ipc, const char *buf);
 
 
 /*
@@ -118,13 +120,23 @@ int CtdlIPCEcho(CtdlIPC *ipc, const char *arg, char *cret)
  */
 int CtdlIPCQuit(CtdlIPC *ipc)
 {
-	register int ret;
+	register int ret = 221;		/* Default to successful quit */
 	char aaa[128];
 
 	CtdlIPC_lock(ipc);
-	CtdlIPC_putline(ipc, "QUIT");
-	CtdlIPC_getline(ipc, aaa);
-	ret = atoi(aaa);
+	if (ipc->sock > -1) {
+		CtdlIPC_putline(ipc, "QUIT");
+		CtdlIPC_getline(ipc, aaa);
+		ret = atoi(aaa);
+	}
+#ifdef HAVE_OPENSSL
+	if (ipc->ssl)
+		SSL_shutdown(ipc->ssl);
+	ipc->ssl = NULL;
+#endif
+	if (ipc->sock)
+		shutdown(ipc->sock, 2);	/* Close connection; we're dead */
+	ipc->sock = -1;
 	CtdlIPC_unlock(ipc);
 	return ret;
 }
@@ -482,23 +494,23 @@ int CtdlIPCGetSingleMessage(CtdlIPC *ipc, long msgnum, int headers, int as_mime,
 				if (!strncasecmp(aaa, "nhdr=yes", 8))
 					mret[0]->nhdr = 1;
 				else if (!strncasecmp(aaa, "from=", 5))
-					strcpy(mret[0]->author, &aaa[5]);
+					safestrncpy(mret[0]->author, &aaa[5], SIZ);
 				else if (!strncasecmp(aaa, "type=", 5))
 					mret[0]->type = atoi(&aaa[5]);
 				else if (!strncasecmp(aaa, "msgn=", 5))
-					strcpy(mret[0]->msgid, &aaa[5]);
+					safestrncpy(mret[0]->msgid, &aaa[5], SIZ);
 				else if (!strncasecmp(aaa, "subj=", 5))
-					strcpy(mret[0]->subject, &aaa[5]);
+					safestrncpy(mret[0]->subject, &aaa[5], SIZ);
 				else if (!strncasecmp(aaa, "rfca=", 5))
-					strcpy(mret[0]->email, &aaa[5]);
+					safestrncpy(mret[0]->email, &aaa[5], SIZ);
 				else if (!strncasecmp(aaa, "hnod=", 5))
-					strcpy(mret[0]->hnod, &aaa[5]);
+					safestrncpy(mret[0]->hnod, &aaa[5], SIZ);
 				else if (!strncasecmp(aaa, "room=", 5))
-					strcpy(mret[0]->room, &aaa[5]);
+					safestrncpy(mret[0]->room, &aaa[5], SIZ);
 				else if (!strncasecmp(aaa, "node=", 5))
-					strcpy(mret[0]->node, &aaa[5]);
+					safestrncpy(mret[0]->node, &aaa[5], SIZ);
 				else if (!strncasecmp(aaa, "rcpt=", 5))
-					strcpy(mret[0]->recipient, &aaa[5]);
+					safestrncpy(mret[0]->recipient, &aaa[5], SIZ);
 				else if (!strncasecmp(aaa, "time=", 5))
 					mret[0]->time = atol(&aaa[5]);
 
@@ -604,7 +616,7 @@ int CtdlIPCWhoKnowsRoom(CtdlIPC *ipc, char **listing, char *cret)
 
 
 /* INFO */
-int CtdlIPCServerInfo(CtdlIPC *ipc, struct CtdlServInfo *ServInfo, char *cret)
+int CtdlIPCServerInfo(CtdlIPC *ipc, char *cret)
 {
 	register int ret;
 	size_t bytes;
@@ -612,7 +624,6 @@ int CtdlIPCServerInfo(CtdlIPC *ipc, struct CtdlServInfo *ServInfo, char *cret)
 	char buf[SIZ];
 
 	if (!cret) return -2;
-	if (!ServInfo) return -2;
 
 	ret = CtdlIPCGenericCommand(ipc, "INFO", NULL, 0, &listing, &bytes, cret);
 	if (ret / 100 == 1) {
@@ -622,29 +633,31 @@ int CtdlIPCServerInfo(CtdlIPC *ipc, struct CtdlServInfo *ServInfo, char *cret)
 			extract_token(buf, listing, 0, '\n');
 			remove_token(listing, 0, '\n');
 			switch (line++) {
-			case 0:		ServInfo->serv_pid = atoi(buf);
+			case 0:		ipc->ServInfo.pid = atoi(buf);
 					break;
-			case 1:		strcpy(ServInfo->serv_nodename,buf);
+			case 1:		strcpy(ipc->ServInfo.nodename,buf);
 					break;
-			case 2:		strcpy(ServInfo->serv_humannode,buf);
+			case 2:		strcpy(ipc->ServInfo.humannode,buf);
 					break;
-			case 3:		strcpy(ServInfo->serv_fqdn,buf);
+			case 3:		strcpy(ipc->ServInfo.fqdn,buf);
 					break;
-			case 4:		strcpy(ServInfo->serv_software,buf);
+			case 4:		strcpy(ipc->ServInfo.software,buf);
 					break;
-			case 5:		ServInfo->serv_rev_level = atoi(buf);
+			case 5:		ipc->ServInfo.rev_level = atoi(buf);
 					break;
-			case 6:		strcpy(ServInfo->serv_bbs_city,buf);
+			case 6:		strcpy(ipc->ServInfo.bbs_city,buf);
 					break;
-			case 7:		strcpy(ServInfo->serv_sysadm,buf);
+			case 7:		strcpy(ipc->ServInfo.sysadm,buf);
 					break;
-			case 9:		strcpy(ServInfo->serv_moreprompt,buf);
+			case 9:		strcpy(ipc->ServInfo.moreprompt,buf);
 					break;
-			case 10:	ServInfo->serv_ok_floors = atoi(buf);
+			case 10:	ipc->ServInfo.ok_floors = atoi(buf);
 					break;
-			case 11:	ServInfo->serv_paging_level = atoi(buf);
+			case 11:	ipc->ServInfo.paging_level = atoi(buf);
 					break;
-			case 13:	ServInfo->serv_supports_qnop = atoi(buf);
+			case 13:	ipc->ServInfo.supports_qnop = atoi(buf);
+					break;
+			case 14:	ipc->ServInfo.supports_ldap = atoi(buf);
 					break;
 			}
 		}
@@ -1128,7 +1141,8 @@ int CtdlIPCOnlineUsers(CtdlIPC *ipc, char **listing, time_t *stamp, char *cret)
 /* OPEN */
 int CtdlIPCFileDownload(CtdlIPC *ipc, const char *filename, void **buf,
 		size_t resume,
-		void (*progress_gauge_callback)(unsigned long, unsigned long),
+		void (*progress_gauge_callback)
+			(CtdlIPC*, unsigned long, unsigned long),
 		char *cret)
 {
 	register int ret;
@@ -1174,7 +1188,8 @@ int CtdlIPCFileDownload(CtdlIPC *ipc, const char *filename, void **buf,
 /* OPNA */
 int CtdlIPCAttachmentDownload(CtdlIPC *ipc, long msgnum, const char *part,
 		void **buf,
-		void (*progress_gauge_callback)(unsigned long, unsigned long),
+		void (*progress_gauge_callback)
+			(CtdlIPC*, unsigned long, unsigned long),
 		char *cret)
 {
 	register int ret;
@@ -1212,7 +1227,8 @@ int CtdlIPCAttachmentDownload(CtdlIPC *ipc, long msgnum, const char *part,
 
 /* OIMG */
 int CtdlIPCImageDownload(CtdlIPC *ipc, const char *filename, void **buf,
-		void (*progress_gauge_callback)(unsigned long, unsigned long),
+		void (*progress_gauge_callback)
+			(CtdlIPC*, unsigned long, unsigned long),
 		char *cret)
 {
 	register int ret;
@@ -1252,7 +1268,8 @@ int CtdlIPCImageDownload(CtdlIPC *ipc, const char *filename, void **buf,
 /* UOPN */
 int CtdlIPCFileUpload(CtdlIPC *ipc, const char *save_as, const char *comment,
 		const char *path,
-		void (*progress_gauge_callback)(unsigned long, unsigned long),
+		void (*progress_gauge_callback)
+			(CtdlIPC*, unsigned long, unsigned long),
 		char *cret)
 {
 	register int ret;
@@ -1284,7 +1301,8 @@ int CtdlIPCFileUpload(CtdlIPC *ipc, const char *save_as, const char *comment,
 /* UIMG */
 int CtdlIPCImageUpload(CtdlIPC *ipc, int for_real, const char *path,
 		const char *save_as,
-		void (*progress_gauge_callback)(unsigned long, unsigned long),
+		void (*progress_gauge_callback)
+			(CtdlIPC*, unsigned long, unsigned long),
 		char *cret)
 {
 	register int ret;
@@ -1712,12 +1730,12 @@ int CtdlIPCAideSetUserParameters(CtdlIPC *ipc, const struct ctdluser *uret, char
 
 
 /* GPEX */
-/* which is 0 = room, 1 = floor, 2 = site */
+/* which is 0 = room, 1 = floor, 2 = site, 3 = default for mailboxes */
 /* caller must free the struct ExpirePolicy */
 int CtdlIPCGetMessageExpirationPolicy(CtdlIPC *ipc, int which,
 		struct ExpirePolicy **policy, char *cret)
 {
-	static char *proto[] = {"room", "floor", "site"};
+	static char *proto[] = {"room", "floor", "site", "mailboxes" };
 	char aaa[11];
 	register int ret;
 
@@ -1725,7 +1743,7 @@ int CtdlIPCGetMessageExpirationPolicy(CtdlIPC *ipc, int which,
 	if (!policy) return -2;
 	if (!*policy) *policy = (struct ExpirePolicy *)calloc(1, sizeof(struct ExpirePolicy));
 	if (!*policy) return -1;
-	if (which < 0 || which > 2) return -2;
+	if (which < 0 || which > 3) return -2;
 	
 	sprintf(aaa, "GPEX %s", proto[which]);
 	ret = CtdlIPCGenericCommand(ipc, aaa, NULL, 0, NULL, NULL, cret);
@@ -1739,16 +1757,16 @@ int CtdlIPCGetMessageExpirationPolicy(CtdlIPC *ipc, int which,
 
 
 /* SPEX */
-/* which is 0 = room, 1 = floor, 2 = site */
+/* which is 0 = room, 1 = floor, 2 = site, 3 = default for mailboxes */
 /* policy is 0 = inherit, 1 = no purge, 2 = by count, 3 = by age (days) */
 int CtdlIPCSetMessageExpirationPolicy(CtdlIPC *ipc, int which,
 		struct ExpirePolicy *policy, char *cret)
 {
 	char aaa[38];
-	char *whichvals[] = { "room", "floor", "site" };
+	char *whichvals[] = { "room", "floor", "site", "mailboxes" };
 
 	if (!cret) return -2;
-	if (which < 0 || which > 2) return -2;
+	if (which < 0 || which > 3) return -2;
 	if (!policy) return -2;
 	if (policy->expire_mode < 0 || policy->expire_mode > 3) return -2;
 	if (policy->expire_mode >= 2 && policy->expire_value < 1) return -2;
@@ -1822,6 +1840,31 @@ int CtdlIPCSetSystemConfigByType(CtdlIPC *ipc, const char *mimetype,
 }
 
 
+/* GNET */
+int CtdlIPCGetRoomNetworkConfig(CtdlIPC *ipc, char **listing, char *cret)
+{
+	size_t bytes;
+
+	if (!cret) return -2;
+	if (!listing) return -2;
+	if (*listing) return -2;
+
+	return CtdlIPCGenericCommand(ipc, "GNET", NULL, 0,
+			listing, &bytes, cret);
+}
+
+
+/* SNET */
+int CtdlIPCSetRoomNetworkConfig(CtdlIPC *ipc, const char *listing, char *cret)
+{
+	if (!cret) return -2;
+	if (!listing) return -2;
+
+	return CtdlIPCGenericCommand(ipc, "SNET", listing, strlen(listing),
+			NULL, NULL, cret);
+}
+
+
 /* REQT */
 int CtdlIPCRequestClientLogout(CtdlIPC *ipc, int session, char *cret)
 {
@@ -2138,7 +2181,8 @@ int CtdlIPCSpecifyPreferredFormats(CtdlIPC *ipc, char *cret, char *formats) {
 
 /* READ */
 int CtdlIPCReadDownload(CtdlIPC *ipc, void **buf, size_t bytes, size_t resume,
-	       void (*progress_gauge_callback)(unsigned long, unsigned long),
+		void (*progress_gauge_callback)
+			(CtdlIPC*, unsigned long, unsigned long),
 	       char *cret)
 {
 	register size_t len;
@@ -2150,7 +2194,7 @@ int CtdlIPCReadDownload(CtdlIPC *ipc, void **buf, size_t bytes, size_t resume,
 
 	len = resume;
 	if (progress_gauge_callback)
-		progress_gauge_callback(len, bytes);
+		progress_gauge_callback(ipc, len, bytes);
 	while (len < bytes) {
 		register size_t block;
 
@@ -2161,7 +2205,7 @@ int CtdlIPCReadDownload(CtdlIPC *ipc, void **buf, size_t bytes, size_t resume,
 		}
 		len += block;
 		if (progress_gauge_callback)
-			progress_gauge_callback(len, bytes);
+			progress_gauge_callback(ipc, len, bytes);
 	}
 	return len;
 }
@@ -2169,7 +2213,8 @@ int CtdlIPCReadDownload(CtdlIPC *ipc, void **buf, size_t bytes, size_t resume,
 /* READ - pipelined */
 int CtdlIPCHighSpeedReadDownload(CtdlIPC *ipc, void **buf, size_t bytes,
 	       size_t resume,
-	       void (*progress_gauge_callback)(unsigned long, unsigned long),
+		void (*progress_gauge_callback)
+			(CtdlIPC*, unsigned long, unsigned long),
 	       char *cret)
 {
 	register size_t len;
@@ -2188,7 +2233,7 @@ int CtdlIPCHighSpeedReadDownload(CtdlIPC *ipc, void **buf, size_t bytes,
 	len = 0;
 	CtdlIPC_lock(ipc);
 	if (progress_gauge_callback)
-		progress_gauge_callback(len, bytes);
+		progress_gauge_callback(ipc, len, bytes);
 
 	/* How many calls will be in the pipeline? */
 	calls = (bytes - resume) / 4096;
@@ -2211,7 +2256,7 @@ int CtdlIPCHighSpeedReadDownload(CtdlIPC *ipc, void **buf, size_t bytes,
 			serv_read(ipc, ((*buf) + (i * 4096)), len);
 		}
 		if (progress_gauge_callback)
-			progress_gauge_callback(i * 4096 + len, bytes);
+			progress_gauge_callback(ipc, i * 4096 + len, bytes);
 	}
 	CtdlIPC_unlock(ipc);
 	return len;
@@ -2236,7 +2281,8 @@ int CtdlIPCEndUpload(CtdlIPC *ipc, int discard, char *cret)
 
 /* WRIT */
 int CtdlIPCWriteUpload(CtdlIPC *ipc, const char *path,
-		void (*progress_gauge_callback)(unsigned long, unsigned long),
+		void (*progress_gauge_callback)
+			(CtdlIPC*, unsigned long, unsigned long),
 		char *cret)
 {
 	register int ret = -1;
@@ -2258,7 +2304,7 @@ int CtdlIPCWriteUpload(CtdlIPC *ipc, const char *path,
 	rewind(fd);
 
 	if (progress_gauge_callback)
-		progress_gauge_callback(0, bytes);
+		progress_gauge_callback(ipc, 0, bytes);
 
 	while (offset < bytes) {
 		register size_t to_write;
@@ -2279,7 +2325,7 @@ int CtdlIPCWriteUpload(CtdlIPC *ipc, const char *path,
 			serv_write(ipc, buf, to_write);
 			offset += to_write;
 			if (progress_gauge_callback)
-				progress_gauge_callback(offset, bytes);
+				progress_gauge_callback(ipc, offset, bytes);
 			/* Detect short reads and back up if needed */
 			/* offset will never be negative anyway */
 			fseek(fd, (signed)offset, SEEK_SET);
@@ -2288,7 +2334,7 @@ int CtdlIPCWriteUpload(CtdlIPC *ipc, const char *path,
 		}
 	}
 	if (progress_gauge_callback)
-		progress_gauge_callback(1, 1);
+		progress_gauge_callback(ipc, 1, 1);
 	return (!ferror(fd) ? ret : -2);
 }
 
@@ -2537,7 +2583,7 @@ static void serv_read(CtdlIPC *ipc, char *buf, unsigned int bytes)
 	while (len < bytes) {
 		rlen = read(ipc->sock, &buf[len], bytes - len);
 		if (rlen < 1) {
-			connection_died(ipc);
+			connection_died(ipc, 0);
 			return;
 		}
 		len += rlen;
@@ -2563,7 +2609,7 @@ static void serv_write(CtdlIPC *ipc, const char *buf, unsigned int nbytes)
 		retval = write(ipc->sock, &buf[bytes_written],
 			       nbytes - bytes_written);
 		if (retval < 1) {
-			connection_died(ipc);
+			connection_died(ipc, 0);
 			return;
 		}
 		bytes_written += retval;
@@ -2603,9 +2649,9 @@ static void serv_read_ssl(CtdlIPC* ipc, char *buf, unsigned int bytes)
 				serv_read(ipc, &buf[len], bytes - len);
 				return;
 			}
-			error_printf("SSL_read in serv_read:\n");
-			ERR_print_errors_fp(stderr);
-			connection_died(ipc);
+			error_printf("SSL_read in serv_read: %s\n",
+					ERR_reason_error_string(ERR_peek_error()));
+			connection_died(ipc, 1);
 			return;
 		}
 		len += rlen;
@@ -2646,9 +2692,9 @@ static void serv_write_ssl(CtdlIPC *ipc, const char *buf, unsigned int nbytes)
 						nbytes - bytes_written);
 				return;
 			}
-			error_printf("SSL_write in serv_write:\n");
-			ERR_print_errors_fp(stderr);
-			connection_died(ipc);
+			error_printf("SSL_write in serv_write: %s\n",
+					ERR_reason_error_string(ERR_peek_error()));
+			connection_died(ipc, 1);
 			return;
 		}
 		bytes_written += retval;
@@ -2756,7 +2802,7 @@ static unsigned long id_callback(void) {
 /*
  * input string from socket - implemented in terms of serv_read()
  */
-void CtdlIPC_getline(CtdlIPC* ipc, char *buf)
+static void CtdlIPC_getline(CtdlIPC* ipc, char *buf)
 {
 	int i;
 
@@ -2777,11 +2823,15 @@ void CtdlIPC_getline(CtdlIPC* ipc, char *buf)
 	if (buf[i] == 13) buf[i--] = 0;
 }
 
+void CtdlIPC_chat_recv(CtdlIPC* ipc, char* buf)
+{
+	return CtdlIPC_getline(ipc, buf);
+}
 
 /*
  * send line to server - implemented in terms of serv_write()
  */
-void CtdlIPC_putline(CtdlIPC *ipc, const char *buf)
+static void CtdlIPC_putline(CtdlIPC *ipc, const char *buf)
 {
 	/* error_printf("< %s\n", buf); */
 	serv_write(ipc, buf, strlen(buf));
@@ -2790,6 +2840,11 @@ void CtdlIPC_putline(CtdlIPC *ipc, const char *buf)
 	ipc->last_command_sent = time(NULL);
 }
 
+void CtdlIPC_chat_send(CtdlIPC* ipc, const char* buf)
+{
+	return CtdlIPC_putline(ipc, buf);
+}
+
 
 /*
  * attach to server
@@ -2800,8 +2855,9 @@ CtdlIPC* CtdlIPC_new(int argc, char **argv, char *hostbuf, char *portbuf)
 	char cithost[SIZ];
 	char citport[SIZ];
 	char sockpath[SIZ];
+	CtdlIPC* ipc;
 
-	CtdlIPC *ipc = ialloc(CtdlIPC);
+	ipc = ialloc(CtdlIPC);
 	if (!ipc) {
 		return 0;
 	}
@@ -2880,6 +2936,38 @@ CtdlIPC* CtdlIPC_new(int argc, char **argv, char *hostbuf, char *portbuf)
 	return ipc;
 }
 
+
+/*
+ * Disconnect and delete the IPC class (destructor)
+ */
+void CtdlIPC_delete(CtdlIPC* ipc)
+{
+#ifdef HAVE_OPENSSL
+	if (ipc->ssl) {
+		SSL_shutdown(ipc->ssl);
+		SSL_free(ipc->ssl);
+		ipc->ssl = NULL;
+	}
+#endif
+	if (ipc->sock > -1) {
+		shutdown(ipc->sock, 2);	/* Close it up */
+		ipc->sock = -1;
+	}
+	ifree(ipc);
+}
+
+
+/*
+ * Disconnect and delete the IPC class (destructor)
+ * Also NULLs out the pointer
+ */
+void CtdlIPC_delete_ptr(CtdlIPC** pipc)
+{
+	CtdlIPC_delete(*pipc);
+	*pipc = NULL;
+}
+
+
 /*
  * return the file descriptor of the server socket so we can select() on it.
  *
