@@ -1,5 +1,5 @@
 /* 
- * $Id: citserver.c,v 613.0 2003/12/15 16:37:02 ajc Exp $
+ * $Id: citserver.c,v 620.0 2004/03/27 02:33:30 ajc Exp $
  *
  * Main source module for the Citadel server
  *
@@ -34,7 +34,6 @@
 #include <string.h>
 #include <errno.h>
 #include <limits.h>
-#include <syslog.h>
 /* #include <dlfcn.h> */
 #include <netdb.h>
 #include <sys/types.h>
@@ -78,10 +77,10 @@ void master_startup(void) {
 	FILE *urandom;
 	struct ctdlroom qrbuf;
 	
-	lprintf(9, "master_startup() started\n");
+	lprintf(CTDL_DEBUG, "master_startup() started\n");
 	time(&server_startup_time);
 
-	lprintf(7, "Opening databases\n");
+	lprintf(CTDL_INFO, "Opening databases\n");
 	open_databases();
 
 	if (do_defrag) {
@@ -90,7 +89,7 @@ void master_startup(void) {
 
 	check_ref_counts();
 
-	lprintf(7, "Creating base rooms (if necessary)\n");
+	lprintf(CTDL_INFO, "Creating base rooms (if necessary)\n");
 	create_room(BASEROOM,		0, "", 0, 1, 0);
 	create_room(AIDEROOM,		3, "", 0, 1, 0);
 	create_room(SYSCONFIGROOM,	3, "", 0, 1, 0);
@@ -102,7 +101,7 @@ void master_startup(void) {
                 lputroom(&qrbuf);
         }
 
-	lprintf(7, "Seeding the pseudo-random number generator...\n");
+	lprintf(CTDL_INFO, "Seeding the pseudo-random number generator...\n");
 	urandom = fopen("/dev/urandom", "r");
 	if (urandom != NULL) {
 		fread(&seed, sizeof seed, 1, urandom);
@@ -114,12 +113,12 @@ void master_startup(void) {
 	}
 	srandom(seed);
 
-	lprintf(7, "Initializing ipgm secret\n");
+	lprintf(CTDL_INFO, "Initializing ipgm secret\n");
 	get_config();
 	config.c_ipgm_secret = rand();
 	put_config();
 
-	lprintf(9, "master_startup() finished\n");
+	lprintf(CTDL_DEBUG, "master_startup() finished\n");
 }
 
 
@@ -137,14 +136,18 @@ void master_cleanup(void) {
 	}
 
 	/* Close databases */
-	lprintf(7, "Closing databases\n");
+	lprintf(CTDL_INFO, "Closing databases\n");
 	close_databases();
 
 	/* Do system-dependent stuff */
 	sysdep_master_cleanup();
 
+#ifdef DEBUG_MEMORY_LEAKS
+	dump_heap();
+#endif
+
 	/* Now go away. */
-	lprintf(3, "citserver: exiting.\n");
+	lprintf(CTDL_NOTICE, "citserver: exiting.\n");
 	fflush(stdout); fflush(stderr);
 	exit(0);
 }
@@ -159,12 +162,12 @@ void deallocate_user_data(struct CitContext *con)
 
 	begin_critical_section(S_SESSION_TABLE);
 	while (con->FirstSessData != NULL) {
-		lprintf(9, "Deallocating user data symbol %ld\n",
+		lprintf(CTDL_DEBUG, "Deallocating user data symbol %ld\n",
 			con->FirstSessData->sym_id);
 		if (con->FirstSessData->sym_data != NULL)
-			phree(con->FirstSessData->sym_data);
+			free(con->FirstSessData->sym_data);
 		ptr = con->FirstSessData->next;
-		phree(con->FirstSessData);
+		free(con->FirstSessData);
 		con->FirstSessData = ptr;
 	}
 	end_critical_section(S_SESSION_TABLE);
@@ -182,16 +185,16 @@ void RemoveContext (struct CitContext *con)
 	struct CitContext *ToFree = NULL;
 
 	if (con==NULL) {
-		lprintf(5, "WARNING: RemoveContext() called with NULL!\n");
+		lprintf(CTDL_ERR, "WARNING: RemoveContext() called with NULL!\n");
 		return;
 	}
-	lprintf(9, "RemoveContext() called\n");
+	lprintf(CTDL_DEBUG, "RemoveContext() called\n");
 
 	/* Remove the context from the global context list.  This needs
 	 * to get done FIRST to avoid concurrency problems.  It is *vitally*
 	 * important to keep num_sessions accurate!!
 	 */
-	lprintf(7, "Removing context for session %d\n", con->cs_pid);
+	lprintf(CTDL_DEBUG, "Removing context for session %d\n", con->cs_pid);
 	begin_critical_section(S_SESSION_TABLE);
 	if (ContextList == con) {
 		ToFree = ContextList;
@@ -210,7 +213,7 @@ void RemoveContext (struct CitContext *con)
 	end_critical_section(S_SESSION_TABLE);
 
 	if (ToFree == NULL) {
-		lprintf(9, "RemoveContext() found nothing to remove\n");
+		lprintf(CTDL_DEBUG, "RemoveContext() found nothing to remove\n");
 		return;
 	}
 
@@ -225,26 +228,25 @@ void RemoveContext (struct CitContext *con)
 	become_session(NULL);
 
 	/* Now handle all of the administrivia. */
-	lprintf(7, "Calling logout(%d)\n", con->cs_pid);
+	lprintf(CTDL_DEBUG, "Calling logout(%d)\n", con->cs_pid);
 	logout(con);
 
 	unlink(con->temp);
-	lprintf(3, "Session ended.\n");
-	syslog(LOG_NOTICE,"Session %d: ended.", con->cs_pid);
+	lprintf(CTDL_NOTICE, "Session %d: ended.\n", con->cs_pid);
 
 	/* Deallocate any user-data attached to this session */
 	deallocate_user_data(con);
 
 	/* If the client is still connected, blow 'em away. */
-	lprintf(7, "Closing socket %d\n", con->client_socket);
+	lprintf(CTDL_DEBUG, "Closing socket %d\n", con->client_socket);
 	close(con->client_socket);
 
 	/* This is where we used to check for scheduled shutdowns. */
 
 	/* Free up the memory used by this context */
-	phree(con);
+	free(con);
 
-	lprintf(9, "Done with RemoveContext()\n");
+	lprintf(CTDL_DEBUG, "Done with RemoveContext()\n");
 }
 
 
@@ -266,7 +268,7 @@ void *CtdlGetUserData(unsigned long requested_sym)
 		if (ptr->sym_id == requested_sym)
 			return(ptr->sym_data);
 
-	lprintf(2, "ERROR! CtdlGetUserData(%ld) symbol not allocated\n",
+	lprintf(CTDL_ERR, "ERROR! CtdlGetUserData(%ld) symbol not allocated\n",
 		requested_sym);
 	return NULL;
 }
@@ -279,7 +281,7 @@ void CtdlAllocUserData(unsigned long requested_sym, size_t num_bytes)
 {
 	struct CtdlSessData *ptr;
 
-	lprintf(9, "CtdlAllocUserData(%ld) called\n", requested_sym);
+	lprintf(CTDL_DEBUG, "CtdlAllocUserData(%ld) called\n", requested_sym);
 
 	/* Fail silently if the symbol is already registered. */
 	for (ptr = CC->FirstSessData; ptr != NULL; ptr = ptr->next)  {
@@ -289,9 +291,9 @@ void CtdlAllocUserData(unsigned long requested_sym, size_t num_bytes)
 	}
 
 	/* Grab us some memory!  Dem's good eatin' !!  */
-	ptr = mallok(sizeof(struct CtdlSessData));
+	ptr = malloc(sizeof(struct CtdlSessData));
 	ptr->sym_id = requested_sym;
-	ptr->sym_data = mallok(num_bytes);
+	ptr->sym_data = malloc(num_bytes);
 	memset(ptr->sym_data, 0, num_bytes);
 
 	begin_critical_section(S_SESSION_TABLE);
@@ -299,7 +301,7 @@ void CtdlAllocUserData(unsigned long requested_sym, size_t num_bytes)
 	CC->FirstSessData = ptr;
 	end_critical_section(S_SESSION_TABLE);
 
-	lprintf(9, "CtdlAllocUserData(%ld) finished\n", requested_sym);
+	lprintf(CTDL_DEBUG, "CtdlAllocUserData(%ld) finished\n", requested_sym);
 }
 
 
@@ -312,12 +314,12 @@ void CtdlReallocUserData(unsigned long requested_sym, size_t num_bytes)
 
 	for (ptr = CC->FirstSessData; ptr != NULL; ptr = ptr->next)  {
 		if (ptr->sym_id == requested_sym) {
-			ptr->sym_data = reallok(ptr->sym_data, num_bytes);
+			ptr->sym_data = realloc(ptr->sym_data, num_bytes);
 			return;
 		}
 	}
 
-	lprintf(2, "CtdlReallocUserData() ERROR: symbol %ld not found!\n",
+	lprintf(CTDL_ERR, "CtdlReallocUserData() ERROR: symbol %ld not found!\n",
 		requested_sym);
 }
 
@@ -345,6 +347,11 @@ void cmd_info(void) {
 	cprintf("1\n"); /* 1 = we support the extended paging options */
 	cprintf("%s\n", CC->cs_nonce);
 	cprintf("1\n"); /* 1 = yes, this system supports the QNOP command */
+#ifdef HAVE_LDAP
+	cprintf("1\n"); /* 1 = yes, this server is LDAP-enabled */
+#else
+	cprintf("0\n"); /* 1 = no, this server is not LDAP-enabled */
+#endif
 	cprintf("000\n");
 }
 
@@ -405,13 +412,13 @@ int is_public_client(void)
 	 */
 	if (stat(PUBLIC_CLIENTS, &statbuf) != 0) {
 		/* No public_clients file exists, so bail out */
-		lprintf(5, "Warning: '%s' does not exist\n", PUBLIC_CLIENTS);
+		lprintf(CTDL_WARNING, "Warning: '%s' does not exist\n", PUBLIC_CLIENTS);
 		return(0);
 	}
 
 	if (statbuf.st_mtime > pc_timestamp) {
 		begin_critical_section(S_PUBLIC_CLIENTS);
-		lprintf(7, "Loading %s\n", PUBLIC_CLIENTS);
+		lprintf(CTDL_INFO, "Loading %s\n", PUBLIC_CLIENTS);
 
 		strcpy(public_clients, "127.0.0.1");
 		if (hostname_to_dotted_quad(addrbuf, config.c_fqdn) == 0) {
@@ -440,18 +447,18 @@ int is_public_client(void)
 		end_critical_section(S_PUBLIC_CLIENTS);
 	}
 
-	lprintf(9, "Checking whether %s is a local or public client\n",
+	lprintf(CTDL_DEBUG, "Checking whether %s is a local or public client\n",
 		CC->cs_addr);
 	for (i=0; i<num_parms(public_clients); ++i) {
 		extract(addrbuf, public_clients, i);
 		if (!strcasecmp(CC->cs_addr, addrbuf)) {
-			lprintf(9, "... yes it is.\n");
+			lprintf(CTDL_DEBUG, "... yes it is.\n");
 			return(1);
 		}
 	}
 
 	/* No hits.  This is not a public client. */
-	lprintf(9, "... no it isn't.\n");
+	lprintf(CTDL_DEBUG, "... no it isn't.\n");
 	return(0);
 }
 
@@ -470,7 +477,7 @@ void cmd_iden(char *argbuf)
 	int do_lookup = 0;
 
 	if (num_parms(argbuf)<4) {
-		cprintf("%d usage error\n",ERROR);
+		cprintf("%d usage error\n", ERROR + ILLEGAL_VALUE);
 		return;
 	}
 
@@ -495,7 +502,7 @@ void cmd_iden(char *argbuf)
 	}
 
 	if (do_lookup) {
-		lprintf(9, "Looking up hostname '%s'\n", from_host);
+		lprintf(CTDL_DEBUG, "Looking up hostname '%s'\n", from_host);
 		if ((addr.s_addr = inet_addr(from_host)) != -1) {
 			locate_host(CC->cs_host, sizeof CC->cs_host,
 				NULL, 0,
@@ -507,15 +514,7 @@ void cmd_iden(char *argbuf)
 		}
 	}
 
-	lprintf(7, "Client %d/%d/%01d.%02d (%s)\n",
-		dev_code,
-		cli_code,
-		(rev_level / 100),
-		(rev_level % 100),
-		desc);
-
-	syslog(LOG_NOTICE,"Session %d: Client %d/%d/%01d.%02d (%s) from %s\n",
-		CC->cs_pid,
+	lprintf(CTDL_NOTICE, "Client %d/%d/%01d.%02d (%s) from %s\n",
 		dev_code,
 		cli_code,
 		(rev_level / 100),
@@ -539,8 +538,8 @@ void cmd_mesg(char *mname)
 
 	extract(buf,mname,0);
 
-	dirs[0]=mallok(64);
-	dirs[1]=mallok(64);
+	dirs[0]=malloc(64);
+	dirs[1]=malloc(64);
 	strcpy(dirs[0],"messages");
 	strcpy(dirs[1],"help");
 	snprintf(buf2, sizeof buf2, "%s.%d.%d", buf, CC->cs_clientdev, CC->cs_clienttyp);
@@ -552,18 +551,18 @@ void cmd_mesg(char *mname)
 			mesg_locate(targ,sizeof targ,buf,2,(const char **)dirs);
 		}	
 	}
-	phree(dirs[0]);
-	phree(dirs[1]);
+	free(dirs[0]);
+	free(dirs[1]);
 
 	if (strlen(targ)==0) {
-		cprintf("%d '%s' not found.\n",ERROR,mname);
+		cprintf("%d '%s' not found.\n",ERROR + FILE_NOT_FOUND, mname);
 		return;
 	}
 
 	mfp = fopen(targ,"r");
 	if (mfp==NULL) {
 		cprintf("%d Cannot open '%s': %s\n",
-			ERROR,targ,strerror(errno));
+			ERROR + INTERNAL_ERROR, targ, strerror(errno));
 		return;
 	}
 	cprintf("%d %s\n",LISTING_FOLLOWS,buf);
@@ -597,13 +596,13 @@ void cmd_emsg(char *mname)
 		if (buf[a] == '/') buf[a] = '.';
 	}
 
-	dirs[0]=mallok(64);
-	dirs[1]=mallok(64);
+	dirs[0]=malloc(64);
+	dirs[1]=malloc(64);
 	strcpy(dirs[0],"messages");
 	strcpy(dirs[1],"help");
 	mesg_locate(targ,sizeof targ,buf,2,(const char**)dirs);
-	phree(dirs[0]);
-	phree(dirs[1]);
+	free(dirs[0]);
+	free(dirs[1]);
 
 	if (strlen(targ)==0) {
 		snprintf(targ, sizeof targ, "./help/%s", buf);
@@ -612,7 +611,7 @@ void cmd_emsg(char *mname)
 	mfp = fopen(targ,"w");
 	if (mfp==NULL) {
 		cprintf("%d Cannot open '%s': %s\n",
-			ERROR,targ,strerror(errno));
+			ERROR + INTERNAL_ERROR, targ, strerror(errno));
 		return;
 	}
 	cprintf("%d %s\n", SEND_LISTING, targ);
@@ -660,14 +659,14 @@ int CtdlAccessCheck(int required_level) {
 	if (CC->internal_pgm) return(0);
 	if (required_level >= ac_internal) {
 		cprintf("%d This is not a user-level command.\n",
-			ERROR+HIGHER_ACCESS_REQUIRED);
+			ERROR + HIGHER_ACCESS_REQUIRED);
 		return(-1);
 	}
 
 	if (CC->user.axlevel >= 6) return(0);
  	if (required_level >= ac_aide) {
 		cprintf("%d This command requires Aide access.\n",
-			ERROR+HIGHER_ACCESS_REQUIRED);
+			ERROR + HIGHER_ACCESS_REQUIRED);
 		return(-1);
 	}
 
@@ -680,7 +679,7 @@ int CtdlAccessCheck(int required_level) {
 
 	if (CC->logged_in) return(0);
 	if (required_level >= ac_logged_in) {
-		cprintf("%d Not logged in.\n", ERROR+NOT_LOGGED_IN);
+		cprintf("%d Not logged in.\n", ERROR + NOT_LOGGED_IN);
 		return(-1);
 	}
 
@@ -702,11 +701,11 @@ void cmd_term(char *cmdbuf)
 
 	session_num = extract_int(cmdbuf, 0);
 	if (session_num == CC->cs_pid) {
-		cprintf("%d You can't kill your own session.\n", ERROR);
+		cprintf("%d You can't kill your own session.\n", ERROR + ILLEGAL_VALUE);
 		return;
 	}
 
-	lprintf(9, "Locating session to kill\n");
+	lprintf(CTDL_DEBUG, "Locating session to kill\n");
 	begin_critical_section(S_SESSION_TABLE);
 	for (ccptr = ContextList; ccptr != NULL; ccptr = ccptr->next) {
 		if (session_num == ccptr->cs_pid) {
@@ -733,7 +732,7 @@ void cmd_term(char *cmdbuf)
 		}
 	}
 	else {
-		cprintf("%d No such session.\n", ERROR);
+		cprintf("%d No such session.\n", ERROR + ILLEGAL_VALUE);
 	}
 }
 
@@ -772,7 +771,7 @@ void cmd_ipgm(char *argbuf)
 	 */
 	if (!CC->is_local_socket) {
 		sleep(5);
-		cprintf("%d Authentication failed.\n",ERROR);
+		cprintf("%d Authentication failed.\n", ERROR + PASSWORD_REQUIRED);
 	}
 	else if (secret == config.c_ipgm_secret) {
 		CC->internal_pgm = 1;
@@ -782,8 +781,8 @@ void cmd_ipgm(char *argbuf)
 	}
 	else {
 		sleep(5);
-		cprintf("%d Authentication failed.\n",ERROR);
-		lprintf(3, "Warning: ipgm authentication failed.\n");
+		cprintf("%d Authentication failed.\n", ERROR + PASSWORD_REQUIRED);
+		lprintf(CTDL_ERR, "Warning: ipgm authentication failed.\n");
 		CC->kill_me = 1;
 	}
 
@@ -907,8 +906,7 @@ void begin_session(struct CitContext *con)
 	if ((config.c_maxsessions > 0)&&(num_sessions > config.c_maxsessions))
 		con->nologin = 1;
 
-	lprintf(3, "Session started.\n");
-	syslog(LOG_NOTICE, "Session %d: started.", con->cs_pid);
+	lprintf(CTDL_NOTICE, "Session started.\n");
 
 	/* Run any session startup routines registered by loadable modules */
 	PerformSessionHooks(EVT_START);
@@ -919,7 +917,7 @@ void citproto_begin_session() {
 	if (CC->nologin==1) {
 		cprintf("%d %s: Too many users are already online "
 			"(maximum is %d)\n",
-			ERROR+MAX_SESSIONS_EXCEEDED,
+			ERROR + MAX_SESSIONS_EXCEEDED,
 			config.c_nodename, config.c_maxsessions);
 	}
 	else {
@@ -940,11 +938,11 @@ void do_command_loop(void) {
 	time(&CC->lastcmd);
 	memset(cmdbuf, 0, sizeof cmdbuf); /* Clear it, just in case */
 	if (client_gets(cmdbuf) < 1) {
-		lprintf(3, "Client socket is broken.  Ending session.\n");
+		lprintf(CTDL_ERR, "Client socket is broken.  Ending session.\n");
 		CC->kill_me = 1;
 		return;
 	}
-	lprintf(5, "Citadel: %s\n", cmdbuf);
+	lprintf(CTDL_INFO, "Citadel: %s\n", cmdbuf);
 
 	/*
 	 * Let other clients see the last command we executed, and
@@ -1309,14 +1307,8 @@ void do_command_loop(void) {
 		cmd_isme(&cmdbuf[5]);
 	}
 
-#ifdef DEBUG_MEMORY_LEAKS
-	else if (!strncasecmp(cmdbuf, "LEAK", 4)) {
-		dump_tracked();
-	}
-#endif
-
 	else if (!DLoader_Exec_Cmd(cmdbuf)) {
-		cprintf("%d Unrecognized or unsupported command.\n", ERROR);
+		cprintf("%d Unrecognized or unsupported command.\n", ERROR + CMD_NOT_SUPPORTED);
 	       }
 
 	/* Run any after-each-command outines registered by modules */
