@@ -1,5 +1,5 @@
 /*
- * $Id: commands.c,v 613.0 2003/12/15 16:37:02 ajc Exp $
+ * $Id: commands.c,v 620.0 2004/03/27 02:33:31 ajc Exp $
  *
  * This file contains functions which implement parts of the
  * text-mode user interface.
@@ -157,7 +157,7 @@ int checkpagin(int lp, unsigned int pagin, unsigned int height)
 	if (!pagin) return(0);
 	if (lp>=(height-1)) {
 		set_keepalives(KA_HALF);
-		hit_any_key();
+		hit_any_key(ipc_for_signal_handlers);	/* Cheating -IO */
 		set_keepalives(KA_YES);
 		return(0);
 	}
@@ -270,7 +270,7 @@ void print_express(void)
 					fprintf(outpipe, "at %d:%02dam",
 						stamp->tm_hour, stamp->tm_min);
 				fprintf(outpipe, " from %s", sender);
-				if (strncmp(serv_info.serv_nodename, node, 32))
+				if (strncmp(ipc_for_signal_handlers->ServInfo.nodename, node, 32))
 					fprintf(outpipe, " @%s", node);
 				fprintf(outpipe, ":\n%s\n", listing);
 				pclose(outpipe);
@@ -307,7 +307,7 @@ void print_express(void)
 		scr_printf(" from %s", sender);
 	
 		/* Remote node, if any */
-		if (strncmp(serv_info.serv_nodename, node, 32))
+		if (strncmp(ipc_for_signal_handlers->ServInfo.nodename, node, 32))
 			scr_printf(" @%s", node);
 	
 		scr_printf(":\n");
@@ -376,8 +376,8 @@ static void really_do_keepalive(void) {
 	 * server supports it) and then do nothing.
 	 */
 	if ( (keepalives_enabled == KA_HALF)
-	   && (serv_info.serv_supports_qnop > 0) ) {
-		CtdlIPC_putline(ipc_for_signal_handlers, "QNOP");
+	   && (ipc_for_signal_handlers->ServInfo.supports_qnop > 0) ) {
+		CtdlIPC_chat_send(ipc_for_signal_handlers, "QNOP");
 	}
 }
 
@@ -478,9 +478,9 @@ int inkey(void)
 		 * necessary and then waits again.
 		 */
 		do {
-			scr_set_windowsize();
+			scr_set_windowsize(ipc_for_signal_handlers);
 			do_keepalive();
-			scr_set_windowsize();
+			scr_set_windowsize(ipc_for_signal_handlers);
 
 			FD_ZERO(&rfds);
 			FD_SET(0, &rfds);
@@ -493,7 +493,7 @@ int inkey(void)
 		/* At this point, there's input, so fetch it.
 		 * (There's a hole in the bucket...)
 		 */
-		a = scr_getc(SCR_NOBLOCK);
+		a = scr_getc(SCR_BLOCK);
 		if (a == 127)
 			a = 8;
 		if (a > 126)
@@ -507,7 +507,7 @@ int inkey(void)
 #ifndef DISABLE_CURSES
 #if defined(HAVE_CURSES_H) || defined(HAVE_NCURSES_H)
 		if (a == ERR)
-			a = 0;
+			logoff(NULL, 3);
 #endif
 #endif
 
@@ -736,6 +736,7 @@ void load_command_set(void)
 	for (i = 0; i < MAX_EDITORS; i++)
 		strcpy(editor_paths[i], "");
 	strcpy(printcmd, "");
+	strcpy(imagecmd, "");
 	strcpy(rc_username, "");
 	strcpy(rc_password, "");
 	rc_floor_mode = 0;
@@ -824,6 +825,9 @@ void load_command_set(void)
 		if (!strncasecmp(buf, "printcmd=", 9))
 			strcpy(printcmd, &buf[9]);
 
+		if (!strncasecmp(buf, "imagecmd=", 9))
+			strcpy(imagecmd, &buf[9]);
+
 		if (!strncasecmp(buf, "expcmd=", 7))
 			strcpy(rc_exp_cmd, &buf[7]);
 
@@ -845,7 +849,7 @@ void load_command_set(void)
 			rc_allow_attachments = atoi(&buf[18]);
 		}
 		if (!strncasecmp(buf, "idle_threshold=", 15)) {
-			rc_idle_threshold = atoi(&buf[15]);
+			rc_idle_threshold = atol(&buf[15]);
 		}
 		if (!strncasecmp(buf, "remember_passwords=", 19)) {
 			rc_remember_passwords = atoi(&buf[19]);
@@ -1627,7 +1631,7 @@ char keymenu(char *menuprompt, char *menustring) {
 					scr_printf(" ");
 				}
 			}
-			scr_printf(" -> ");
+			scr_printf("-> ");
 			display_prompt = 0;
 		}
 		ch = lkey();
@@ -1656,7 +1660,7 @@ char keymenu(char *menuprompt, char *menustring) {
 							scr_putc(buf[a]);
 						}
 					}
-					scr_printf("\n\n");
+					scr_printf("\n");
 					return ch;
 				}
 			}
