@@ -1,5 +1,5 @@
 /*
- * $Id: imap_fetch.c,v 613.0 2003/12/15 16:37:02 ajc Exp $
+ * $Id: imap_fetch.c,v 620.0 2004/03/27 02:33:32 ajc Exp $
  *
  * Implements the FETCH command in IMAP.
  * This command is way too convoluted.  Marc Crispin is a fscking idiot.
@@ -63,8 +63,6 @@ struct imap_fetch_part {
  * Individual field functions for imap_do_fetch_msg() ...
  */
 
-
-
 void imap_fetch_uid(int seq) {
 	cprintf("UID %ld", IMAP->msgids[seq-1]);
 }
@@ -115,31 +113,51 @@ void imap_fetch_internaldate(struct CtdlMessage *msg) {
  *	"RFC822.SIZE"	size of translated message
  *	"RFC822.TEXT"	body only (without leading blank line)
  */
-void imap_fetch_rfc822(int msgnum, char *whichfmt, struct CtdlMessage *msg) {
+void imap_fetch_rfc822(long msgnum, char *whichfmt) {
 	char buf[1024];
 	char *ptr;
 	long headers_size, text_size, total_size;
 	long bytes_remaining = 0;
 	long blocksize;
-	FILE *tmp;
+	FILE *tmp = NULL;
 
-	tmp = tmpfile();
-	if (tmp == NULL) {
-		lprintf(1, "Cannot open temp file: %s\n",
-						strerror(errno));
-		return;
+	/* Cache the most recent RFC822 FETCH because some clients like to
+	 * fetch in pieces, and we don't want to have to go back to the
+	 * message store for each piece.
+	 */
+	if ((IMAP->cached_fetch != NULL) && (IMAP->cached_msgnum == msgnum)) {
+		/* Good to go! */
+		tmp = IMAP->cached_fetch;
+	}
+	else if ((IMAP->cached_fetch != NULL) && (IMAP->cached_msgnum != msgnum)) {
+		/* Some other message is cached -- free it */
+		fclose(IMAP->cached_fetch);
+		IMAP->cached_fetch == NULL;
+		IMAP->cached_msgnum = (-1);
+		tmp = NULL;
 	}
 
-	/*
-	 * Load the message into a temp file for translation
-	 * and measurement
+	/* At this point, we now can fetch and convert the message iff it's not
+	 * the one we had cached.
 	 */
-	CtdlRedirectOutput(tmp, -1);
-	CtdlOutputPreLoadedMsg(msg, msgnum, MT_RFC822,
-				HEADERS_ALL, 0, 1);
-	CtdlRedirectOutput(NULL, -1);
-	if (!is_valid_message(msg)) {
-		lprintf(1, "WARNING: output clobbered the message!\n");
+	if (tmp == NULL) {
+		tmp = tmpfile();
+		if (tmp == NULL) {
+			lprintf(CTDL_CRIT, "Cannot open temp file: %s\n",
+					strerror(errno));
+			return;
+		}
+	
+		/*
+	 	* Load the message into a temp file for translation
+	 	* and measurement
+	 	*/
+		CtdlRedirectOutput(tmp, -1);
+		CtdlOutputMsg(msgnum, MT_RFC822, HEADERS_ALL, 0, 1);
+		CtdlRedirectOutput(NULL, -1);
+
+		IMAP->cached_fetch = tmp;
+		IMAP->cached_msgnum = msgnum;
 	}
 
 	/*
@@ -163,7 +181,6 @@ void imap_fetch_rfc822(int msgnum, char *whichfmt, struct CtdlMessage *msg) {
 
 	if (!strcasecmp(whichfmt, "RFC822.SIZE")) {
 		cprintf("RFC822.SIZE %ld", total_size);
-		fclose(tmp);
 		return;
 	}
 
@@ -191,7 +208,6 @@ void imap_fetch_rfc822(int msgnum, char *whichfmt, struct CtdlMessage *msg) {
 		bytes_remaining = bytes_remaining - blocksize;
 	}
 
-	fclose(tmp);
 }
 
 
@@ -382,7 +398,7 @@ void imap_fetch_envelope(long msgnum, struct CtdlMessage *msg) {
 	fieldptr = rfc822_fetch_field(msg->cm_fields['M'], "Sender");
 	if (fieldptr != NULL) {
 		imap_output_envelope_addr(fieldptr);
-		phree(fieldptr);
+		free(fieldptr);
 	}
 	else {
 		imap_output_envelope_from(msg);
@@ -392,7 +408,7 @@ void imap_fetch_envelope(long msgnum, struct CtdlMessage *msg) {
 	fieldptr = rfc822_fetch_field(msg->cm_fields['M'], "Reply-to");
 	if (fieldptr != NULL) {
 		imap_output_envelope_addr(fieldptr);
-		phree(fieldptr);
+		free(fieldptr);
 	}
 	else {
 		imap_output_envelope_from(msg);
@@ -404,18 +420,18 @@ void imap_fetch_envelope(long msgnum, struct CtdlMessage *msg) {
 	/* Cc */
 	fieldptr = rfc822_fetch_field(msg->cm_fields['M'], "Cc");
 	imap_output_envelope_addr(fieldptr);
-	if (fieldptr != NULL) phree(fieldptr);
+	if (fieldptr != NULL) free(fieldptr);
 
 	/* Bcc */
 	fieldptr = rfc822_fetch_field(msg->cm_fields['M'], "Bcc");
 	imap_output_envelope_addr(fieldptr);
-	if (fieldptr != NULL) phree(fieldptr);
+	if (fieldptr != NULL) free(fieldptr);
 
 	/* In-reply-to */
 	fieldptr = rfc822_fetch_field(msg->cm_fields['M'], "In-reply-to");
 	imap_strout(fieldptr);
 	cprintf(" ");
-	if (fieldptr != NULL) phree(fieldptr);
+	if (fieldptr != NULL) free(fieldptr);
 
 	/* message ID */
 	imap_strout(msg->cm_fields['I']);
@@ -440,7 +456,7 @@ void imap_strip_headers(FILE *fp, char *section) {
 	int ok = 0;
 	int done_headers = 0;
 
-	which_fields = strdoop(section);
+	which_fields = strdup(section);
 
 	if (!strncasecmp(which_fields, "HEADER.FIELDS", 13))
 		doing_headers = 1;
@@ -458,7 +474,7 @@ void imap_strip_headers(FILE *fp, char *section) {
 	num_parms = imap_parameterize(parms, which_fields);
 
 	fseek(fp, 0L, SEEK_END);
-	boiled_headers = mallok((size_t)(ftell(fp) + 256L));
+	boiled_headers = malloc((size_t)(ftell(fp) + 256L));
 	strcpy(boiled_headers, "");
 
 	rewind(fp);
@@ -499,22 +515,22 @@ void imap_strip_headers(FILE *fp, char *section) {
 	ftruncate(fileno(fp), ftell(fp));
 	fflush(fp);
 	rewind(fp);
-	phree(which_fields);
-	phree(boiled_headers);
+	free(which_fields);
+	free(boiled_headers);
 }
 
 
 /*
  * Implements the BODY and BODY.PEEK fetch items
  */
-void imap_fetch_body(long msgnum, char *item, int is_peek,
-		struct CtdlMessage *msg) {
+void imap_fetch_body(long msgnum, char *item, int is_peek) {
+	struct CtdlMessage *msg = NULL;
 	char section[1024];
 	char partial[1024];
 	int is_partial = 0;
 	char buf[1024];
 	int i;
-	FILE *tmp;
+	FILE *tmp = NULL;
 	long bytes_remaining = 0;
 	long blocksize;
 	long pstart, pbytes;
@@ -528,7 +544,20 @@ void imap_fetch_body(long msgnum, char *item, int is_peek,
 	for (i=0; i<strlen(section); ++i) {
 		if (section[i]==']') section[i] = 0;
 	}
-	lprintf(9, "Section is %s\n", section);
+	lprintf(CTDL_DEBUG, "Section is %s\n", section);
+
+	/* Burn the cache if we don't have the same section of the 
+	 * same message again.
+	 */
+	if (IMAP->cached_body != NULL) {
+		if ((IMAP->cached_bodymsgnum != msgnum)
+		   || (strcasecmp(IMAP->cached_bodypart, section)) ) {
+			fclose(IMAP->cached_body);
+			IMAP->cached_body = NULL;
+			IMAP->cached_bodymsgnum = (-1);
+			strcpy(IMAP->cached_bodypart, "");
+		}
+	}
 
 	/* extract partial */
 	strcpy(partial, item);
@@ -542,17 +571,23 @@ void imap_fetch_body(long msgnum, char *item, int is_peek,
 		if (partial[i]=='>') partial[i] = 0;
 	}
 	if (is_partial == 0) strcpy(partial, "");
-	if (strlen(partial) > 0) lprintf(9, "Partial is %s\n", partial);
+	if (strlen(partial) > 0) lprintf(CTDL_DEBUG, "Partial is %s\n", partial);
 
-	tmp = tmpfile();
-	if (tmp == NULL) {
-		lprintf(1, "Cannot open temp file: %s\n", strerror(errno));
-		return;
+	if (IMAP->cached_body == NULL) {
+		tmp = tmpfile();
+		if (tmp == NULL) {
+			lprintf(CTDL_CRIT, "Cannot open temp file: %s\n", strerror(errno));
+			return;
+		}
+		msg = CtdlFetchMessage(msgnum);
 	}
 
 	/* Now figure out what the client wants, and get it */
 
-	if ( (!strcmp(section, "1")) && (msg->cm_format_type != 4) ) {
+	if (IMAP->cached_body != NULL) {
+		tmp = IMAP->cached_body;
+	}
+	else if ( (!strcmp(section, "1")) && (msg->cm_format_type != 4) ) {
 		CtdlRedirectOutput(tmp, -1);
 		CtdlOutputPreLoadedMsg(msg, msgnum, MT_RFC822,
 						HEADERS_NONE, 0, 1);
@@ -630,7 +665,15 @@ void imap_fetch_body(long msgnum, char *item, int is_peek,
 		bytes_remaining = bytes_remaining - blocksize;
 	}
 
-	fclose(tmp);
+	/* Don't close it ... cache it! */
+	/* fclose(tmp); */
+	IMAP->cached_body = tmp;
+	IMAP->cached_bodymsgnum = msgnum;
+	strcpy(IMAP->cached_bodypart, section);
+
+	if (msg != NULL) {
+		CtdlFreeMessage(msg);
+	}
 
 	/* Mark this message as "seen" *unless* this is a "peek" operation */
 	if (is_peek == 0) {
@@ -839,63 +882,74 @@ void imap_fetch_bodystructure (long msgnum, char *item,
 }
 
 
-
-
-
-
 /*
  * imap_do_fetch() calls imap_do_fetch_msg() to output the data of an
- * individual message, once it has been successfully loaded from disk.
+ * individual message, once it has been selected for output.
  */
-void imap_do_fetch_msg(int seq, struct CtdlMessage *msg,
+void imap_do_fetch_msg(int seq,
 			int num_items, char **itemlist) {
 	int i;
+	struct CtdlMessage *msg = NULL;
 
 	cprintf("* %d FETCH (", seq);
 
 	for (i=0; i<num_items; ++i) {
 
-		if (!strncasecmp(itemlist[i], "BODY[", 5)) {
-			imap_fetch_body(IMAP->msgids[seq-1], itemlist[i],
-					0, msg);
-		}
-		else if (!strncasecmp(itemlist[i], "BODY.PEEK[", 10)) {
-			imap_fetch_body(IMAP->msgids[seq-1], itemlist[i],
-					1, msg);
-		}
-		else if (!strcasecmp(itemlist[i], "BODYSTRUCTURE")) {
-			imap_fetch_bodystructure(IMAP->msgids[seq-1],
-					itemlist[i], msg);
-		}
-		else if (!strcasecmp(itemlist[i], "ENVELOPE")) {
-			imap_fetch_envelope(IMAP->msgids[seq-1], msg);
+		/* Fetchable without going to the message store at all */
+		if (!strcasecmp(itemlist[i], "UID")) {
+			imap_fetch_uid(seq);
 		}
 		else if (!strcasecmp(itemlist[i], "FLAGS")) {
 			imap_fetch_flags(seq-1);
 		}
-		else if (!strcasecmp(itemlist[i], "INTERNALDATE")) {
-			imap_fetch_internaldate(msg);
-		}
+
+		/* Potentially fetchable from cache, if the client requests
+		 * stuff from the same message several times in a row.
+		 */
 		else if (!strcasecmp(itemlist[i], "RFC822")) {
-			imap_fetch_rfc822(IMAP->msgids[seq-1], itemlist[i], msg);
+			imap_fetch_rfc822(IMAP->msgids[seq-1], itemlist[i]);
 		}
 		else if (!strcasecmp(itemlist[i], "RFC822.HEADER")) {
-			imap_fetch_rfc822(IMAP->msgids[seq-1], itemlist[i], msg);
+			imap_fetch_rfc822(IMAP->msgids[seq-1], itemlist[i]);
 		}
 		else if (!strcasecmp(itemlist[i], "RFC822.SIZE")) {
-			imap_fetch_rfc822(IMAP->msgids[seq-1], itemlist[i], msg);
+			imap_fetch_rfc822(IMAP->msgids[seq-1], itemlist[i]);
 		}
 		else if (!strcasecmp(itemlist[i], "RFC822.TEXT")) {
-			imap_fetch_rfc822(IMAP->msgids[seq-1], itemlist[i], msg);
+			imap_fetch_rfc822(IMAP->msgids[seq-1], itemlist[i]);
 		}
-		else if (!strcasecmp(itemlist[i], "UID")) {
-			imap_fetch_uid(seq);
+
+		/* BODY fetches do their own fetching and caching too. */
+		else if (!strncasecmp(itemlist[i], "BODY[", 5)) {
+			imap_fetch_body(IMAP->msgids[seq-1], itemlist[i], 0);
+		}
+		else if (!strncasecmp(itemlist[i], "BODY.PEEK[", 10)) {
+			imap_fetch_body(IMAP->msgids[seq-1], itemlist[i], 1);
+		}
+
+		/* Otherwise, load the message into memory.
+		 */
+		else if (!strcasecmp(itemlist[i], "BODYSTRUCTURE")) {
+			if (msg == NULL) msg = CtdlFetchMessage(IMAP->msgids[seq-1]);
+			imap_fetch_bodystructure(IMAP->msgids[seq-1],
+					itemlist[i], msg);
+		}
+		else if (!strcasecmp(itemlist[i], "ENVELOPE")) {
+			if (msg == NULL) msg = CtdlFetchMessage(IMAP->msgids[seq-1]);
+			imap_fetch_envelope(IMAP->msgids[seq-1], msg);
+		}
+		else if (!strcasecmp(itemlist[i], "INTERNALDATE")) {
+			if (msg == NULL) msg = CtdlFetchMessage(IMAP->msgids[seq-1]);
+			imap_fetch_internaldate(msg);
 		}
 
 		if (i != num_items-1) cprintf(" ");
 	}
 
 	cprintf(")\r\n");
+	if (msg != NULL) {
+		CtdlFreeMessage(msg);
+	}
 }
 
 
@@ -906,18 +960,12 @@ void imap_do_fetch_msg(int seq, struct CtdlMessage *msg,
  */
 void imap_do_fetch(int num_items, char **itemlist) {
 	int i;
-	struct CtdlMessage *msg;
 
-	if (IMAP->num_msgs > 0)
-	 for (i = 0; i < IMAP->num_msgs; ++i)
-	  if (IMAP->flags[i] & IMAP_SELECTED) {
-		msg = CtdlFetchMessage(IMAP->msgids[i]);
-		if (msg != NULL) {
-			imap_do_fetch_msg(i+1, msg, num_items, itemlist);
-			CtdlFreeMessage(msg);
-		}
-		else {
-			cprintf("* %d FETCH <internal error>\r\n", i+1);
+	if (IMAP->num_msgs > 0) {
+		for (i = 0; i < IMAP->num_msgs; ++i) {
+			if (IMAP->flags[i] & IMAP_SELECTED) {
+				imap_do_fetch_msg(i+1, num_items, itemlist);
+			}
 		}
 	}
 }
