@@ -1,5 +1,5 @@
 /*
- * $Id: mime_parser.h,v 613.0 2003/12/15 16:37:03 ajc Exp $
+ * $Id: mime_parser.h,v 620.0 2004/03/27 02:33:32 ajc Exp $
  *
  */
 
@@ -11,21 +11,6 @@
 #define SIZ	4096
 #endif
 
-#ifndef mallok
-#define mallok(x) malloc(x)
-#endif
-
-#ifndef phree
-#define phree(x) free(x)
-#endif
-
-#ifndef reallok
-#define reallok(x,y) realloc(x,y)
-#endif
-
-#ifndef strdoop
-#define strdoop(x) strdup(x)
-#endif
 
 /* 
  * Declarations for functions in the parser
