@@ -1,5 +1,5 @@
 /* 
- * $Id: room_ops.c,v 613.0 2003/12/15 16:37:03 ajc Exp $
+ * $Id: room_ops.c,v 620.0 2004/03/27 02:33:32 ajc Exp $
  * 
  * Server functions which perform operations on room objects.
  *
@@ -333,17 +333,31 @@ void lgetfloor(struct floor *flbuf, int floor_num)
 struct floor *cgetfloor(int floor_num) {
 	static int initialized = 0;
 	int i;
+	int fetch_new = 0;
+	struct floor *fl = NULL;
 
+	begin_critical_section(S_FLOORCACHE);
 	if (initialized == 0) {
 		for (i=0; i<MAXFLOORS; ++i) {
 			floorcache[floor_num] = NULL;
 		}
 	initialized = 1;
 	}
-	
 	if (floorcache[floor_num] == NULL) {
-		floorcache[floor_num] = mallok(sizeof(struct floor));
-		getfloor(floorcache[floor_num], floor_num);
+		fetch_new = 1;
+	}
+	end_critical_section(S_FLOORCACHE);
+
+	if (fetch_new) {
+		lprintf(CTDL_DEBUG, "fetch_new is active ... going to disk\n");
+		fl = malloc(sizeof(struct floor));
+		getfloor(fl, floor_num);
+		begin_critical_section(S_FLOORCACHE);
+		if (floorcache[floor_num] != NULL) {
+			free(floorcache[floor_num]);
+		}
+		floorcache[floor_num] = fl;
+		end_critical_section(S_FLOORCACHE);
 	}
 
 	return(floorcache[floor_num]);
@@ -356,14 +370,17 @@ struct floor *cgetfloor(int floor_num) {
  */
 void putfloor(struct floor *flbuf, int floor_num)
 {
+	/* If we've cached this, clear it out, 'cuz it's WRONG now! */
+	begin_critical_section(S_FLOORCACHE);
+	if (floorcache[floor_num] != NULL) {
+		free(floorcache[floor_num]);
+		floorcache[floor_num] = malloc(sizeof(struct floor));
+		memcpy(floorcache[floor_num], flbuf, sizeof(struct floor));
+	}
+	end_critical_section(S_FLOORCACHE);
+
 	cdb_store(CDB_FLOORTAB, &floor_num, sizeof(int),
 		  flbuf, sizeof(struct floor));
-
-	/* If we've cached this, clear it out, 'cuz it's WRONG now! */
-	if (floorcache[floor_num] != NULL) {
-		phree(floorcache[floor_num]);
-		floorcache[floor_num] = NULL;
-	}
 }
 
 
@@ -788,7 +805,7 @@ void usergoto(char *where, int display_result, int transiently,
 	get_mm();
         cdbfr = cdb_fetch(CDB_MSGLISTS, &CC->room.QRnumber, sizeof(long));
         if (cdbfr != NULL) {
-        	msglist = mallok(cdbfr->len);
+        	msglist = malloc(cdbfr->len);
         	memcpy(msglist, cdbfr->ptr, cdbfr->len);
         	num_msgs = cdbfr->len / sizeof(long);
         	cdb_free(cdbfr);
@@ -803,7 +820,7 @@ void usergoto(char *where, int display_result, int transiently,
 		}
 	}
 
-	if (msglist != NULL) phree(msglist);
+	if (msglist != NULL) free(msglist);
 
 	if (CC->room.QRflags & QR_MAILBOX)
 		rmailflag = 1;
@@ -824,7 +841,7 @@ void usergoto(char *where, int display_result, int transiently,
 
 	if (retmsgs != NULL) *retmsgs = total_messages;
 	if (retnew != NULL) *retnew = new_messages;
-	lprintf(9, "<%s> %d new of %d total messages\n",
+	lprintf(CTDL_DEBUG, "<%s> %d new of %d total messages\n",
 		CC->room.QRname,
 		new_messages, total_messages
 	);
@@ -934,8 +951,7 @@ void cmd_goto(char *gargs)
 				   ((ra & UA_KNOWN) == 0) &&
 			           (CC->user.axlevel < 6)
                                   ) {
-				lprintf(9, "Failed to acquire private room\n");
-				goto NOPE;
+				lprintf(CTDL_DEBUG, "Failed to acquire private room\n");
 			} else {
 				memcpy(&CC->room, &QRscratch,
 					sizeof(struct ctdlroom));
@@ -945,7 +961,7 @@ void cmd_goto(char *gargs)
 		}
 	}
 
-NOPE:	cprintf("%d room '%s' not found\n", ERROR + ROOM_NOT_FOUND, towhere);
+	cprintf("%d room '%s' not found\n", ERROR + ROOM_NOT_FOUND, towhere);
 }
 
 
@@ -1102,7 +1118,7 @@ int CtdlRenameRoom(char *old_name, char *new_name, int new_floor) {
 	long owner = 0L;
 	char actual_old_name[SIZ];
 
-	lprintf(9, "CtdlRenameRoom(%s, %s, %d)\n",
+	lprintf(CTDL_DEBUG, "CtdlRenameRoom(%s, %s, %d)\n",
 		old_name, new_name, new_floor);
 
 	if (new_floor >= 0) {
@@ -1193,11 +1209,11 @@ int CtdlRenameRoom(char *old_name, char *new_name, int new_floor) {
 		lgetfloor(&flbuf, old_floor);
 		--flbuf.f_ref_count;
 		lputfloor(&flbuf, old_floor);
-		lprintf(9, "Reference count for floor %d is now %d\n", old_floor, flbuf.f_ref_count);
+		lprintf(CTDL_DEBUG, "Reference count for floor %d is now %d\n", old_floor, flbuf.f_ref_count);
 		lgetfloor(&flbuf, new_floor);
 		++flbuf.f_ref_count;
 		lputfloor(&flbuf, new_floor);
-		lprintf(9, "Reference count for floor %d is now %d\n", new_floor, flbuf.f_ref_count);
+		lprintf(CTDL_DEBUG, "Reference count for floor %d is now %d\n", new_floor, flbuf.f_ref_count);
 	}
 
 	/* ...and everybody say "YATTA!" */	
@@ -1237,12 +1253,12 @@ void cmd_setr(char *args)
 	r = CtdlRenameRoom(CC->room.QRname, new_name, new_floor);
 
 	if (r == crr_room_not_found) {
-		cprintf("%d Internal error - room not found?\n", ERROR);
+		cprintf("%d Internal error - room not found?\n", ERROR + INTERNAL_ERROR);
 	} else if (r == crr_already_exists) {
 		cprintf("%d '%s' already exists.\n",
 			ERROR + ALREADY_EXISTS, new_name);
 	} else if (r == crr_noneditable) {
-		cprintf("%d Cannot edit this room.\n", ERROR);
+		cprintf("%d Cannot edit this room.\n", ERROR + NOT_HERE);
 	} else if (r == crr_invalid_floor) {
 		cprintf("%d Target floor does not exist.\n",
 			ERROR + INVALID_FLOOR_OPERATION);
@@ -1252,7 +1268,7 @@ void cmd_setr(char *args)
 			CC->room.QRname);
 	} else if (r != crr_ok) {
 		cprintf("%d Error: CtdlRenameRoom() returned %d\n",
-			ERROR, r);
+			ERROR + INTERNAL_ERROR, r);
 	}
 
 	if (r != crr_ok) {
@@ -1430,7 +1446,7 @@ void cmd_rinf(void)
 	info_fp = fopen(filename, "r");
 
 	if (info_fp == NULL) {
-		cprintf("%d No info file.\n", ERROR);
+		cprintf("%d No info file.\n", ERROR + FILE_NOT_FOUND);
 		return;
 	}
 	cprintf("%d Info:\n", LISTING_FOLLOWS);
@@ -1451,7 +1467,7 @@ void delete_room(struct ctdlroom *qrbuf)
 	struct floor flbuf;
 	char filename[100];
 
-	lprintf(9, "Deleting room <%s>\n", qrbuf->QRname);
+	lprintf(CTDL_NOTICE, "Deleting room <%s>\n", qrbuf->QRname);
 
 	/* Delete the info file */
 	assoc_file_name(filename, sizeof filename, qrbuf, "info");
@@ -1581,9 +1597,9 @@ unsigned create_room(char *new_room_name,
 	struct floor flbuf;
 	struct visit vbuf;
 
-	lprintf(9, "create_room(%s)\n", new_room_name);
+	lprintf(CTDL_DEBUG, "create_room(%s)\n", new_room_name);
 	if (getroom(&qrbuf, new_room_name) == 0) {
-		lprintf(9, "%s already exists.\n", new_room_name);
+		lprintf(CTDL_DEBUG, "%s already exists.\n", new_room_name);
 		return (0);	/* already exists */
 	}
 
@@ -1681,7 +1697,7 @@ void cmd_cre8(char *args)
 	new_room_floor = 0;
 
 	if ((strlen(new_room_name) == 0) && (cre8_ok == 1)) {
-		cprintf("%d Invalid room name.\n", ERROR);
+		cprintf("%d Invalid room name.\n", ERROR + ILLEGAL_VALUE);
 		return;
 	}
 
@@ -1693,7 +1709,12 @@ void cmd_cre8(char *args)
 
 	if (num_parms(args) >= 5) {
 		fl = cgetfloor(extract_int(args, 4));
-		if ((fl->f_flags & F_INUSE) == 0) {
+		if (fl == NULL) {
+			cprintf("%d Invalid floor number.\n",
+				ERROR + INVALID_FLOOR_OPERATION);
+			return;
+		}
+		else if ((fl->f_flags & F_INUSE) == 0) {
 			cprintf("%d Invalid floor number.\n",
 				ERROR + INVALID_FLOOR_OPERATION);
 			return;
@@ -1716,7 +1737,7 @@ void cmd_cre8(char *args)
 	}
 
 	if ((new_room_type < 0) || (new_room_type > 5)) {
-		cprintf("%d Invalid room type.\n", ERROR);
+		cprintf("%d Invalid room type.\n", ERROR + ILLEGAL_VALUE);
 		return;
 	}
 
@@ -1724,7 +1745,7 @@ void cmd_cre8(char *args)
 		if ((config.c_aide_mailboxes == 0)
 		   || (CC->user.axlevel < 6)) {
 			cprintf("%d Higher access required\n", 
-				ERROR+HIGHER_ACCESS_REQUIRED);
+				ERROR + HIGHER_ACCESS_REQUIRED);
 			return;
 		}
 	}
@@ -1784,9 +1805,9 @@ void cmd_einf(char *ok)
 		return;
 	}
 	assoc_file_name(infofilename, sizeof infofilename, &CC->room, "info");
-	lprintf(9, "opening\n");
+	lprintf(CTDL_DEBUG, "opening\n");
 	fp = fopen(infofilename, "w");
-	lprintf(9, "checking\n");
+	lprintf(CTDL_DEBUG, "checking\n");
 	if (fp == NULL) {
 		cprintf("%d Cannot open %s: %s\n",
 		  ERROR + INTERNAL_ERROR, infofilename, strerror(errno));
@@ -1852,7 +1873,7 @@ void cmd_cflr(char *argbuf)
 
 	if (strlen(new_floor_name) == 0) {
 		cprintf("%d Blank floor name not allowed.\n",
-			ERROR+ILLEGAL_VALUE);
+			ERROR + ILLEGAL_VALUE);
 		return;
 	}
 
@@ -1948,7 +1969,7 @@ void cmd_eflr(char *argbuf)
 
 	np = num_parms(argbuf);
 	if (np < 1) {
-		cprintf("%d Usage error.\n", ERROR);
+		cprintf("%d Usage error.\n", ERROR + ILLEGAL_VALUE);
 		return;
 	}
 
