@@ -1,5 +1,5 @@
 /*
- * $Id: rooms.c,v 613.0 2003/12/15 16:37:03 ajc Exp $
+ * $Id: rooms.c,v 620.0 2004/03/27 02:33:32 ajc Exp $
  *
  * 
  * Client-side functions which perform room operations
@@ -35,9 +35,8 @@
 
 
 void sttybbs(int cmd);
-void hit_any_key(void);
 void dotgoto(CtdlIPC *ipc, char *towhere, int display_name, int fromungoto);
-void progress(unsigned long curr, unsigned long cmax);
+void progress(CtdlIPC* ipc, unsigned long curr, unsigned long cmax);
 int pattern(char *search, char *patn);
 int file_checksum(char *filename);
 int nukedir(char *dirname);
@@ -313,7 +312,7 @@ void knrooms(CtdlIPC *ipc, int kn_floor_mode)
 	};
 
 	color(DIM_WHITE);
-	IFNEXPERT hit_any_key();
+	IFNEXPERT hit_any_key(ipc);
 }
 
 
@@ -344,7 +343,7 @@ void listzrooms(CtdlIPC *ipc)
 	};
 
 	color(DIM_WHITE);
-	IFNEXPERT hit_any_key();
+	IFNEXPERT hit_any_key(ipc);
 }
 
 
@@ -846,19 +845,23 @@ void roomdir(CtdlIPC *ipc)
 	char flsz[32];
 	char comment[SIZ];
 	char buf[SIZ];
+	char *listing = NULL;	/* Returned directory listing */
+	int r;
 
-	CtdlIPC_putline(ipc, "RDIR");
-	CtdlIPC_getline(ipc, buf);
-	if (buf[0] != '1') {
-		pprintf("%s\n", &buf[4]);
+	r = CtdlIPCReadDirectory(ipc, &listing, buf);
+	if (r / 100 != 1) {
+		pprintf("%s\n", buf);
 		return;
 	}
 
-	extract(comment, &buf[4], 0);
-	extract(flnm, &buf[4], 1);
+	extract(comment, buf, 0);
+	extract(flnm, buf, 1);
 	pprintf("\nDirectory of %s on %s\n", flnm, comment);
 	pprintf("-----------------------\n");
-	while (CtdlIPC_getline(ipc, buf), strcmp(buf, "000")) {
+	while (*listing && strlen(listing)) {
+		extract_token(buf, listing, 0, '\n');
+		remove_token(listing, 0, '\n');
+
 		extract(flnm, buf, 0);
 		extract(flsz, buf, 1);
 		extract(comment, buf, 2);
@@ -1108,19 +1111,19 @@ void do_edit(CtdlIPC *ipc,
 	fp = fopen(temp, "w");
 	fclose(fp);
 
-	CtdlIPC_putline(ipc, check_cmd);
-	CtdlIPC_getline(ipc, cmd);
+	CtdlIPC_chat_send(ipc, check_cmd);
+	CtdlIPC_chat_recv(ipc, cmd);
 	if (cmd[0] != '2') {
 		scr_printf("%s\n", &cmd[4]);
 		return;
 	}
 
 	if (strlen(editor_paths[0]) > 0) {
-		CtdlIPC_putline(ipc, read_cmd);
-		CtdlIPC_getline(ipc, cmd);
+		CtdlIPC_chat_send(ipc, read_cmd);
+		CtdlIPC_chat_recv(ipc, cmd);
 		if (cmd[0] == '1') {
 			fp = fopen(temp, "w");
-			while (CtdlIPC_getline(ipc, cmd), strcmp(cmd, "000")) {
+			while (CtdlIPC_chat_recv(ipc, cmd), strcmp(cmd, "000")) {
 				fprintf(fp, "%s\n", cmd);
 			}
 			fclose(fp);
@@ -1164,8 +1167,8 @@ void do_edit(CtdlIPC *ipc,
 	}
 
 	else {
-		CtdlIPC_putline(ipc, write_cmd);
-		CtdlIPC_getline(ipc, cmd);
+		CtdlIPC_chat_send(ipc, write_cmd);
+		CtdlIPC_chat_recv(ipc, cmd);
 		if (cmd[0] != '4') {
 			scr_printf("%s\n", &cmd[4]);
 			return;
@@ -1174,10 +1177,10 @@ void do_edit(CtdlIPC *ipc,
 		fp = fopen(temp, "r");
 		while (fgets(cmd, SIZ - 1, fp) != NULL) {
 			cmd[strlen(cmd) - 1] = 0;
-			CtdlIPC_putline(ipc, cmd);
+			CtdlIPC_chat_send(ipc, cmd);
 		}
 		fclose(fp);
-		CtdlIPC_putline(ipc, "000");
+		CtdlIPC_chat_send(ipc, "000");
 	}
 
 	unlink(temp);
