@@ -1,4 +1,4 @@
-/* $Id: rooms.h,v 613.0 2003/12/15 16:37:03 ajc Exp $ */
+/* $Id: rooms.h,v 620.0 2004/03/27 02:33:32 ajc Exp $ */
 void listzrooms(CtdlIPC *ipc);
 void readinfo(CtdlIPC *ipc);
 void forget(CtdlIPC *ipc);
@@ -19,7 +19,7 @@ void create_floor(CtdlIPC *ipc);
 void edit_floor(CtdlIPC *ipc);
 void kill_floor(CtdlIPC *ipc);
 void enter_bio(CtdlIPC *ipc);
-void hit_any_key(void);
+void hit_any_key(CtdlIPC *ipc);
 int save_buffer(void *file, size_t filelen, const char *pathname);
 void destination_directory(char *dest, const char *supplied_filename);
 void do_edit(CtdlIPC *ipc,
