@@ -1,14 +1,14 @@
-/* $Id: routines.h,v 613.0 2003/12/15 16:37:03 ajc Exp $ */
+/* $Id: routines.h,v 620.0 2004/03/27 02:33:32 ajc Exp $ */
 void edituser(CtdlIPC *ipc);
 void interr(int errnum);
 int struncmp(char *lstr, char *rstr, int len);
 int pattern(char *search, char *patn);
-void enter_config(CtdlIPC *ipc, int mode);
-void locate_host(char *hbuf);
+void enter_config(CtdlIPC* ipc, int mode);
+void locate_host(CtdlIPC* ipc, char *hbuf);
 void misc_server_cmd(CtdlIPC *ipc, char *cmd);
 int nukedir(char *dirname);
 int num_parms(char *source);
 void strproc(char *string);
 void back(int spaces);
-void progress(unsigned long curr, unsigned long cmax);
+void progress(CtdlIPC* ipc, unsigned long curr, unsigned long cmax);
 int set_attr(CtdlIPC *ipc, unsigned int sval, char *prompt, unsigned int sbit, int backwards);
