@@ -1,11 +1,10 @@
-/* $Id: routines2.c,v 613.0 2003/12/15 16:37:03 ajc Exp $
+/* $Id: routines2.c,v 620.0 2004/03/27 02:33:32 ajc Exp $
  *
  * More client-side support functions.
  * Unlike routines.c, some of these DO use global variables.
  *
  */
 
-#include "sysdep.h"
 #include <stdlib.h>
 #include <unistd.h>
 #include <sys/types.h>
@@ -31,6 +30,7 @@
 #include <pwd.h>
 #include <errno.h>
 #include <stdarg.h>
+#include "sysdep.h"
 #include "citadel.h"
 #include "citadel_ipc.h"
 #include "citadel_decls.h"
@@ -161,7 +161,7 @@ void entregis(CtdlIPC *ipc)
 			striplt(diruser);
 			striplt(dirnode);
 			if ((strcasecmp(diruser, fullname))
-			   || (strcasecmp(dirnode, serv_info.serv_nodename))) {
+			   || (strcasecmp(dirnode, ipc->ServInfo.nodename))) {
 				scr_printf(
 					"\nYou can't use %s as your address.\n",
 					tmpemail);
@@ -510,10 +510,11 @@ void validate(CtdlIPC *ipc)
 void subshell(void)
 {
 	int a, b;
+
+	screen_reset();
+	sttybbs(SB_RESTORE);
 	a = fork();
 	if (a == 0) {
-		screen_reset();
-		sttybbs(SB_RESTORE);
 		signal(SIGINT, SIG_DFL);
 		signal(SIGQUIT, SIG_DFL);
 		execlp(getenv("SHELL"), getenv("SHELL"), NULL);
@@ -642,10 +643,14 @@ void read_bio(CtdlIPC *ipc)
  */
 void do_system_configuration(CtdlIPC *ipc)
 {
+
+#define NUM_CONFIGS 37
+
 	char buf[SIZ];
-	char sc[32][SIZ];
+	char sc[NUM_CONFIGS][SIZ];
 	char *resp = NULL;
-	struct ExpirePolicy *expirepolicy = NULL;
+	struct ExpirePolicy *site_expirepolicy = NULL;
+	struct ExpirePolicy *mbx_expirepolicy = NULL;
 	int a;
 	int logpages = 0;
 	int r;			/* IPC response code */
@@ -660,7 +665,7 @@ void do_system_configuration(CtdlIPC *ipc)
 		while (strlen(resp)) {
 			extract_token(buf, resp, 0, '\n');
 			remove_token(resp, 0, '\n');
-			if (a < 32) {
+			if (a < NUM_CONFIGS) {
 				strcpy(&sc[a][0], buf);
 			}
 			++a;
@@ -671,7 +676,8 @@ void do_system_configuration(CtdlIPC *ipc)
 	/* Fetch the expire policy (this will silently fail on old servers,
 	 * resulting in "default" policy)
 	 */
-	r = CtdlIPCGetMessageExpirationPolicy(ipc, 2, &expirepolicy, buf);
+	r = CtdlIPCGetMessageExpirationPolicy(ipc, 2, &site_expirepolicy, buf);
+	r = CtdlIPCGetMessageExpirationPolicy(ipc, 3, &mbx_expirepolicy, buf);
 
 	/* Identification parameters */
 
@@ -745,13 +751,32 @@ void do_system_configuration(CtdlIPC *ipc)
 	a = (a ? 0 : 1);
 	snprintf(sc[25], sizeof sc[25], "%d", a);
 
+	/* LDAP settings */
+	if (ipc->ServInfo.supports_ldap) {
+		a = strlen(&sc[32][0]);
+		a = (a ? 1 : 0);	/* Set only to 1 or 0 */
+		a = boolprompt("Connect this Citadel to an LDAP directory", a);
+		if (a) {
+			strprompt("Host name of LDAP server",
+				&sc[32][0], 127);
+			strprompt("Port number of LDAP service",
+				&sc[33][0], 5);
+			strprompt("Base DN", &sc[34][0], 255);
+			strprompt("Bind DN", &sc[35][0], 255);
+			strprompt("Password for bind DN", &sc[36][0], 255);
+		}
+		else {
+			strcpy(&sc[32][0], "");
+		}
+	}
+
 	/* Expiry settings */
 	strprompt("Default user purge time (days)", &sc[16][0], 5);
 	strprompt("Default room purge time (days)", &sc[17][0], 5);
 
 	/* Angels and demons dancing in my head... */
 	do {
-		snprintf(buf, sizeof buf, "%d", expirepolicy->expire_mode);
+		snprintf(buf, sizeof buf, "%d", site_expirepolicy->expire_mode);
 		strprompt("System default message expire policy (? for list)",
 			  buf, 1);
 		if (buf[0] == '?') {
@@ -761,18 +786,45 @@ void do_system_configuration(CtdlIPC *ipc)
 				"3. Expire by message age\n");
 		}
 	} while ((buf[0] < '1') || (buf[0] > '3'));
-	expirepolicy->expire_mode = buf[0] - '0';
+	site_expirepolicy->expire_mode = buf[0] - '0';
 
 	/* ...lunatics and monsters underneath my bed */
-	if (expirepolicy->expire_mode == 2) {
-		snprintf(buf, sizeof buf, "%d", expirepolicy->expire_value);
+	if (site_expirepolicy->expire_mode == 2) {
+		snprintf(buf, sizeof buf, "%d", site_expirepolicy->expire_value);
 		strprompt("Keep how many messages online?", buf, 10);
-		expirepolicy->expire_value = atol(buf);
+		site_expirepolicy->expire_value = atol(buf);
 	}
-	if (expirepolicy->expire_mode == 3) {
-		snprintf(buf, sizeof buf, "%d", expirepolicy->expire_value);
+	if (site_expirepolicy->expire_mode == 3) {
+		snprintf(buf, sizeof buf, "%d", site_expirepolicy->expire_value);
 		strprompt("Keep messages for how many days?", buf, 10);
-		expirepolicy->expire_value = atol(buf);
+		site_expirepolicy->expire_value = atol(buf);
+	}
+
+	/* Media messiahs preying on my fears... */
+	do {
+		snprintf(buf, sizeof buf, "%d", mbx_expirepolicy->expire_mode);
+		strprompt("Mailbox default message expire policy (? for list)",
+			  buf, 1);
+		if (buf[0] == '?') {
+			scr_printf("\n"
+				"0. Go with the system default\n"
+				"1. Never automatically expire messages\n"
+				"2. Expire by message count\n"
+				"3. Expire by message age\n");
+		}
+	} while ((buf[0] < '0') || (buf[0] > '3'));
+	mbx_expirepolicy->expire_mode = buf[0] - '0';
+
+	/* ...Pop culture prophets playing in my ears */
+	if (mbx_expirepolicy->expire_mode == 2) {
+		snprintf(buf, sizeof buf, "%d", mbx_expirepolicy->expire_value);
+		strprompt("Keep how many messages online?", buf, 10);
+		mbx_expirepolicy->expire_value = atol(buf);
+	}
+	if (mbx_expirepolicy->expire_mode == 3) {
+		snprintf(buf, sizeof buf, "%d", mbx_expirepolicy->expire_value);
+		strprompt("Keep messages for how many days?", buf, 10);
+		mbx_expirepolicy->expire_value = atol(buf);
 	}
 
 	strprompt("How often to run network jobs (in seconds)", &sc[28][0], 5);
@@ -782,14 +834,14 @@ void do_system_configuration(CtdlIPC *ipc)
 	scr_printf("Save this configuration? ");
 	if (yesno()) {
 		r = 1;
-		for (a = 0; a < 32; a++)
+		for (a = 0; a < NUM_CONFIGS; a++)
 			r += 1 + strlen(sc[a]);
 		resp = (char *)calloc(1, r);
 		if (!resp) {
 			err_printf("Can't save config - out of memory!\n");
 			logoff(ipc, 1);
 		}
-		for (a = 0; a < 32; a++) {
+		for (a = 0; a < NUM_CONFIGS; a++) {
 			strcat(resp, sc[a]);
 			strcat(resp, "\n");
 		}
@@ -798,10 +850,17 @@ void do_system_configuration(CtdlIPC *ipc)
 			err_printf("%s\n", buf);
 		}
 		free(resp);
-		r = CtdlIPCSetMessageExpirationPolicy(ipc, 2, expirepolicy, buf);
+
+		r = CtdlIPCSetMessageExpirationPolicy(ipc, 2, site_expirepolicy, buf);
 		if (r / 100 != 2) {
 			err_printf("%s\n", buf);
 		}
+
+		r = CtdlIPCSetMessageExpirationPolicy(ipc, 3, mbx_expirepolicy, buf);
+		if (r / 100 != 2) {
+			err_printf("%s\n", buf);
+		}
+
 	}
 }
 
@@ -849,6 +908,7 @@ void do_internet_configuration(CtdlIPC *ipc)
 	int badkey;
 	int i, j;
 	int quitting = 0;
+	int modified = 0;
 	int r;
 	
 	r = CtdlIPCGetSystemConfigByType(ipc, INTERNETCFG, &resp, buf);
@@ -900,6 +960,7 @@ void do_internet_configuration(CtdlIPC *ipc)
 							&buf[strlen(buf)]);
 					recs[num_recs-1] = strdup(buf);
 				}
+				modified = 1;
 				break;
 			case 'd':
 				i = intprompt("Delete which one",
@@ -908,6 +969,7 @@ void do_internet_configuration(CtdlIPC *ipc)
 				--num_recs;
 				for (j=i; j<num_recs; ++j)
 					recs[j] = recs[j+1];
+				modified = 1;
 				break;
 			case 's':
 				r = 1;
@@ -925,17 +987,19 @@ void do_internet_configuration(CtdlIPC *ipc)
 				r = CtdlIPCSetSystemConfigByType(ipc, INTERNETCFG, resp, buf);
 				if (r / 100 != 4) {
 					err_printf("%s\n", buf);
+				} else {
+					scr_printf("Wrote %d records.\n", num_recs);
+					modified = 0;
 				}
-				quitting = 1;
 				break;
 			case 'q':
-				quitting = boolprompt(
+				quitting = !modified || boolprompt(
 					"Quit without saving", 0);
 				break;
 			default:
 				badkey = 1;
 		}
-	} while (quitting == 0);
+	} while (!quitting);
 
 	if (recs != NULL) {
 		for (i=0; i<num_recs; ++i) free(recs[i]);
@@ -961,6 +1025,8 @@ void network_config_management(CtdlIPC *ipc, char *entrytype, char *comment)
 	char addr[SIZ];
 	FILE *tempfp;
 	FILE *changefp;
+	char *listing = NULL;
+	int r;
 
 	if (strlen(editor_paths[0]) == 0) {
 		scr_printf("You must have an external editor configured in"
@@ -982,10 +1048,11 @@ void network_config_management(CtdlIPC *ipc, char *entrytype, char *comment)
 	fprintf(tempfp, "# Specify one per line.\n"
 			"\n\n");
 
-	CtdlIPC_putline(ipc, "GNET");
-	CtdlIPC_getline(ipc, buf);
-	if (buf[0] == '1') {
-		while(CtdlIPC_getline(ipc, buf), strcmp(buf, "000")) {
+	r = CtdlIPCGetRoomNetworkConfig(ipc, &listing, buf);
+	if (r / 100 == 1) {
+		while(listing && strlen(listing)) {
+			extract_token(buf, listing, 0, '\n');
+			remove_token(listing, 0, '\n');
 			extract(instr, buf, 0);
 			if (!strcasecmp(instr, entrytype)) {
 				extract(addr, buf, 1);
@@ -993,6 +1060,10 @@ void network_config_management(CtdlIPC *ipc, char *entrytype, char *comment)
 			}
 		}
 	}
+	if (listing) {
+		free(listing);
+		listing = NULL;
+	}
 	fclose(tempfp);
 
 	e_ex_code = 1;	/* start with a failed exit code */
@@ -1023,16 +1094,21 @@ void network_config_management(CtdlIPC *ipc, char *entrytype, char *comment)
 
 	if (e_ex_code == 0) { 		/* Save changes */
 		changefp = fopen(changefile, "w");
-		CtdlIPC_putline(ipc, "GNET");
-		CtdlIPC_getline(ipc, buf);
-		if (buf[0] == '1') {
-			while(CtdlIPC_getline(ipc, buf), strcmp(buf, "000")) {
+		r = CtdlIPCGetRoomNetworkConfig(ipc, &listing, buf);
+		if (r / 100 == 1) {
+			while(listing && strlen(listing)) {
+				extract_token(buf, listing, 0, '\n');
+				remove_token(listing, 0, '\n');
 				extract(instr, buf, 0);
 				if (strcasecmp(instr, entrytype)) {
 					fprintf(changefp, "%s\n", buf);
 				}
 			}
 		}
+		if (listing) {
+			free(listing);
+			listing = NULL;
+		}
 		tempfp = fopen(filename, "r");
 		while (fgets(buf, sizeof buf, tempfp) != NULL) {
 			for (i=0; i<strlen(buf); ++i) {
@@ -1047,19 +1123,15 @@ void network_config_management(CtdlIPC *ipc, char *entrytype, char *comment)
 		fclose(changefp);
 
 		/* now write it to the server... */
-		CtdlIPC_putline(ipc, "SNET");
-		CtdlIPC_getline(ipc, buf);
-		if (buf[0] == '4') {
-			changefp = fopen(changefile, "r");
-			if (changefp != NULL) {
-				while (fgets(buf, sizeof buf,
-				       changefp) != NULL) {
-					buf[strlen(buf) - 1] = 0;
-					CtdlIPC_putline(ipc, buf);
-				}
-				fclose(changefp);
+		changefp = fopen(changefile, "r");
+		if (changefp != NULL) {
+			listing = load_message_from_file(changefp);
+			if (listing) {
+				r = CtdlIPCSetRoomNetworkConfig(ipc, listing, buf);
+				free(listing);
+				listing = NULL;
 			}
-			CtdlIPC_putline(ipc, "000");
+			fclose(changefp);
 		}
 	}
 
@@ -1079,18 +1151,22 @@ void do_ignet_configuration(CtdlIPC *ipc) {
 	int badkey;
 	int i, j;
 	int quitting = 0;
-	
+	int modified = 0;
+	char *listing = NULL;
+	int r;
+
+	r = CtdlIPCGetSystemConfigByType(ipc, IGNETCFG, &listing, buf);
+	if (r / 100 == 1) while (*listing && strlen(listing)) {
+		extract_token(buf, listing, 0, '\n');
+		remove_token(listing, 0, '\n');
 
-	snprintf(buf, sizeof buf, "CONF getsys|%s", IGNETCFG);
-	CtdlIPC_putline(ipc, buf);
-	CtdlIPC_getline(ipc, buf);
-	if (buf[0] == '1') while (CtdlIPC_getline(ipc, buf), strcmp(buf, "000")) {
 		++num_recs;
 		if (num_recs == 1) recs = malloc(sizeof(char *));
 		else recs = realloc(recs, (sizeof(char *)) * num_recs);
 		recs[num_recs-1] = malloc(SIZ);
 		strcpy(recs[num_recs-1], buf);
 	}
+	if (listing) free(listing);
 
 	do {
 		scr_printf("\n");
@@ -1123,6 +1199,7 @@ void do_ignet_configuration(CtdlIPC *ipc) {
 		scr_printf("%-3s\n", buf);
 		color(DIM_WHITE);
 		}
+		scr_printf("\n");
 
 		ch = keymenu("", "<A>dd|<D>elete|<S>ave|<Q>uit");
 		switch(ch) {
@@ -1143,6 +1220,7 @@ void do_ignet_configuration(CtdlIPC *ipc) {
 				strprompt("Enter port number  : ",
 					&buf[strlen(buf)-3], 5);
 				recs[num_recs-1] = strdup(buf);
+				modified = 1;
 				break;
 			case 'd':
 				i = intprompt("Delete which one",
@@ -1151,30 +1229,37 @@ void do_ignet_configuration(CtdlIPC *ipc) {
 				--num_recs;
 				for (j=i; j<num_recs; ++j)
 					recs[j] = recs[j+1];
+				modified = 1;
 				break;
 			case 's':
-				snprintf(buf, sizeof buf, "CONF putsys|%s", IGNETCFG);
-				CtdlIPC_putline(ipc, buf);
-				CtdlIPC_getline(ipc, buf);
-				if (buf[0] == '4') {
-					for (i=0; i<num_recs; ++i) {
-						CtdlIPC_putline(ipc, recs[i]);
-					}
-					CtdlIPC_putline(ipc, "000");
+				r = 1;
+				for (i = 0; i < num_recs; ++i)
+					r += 1 + strlen(recs[i]);
+				listing = (char*) calloc(1, r);
+				if (!listing) {
+					err_printf("Can't save config - out of memory!\n");
+					logoff(ipc, 1);
 				}
-				else {
-					scr_printf("%s\n", &buf[4]);
+				if (num_recs) for (i = 0; i < num_recs; ++i) {
+					strcat(listing, recs[i]);
+					strcat(listing, "\n");
+				}
+				r = CtdlIPCSetSystemConfigByType(ipc, IGNETCFG, listing, buf);
+				if (r / 100 != 4) {
+					scr_printf("%s\n", buf);
+				} else {
+					scr_printf("Wrote %d records.\n", num_recs);
+					modified = 0;
 				}
-				quitting = 1;
 				break;
 			case 'q':
-				quitting = boolprompt(
+				quitting = !modified || boolprompt(
 					"Quit without saving", 0);
 				break;
 			default:
 				badkey = 1;
 		}
-	} while (quitting == 0);
+	} while (!quitting);
 
 	if (recs != NULL) {
 		for (i=0; i<num_recs; ++i) free(recs[i]);
@@ -1194,18 +1279,22 @@ void do_filterlist_configuration(CtdlIPC *ipc)
 	int badkey;
 	int i, j;
 	int quitting = 0;
-	
+	int modified = 0;
+	char *listing = NULL;
+	int r;
+
+	r = CtdlIPCGetSystemConfigByType(ipc, FILTERLIST, &listing, buf);
+	if (r / 100 == 1) while (*listing && strlen(listing)) {
+		extract_token(buf, listing, 0, '\n');
+		remove_token(listing, 0, '\n');
 
-	snprintf(buf, sizeof buf, "CONF getsys|%s", FILTERLIST);
-	CtdlIPC_putline(ipc, buf);
-	CtdlIPC_getline(ipc, buf);
-	if (buf[0] == '1') while (CtdlIPC_getline(ipc, buf), strcmp(buf, "000")) {
 		++num_recs;
 		if (num_recs == 1) recs = malloc(sizeof(char *));
 		else recs = realloc(recs, (sizeof(char *)) * num_recs);
 		recs[num_recs-1] = malloc(SIZ);
 		strcpy(recs[num_recs-1], buf);
 	}
+	if (listing) free(listing);
 
 	do {
 		scr_printf("\n");
@@ -1254,6 +1343,7 @@ void do_filterlist_configuration(CtdlIPC *ipc)
 					&buf[strlen(buf)], 16);
 				strcat(buf, "|");
 				recs[num_recs-1] = strdup(buf);
+				modified = 1;
 				break;
 			case 'd':
 				i = intprompt("Delete which one",
@@ -1262,30 +1352,37 @@ void do_filterlist_configuration(CtdlIPC *ipc)
 				--num_recs;
 				for (j=i; j<num_recs; ++j)
 					recs[j] = recs[j+1];
+				modified = 1;
 				break;
 			case 's':
-				snprintf(buf, sizeof buf, "CONF putsys|%s", FILTERLIST);
-				CtdlIPC_putline(ipc, buf);
-				CtdlIPC_getline(ipc, buf);
-				if (buf[0] == '4') {
-					for (i=0; i<num_recs; ++i) {
-						CtdlIPC_putline(ipc, recs[i]);
-					}
-					CtdlIPC_putline(ipc, "000");
+				r = 1;
+				for (i = 0; i < num_recs; ++i)
+					r += 1 + strlen(recs[i]);
+				listing = (char*) calloc(1, r);
+				if (!listing) {
+					err_printf("Can't save config - out of memory!\n");
+					logoff(ipc, 1);
 				}
-				else {
-					scr_printf("%s\n", &buf[4]);
+				if (num_recs) for (i = 0; i < num_recs; ++i) {
+					strcat(listing, recs[i]);
+					strcat(listing, "\n");
+				}
+				r = CtdlIPCSetSystemConfigByType(ipc, FILTERLIST, listing, buf);
+				if (r / 100 != 4) {
+					scr_printf("%s\n", buf);
+				} else {
+					scr_printf("Wrote %d records.\n", num_recs);
+					modified = 0;
 				}
-				quitting = 1;
 				break;
 			case 'q':
-				quitting = boolprompt(
+				quitting = !modified || boolprompt(
 					"Quit without saving", 0);
 				break;
 			default:
 				badkey = 1;
 		}
-	} while (quitting == 0);
+	} while (!quitting);
 
 	if (recs != NULL) {
 		for (i=0; i<num_recs; ++i) free(recs[i]);
