@@ -1,4 +1,4 @@
-/* $Id: screen.h,v 613.0 2003/12/15 16:37:03 ajc Exp $ */
+/* $Id: screen.h,v 620.0 2004/03/27 02:33:32 ajc Exp $ */
 
 /* client code may need the ERR define: */
 
@@ -31,7 +31,7 @@ int scr_color(int colornum);
 void scr_flush(void);
 void err_flush(void);
 void sln_flush(void);
-int scr_set_windowsize(void);
+int scr_set_windowsize(CtdlIPC* ipc);
 void windows_new(void);
 void windows_delete(void);
 int scr_blockread(void);
