@@ -1,5 +1,5 @@
 /* 
- * $Id: serv_calendar.c,v 613.0 2003/12/15 16:37:03 ajc Exp $ 
+ * $Id: serv_calendar.c,v 620.0 2004/03/27 02:33:32 ajc Exp $ 
  *
  * This module implements iCalendar object processing and the Calendar>
  * room on a Citadel/UX server.  It handles iCalendar objects using the
@@ -10,6 +10,7 @@
 #define PRODID "-//Citadel//NONSGML Citadel Calendar//EN"
 
 #include "sysdep.h"
+#include <stdlib.h>
 #include <unistd.h>
 #include <sys/types.h>
 #include <limits.h>
@@ -52,7 +53,7 @@ icalcomponent *icalcomponent_new_citadel_vcalendar(void) {
 
 	encaps = icalcomponent_new_vcalendar();
 	if (encaps == NULL) {
-		lprintf(3, "Error at %s:%d - could not allocate component!\n",
+		lprintf(CTDL_CRIT, "Error at %s:%d - could not allocate component!\n",
 			__FILE__, __LINE__);
 		return NULL;
 	}
@@ -209,13 +210,13 @@ void ical_send_a_reply(icalcomponent *request, char *action) {
 	strcpy(summary_string, "Calendar item");
 
 	if (request == NULL) {
-		lprintf(3, "ERROR: trying to reply to NULL event?\n");
+		lprintf(CTDL_ERR, "ERROR: trying to reply to NULL event?\n");
 		return;
 	}
 
 	the_reply = icalcomponent_new_clone(request);
 	if (the_reply == NULL) {
-		lprintf(3, "ERROR: cannot clone request\n");
+		lprintf(CTDL_ERR, "ERROR: cannot clone request\n");
 		return;
 	}
 
@@ -243,7 +244,7 @@ void ical_send_a_reply(icalcomponent *request, char *action) {
 							if (me_attend) icalproperty_free(me_attend);
 							me_attend = icalproperty_new_clone(attendee);
 						}
-						phree(recp);
+						free(recp);
 					}
 				}
 			}
@@ -302,11 +303,11 @@ void ical_send_a_reply(icalcomponent *request, char *action) {
 	}
 
 	/* Now generate the reply message and send it out. */
-	serialized_reply = strdoop(icalcomponent_as_ical_string(the_reply));
+	serialized_reply = strdup(icalcomponent_as_ical_string(the_reply));
 	icalcomponent_free(the_reply);	/* don't need this anymore */
 	if (serialized_reply == NULL) return;
 
-	reply_message_text = mallok(strlen(serialized_reply) + SIZ);
+	reply_message_text = malloc(strlen(serialized_reply) + SIZ);
 	if (reply_message_text != NULL) {
 		sprintf(reply_message_text,
 			"Content-type: text/calendar\r\n\r\n%s\r\n",
@@ -325,7 +326,7 @@ void ical_send_a_reply(icalcomponent *request, char *action) {
 			CtdlFreeMessage(msg);
 		}
 	}
-	phree(serialized_reply);
+	free(serialized_reply);
 }
 
 
@@ -388,7 +389,7 @@ void ical_respond(long msgnum, char *partnum, char *action) {
 	msg = CtdlFetchMessage(msgnum);
 	if (msg == NULL) {
 		cprintf("%d Message %ld not found.\n",
-			ERROR+ILLEGAL_VALUE,
+			ERROR + ILLEGAL_VALUE,
 			(long)msgnum
 		);
 		return;
@@ -435,7 +436,7 @@ void ical_respond(long msgnum, char *partnum, char *action) {
 		return;
 	}
 	else {
-		cprintf("%d No calendar object found\n", ERROR);
+		cprintf("%d No calendar object found\n", ERROR + ROOM_NOT_FOUND);
 		return;
 	}
 
@@ -618,13 +619,13 @@ int ical_update_my_calendar_with_reply(icalcomponent *cal) {
 	/* Figure out just what event it is we're dealing with */
 	strcpy(uid, "--==<< InVaLiD uId >>==--");
 	ical_learn_uid_of_reply(uid, cal);
-	lprintf(9, "UID of event being replied to is <%s>\n", uid);
+	lprintf(CTDL_DEBUG, "UID of event being replied to is <%s>\n", uid);
 
 	strcpy(hold_rm, CC->room.QRname);	/* save current room */
 
 	if (getroom(&CC->room, USERCALENDARROOM) != 0) {
 		getroom(&CC->room, hold_rm);
-		lprintf(3, "cannot get user calendar room\n");
+		lprintf(CTDL_CRIT, "cannot get user calendar room\n");
 		return(2);
 	}
 
@@ -635,15 +636,15 @@ int ical_update_my_calendar_with_reply(icalcomponent *cal) {
 	 * the event, this will work.
 	 */
 	template = (struct CtdlMessage *)
-		mallok(sizeof(struct CtdlMessage));
+		malloc(sizeof(struct CtdlMessage));
 	memset(template, 0, sizeof(struct CtdlMessage));
-	template->cm_fields['E'] = strdoop(uid);
+	template->cm_fields['E'] = strdup(uid);
 	CtdlForEachMessage(MSGS_ALL, 0, "text/calendar",
 		template, ical_hunt_for_event_to_update, &msgnum_being_replaced);
 	CtdlFreeMessage(template);
 	getroom(&CC->room, hold_rm);	/* return to saved room */
 
-	lprintf(9, "msgnum_being_replaced == %ld\n", msgnum_being_replaced);
+	lprintf(CTDL_DEBUG, "msgnum_being_replaced == %ld\n", msgnum_being_replaced);
 	if (msgnum_being_replaced == 0) {
 		return(1);			/* no calendar event found */
 	}
@@ -670,7 +671,7 @@ int ical_update_my_calendar_with_reply(icalcomponent *cal) {
 
 	original_event = oec.c;
 	if (original_event == NULL) {
-		lprintf(3, "ERROR: Original_component is NULL.\n");
+		lprintf(CTDL_ERR, "ERROR: Original_component is NULL.\n");
 		return(2);
 	}
 
@@ -678,13 +679,13 @@ int ical_update_my_calendar_with_reply(icalcomponent *cal) {
 	ical_merge_attendee_reply(original_event, cal);
 
 	/* Serialize it */
-	serialized_event = strdoop(icalcomponent_as_ical_string(original_event));
+	serialized_event = strdup(icalcomponent_as_ical_string(original_event));
 	icalcomponent_free(original_event);	/* Don't need this anymore. */
 	if (serialized_event == NULL) return(2);
 
 	MailboxName(roomname, sizeof roomname, &CC->user, USERCALENDARROOM);
 
-	message_text = mallok(strlen(serialized_event) + SIZ);
+	message_text = malloc(strlen(serialized_event) + SIZ);
 	if (message_text != NULL) {
 		sprintf(message_text,
 			"Content-type: text/calendar\r\n\r\n%s\r\n",
@@ -706,7 +707,7 @@ int ical_update_my_calendar_with_reply(icalcomponent *cal) {
 			CIT_ICAL->avoid_sending_invitations = 0;
 		}
 	}
-	phree(serialized_event);
+	free(serialized_event);
 	return(0);
 }
 
@@ -734,7 +735,7 @@ void ical_handle_rsvp(long msgnum, char *partnum, char *action) {
 	msg = CtdlFetchMessage(msgnum);
 	if (msg == NULL) {
 		cprintf("%d Message %ld not found.\n",
-			ERROR+ILLEGAL_VALUE,
+			ERROR + ILLEGAL_VALUE,
 			(long)msgnum
 		);
 		return;
@@ -790,7 +791,7 @@ void ical_handle_rsvp(long msgnum, char *partnum, char *action) {
 		return;
 	}
 	else {
-		cprintf("%d No calendar object found\n", ERROR);
+		cprintf("%d No calendar object found\n", ERROR + ROOM_NOT_FOUND);
 		return;
 	}
 
@@ -975,7 +976,7 @@ void ical_hunt_for_conflicts(icalcomponent *cal) {
 
 	if (getroom(&CC->room, USERCALENDARROOM) != 0) {
 		getroom(&CC->room, hold_rm);
-		cprintf("%d You do not have a calendar.\n", ERROR);
+		cprintf("%d You do not have a calendar.\n", ERROR + ROOM_NOT_FOUND);
 		return;
 	}
 
@@ -1004,7 +1005,7 @@ void ical_conflicts(long msgnum, char *partnum) {
 	msg = CtdlFetchMessage(msgnum);
 	if (msg == NULL) {
 		cprintf("%d Message %ld not found.\n",
-			ERROR+ILLEGAL_VALUE,
+			ERROR + ILLEGAL_VALUE,
 			(long)msgnum
 		);
 		return;
@@ -1028,7 +1029,7 @@ void ical_conflicts(long msgnum, char *partnum) {
 		return;
 	}
 	else {
-		cprintf("%d No calendar object found\n", ERROR);
+		cprintf("%d No calendar object found\n", ERROR + ROOM_NOT_FOUND);
 		return;
 	}
 
@@ -1152,23 +1153,23 @@ void ical_freebusy(char *who) {
 	strcpy(hold_rm, CC->room.QRname);	/* save current room */
 
 	if (getroom(&CC->room, calendar_room_name) != 0) {
-		cprintf("%d Cannot open calendar\n", ERROR+ROOM_NOT_FOUND);
+		cprintf("%d Cannot open calendar\n", ERROR + ROOM_NOT_FOUND);
 		getroom(&CC->room, hold_rm);
 		return;
 	}
 
 	/* Create a VFREEBUSY subcomponent */
-	lprintf(9, "Creating VFREEBUSY component\n");
+	lprintf(CTDL_DEBUG, "Creating VFREEBUSY component\n");
 	fb = icalcomponent_new_vfreebusy();
 	if (fb == NULL) {
 		cprintf("%d Internal error: cannot allocate memory.\n",
-			ERROR+INTERNAL_ERROR);
+			ERROR + INTERNAL_ERROR);
 		icalcomponent_free(encaps);
 		getroom(&CC->room, hold_rm);
 		return;
 	}
 
-	lprintf(9, "Adding busy time from events\n");
+	lprintf(CTDL_DEBUG, "Adding busy time from events\n");
 	CtdlForEachMessage(MSGS_ALL, 0, "text/calendar",
 		NULL, ical_freebusy_backend, (void *)fb
 	);
@@ -1184,29 +1185,29 @@ void ical_freebusy(char *who) {
 	 */
 
 	/* Put the freebusy component into the calendar component */
-	lprintf(9, "Encapsulating\n");
+	lprintf(CTDL_DEBUG, "Encapsulating\n");
 	encaps = ical_encapsulate_subcomponent(fb);
 	if (encaps == NULL) {
 		icalcomponent_free(fb);
 		cprintf("%d Internal error: cannot allocate memory.\n",
-			ERROR+INTERNAL_ERROR);
+			ERROR + INTERNAL_ERROR);
 		getroom(&CC->room, hold_rm);
 		return;
 	}
 
 	/* Set the method to PUBLISH */
-	lprintf(9, "Setting method\n");
+	lprintf(CTDL_DEBUG, "Setting method\n");
 	icalcomponent_set_method(encaps, ICAL_METHOD_PUBLISH);
 
 	/* Serialize it */
-	lprintf(9, "Serializing\n");
-	serialized_request = strdoop(icalcomponent_as_ical_string(encaps));
+	lprintf(CTDL_DEBUG, "Serializing\n");
+	serialized_request = strdup(icalcomponent_as_ical_string(encaps));
 	icalcomponent_free(encaps);	/* Don't need this anymore. */
 
 	cprintf("%d Here is the free/busy data:\n", LISTING_FOLLOWS);
 	if (serialized_request != NULL) {
 		client_write(serialized_request, strlen(serialized_request));
-		phree(serialized_request);
+		free(serialized_request);
 	}
 	cprintf("\n000\n");
 
@@ -1232,20 +1233,28 @@ void cmd_ical(char *argbuf)
 
 	/* Allow "test" and "freebusy" subcommands without logging in. */
 
-	if (!strcmp(subcmd, "test")) {
+	if (!strcasecmp(subcmd, "test")) {
 		cprintf("%d This server supports calendaring\n", CIT_OK);
 		return;
 	}
 
-	if (!strcmp(subcmd, "freebusy")) {
+	if (!strcasecmp(subcmd, "freebusy")) {
 		extract(who, argbuf, 1);
 		ical_freebusy(who);
 		return;
 	}
 
+	if (!strcasecmp(subcmd, "sgi")) {
+		CIT_ICAL->server_generated_invitations =
+			(extract_int(argbuf, 1) ? 1 : 0) ;
+		cprintf("%d %d\n",
+			CIT_OK, CIT_ICAL->server_generated_invitations);
+		return;
+	}
+
 	if (CtdlAccessCheck(ac_logged_in)) return;
 
-	if (!strcmp(subcmd, "respond")) {
+	if (!strcasecmp(subcmd, "respond")) {
 		msgnum = extract_long(argbuf, 1);
 		extract(partnum, argbuf, 2);
 		extract(action, argbuf, 3);
@@ -1253,7 +1262,7 @@ void cmd_ical(char *argbuf)
 		return;
 	}
 
-	if (!strcmp(subcmd, "handle_rsvp")) {
+	if (!strcasecmp(subcmd, "handle_rsvp")) {
 		msgnum = extract_long(argbuf, 1);
 		extract(partnum, argbuf, 2);
 		extract(action, argbuf, 3);
@@ -1261,14 +1270,14 @@ void cmd_ical(char *argbuf)
 		return;
 	}
 
-	if (!strcmp(subcmd, "conflicts")) {
+	if (!strcasecmp(subcmd, "conflicts")) {
 		msgnum = extract_long(argbuf, 1);
 		extract(partnum, argbuf, 2);
 		ical_conflicts(msgnum, partnum);
 		return;
 	}
 
-	cprintf("%d Invalid subcommand\n", ERROR+CMD_NOT_SUPPORTED);
+	cprintf("%d Invalid subcommand\n", ERROR + CMD_NOT_SUPPORTED);
 }
 
 
@@ -1286,7 +1295,7 @@ void ical_create_room(void)
 
 	/* Set expiration policy to manual; otherwise objects will be lost! */
 	if (lgetroom(&qr, USERCALENDARROOM)) {
-		lprintf(3, "Couldn't get the user calendar room!\n");
+		lprintf(CTDL_CRIT, "Couldn't get the user calendar room!\n");
 		return;
 	}
 	qr.QRep.expire_mode = EXPIRE_MANUAL;
@@ -1303,7 +1312,7 @@ void ical_create_room(void)
 
 	/* Set expiration policy to manual; otherwise objects will be lost! */
 	if (lgetroom(&qr, USERTASKSROOM)) {
-		lprintf(3, "Couldn't get the user calendar room!\n");
+		lprintf(CTDL_CRIT, "Couldn't get the user calendar room!\n");
 		return;
 	}
 	qr.QRep.expire_mode = EXPIRE_MANUAL;
@@ -1338,7 +1347,7 @@ void ical_send_out_invitations(icalcomponent *cal) {
 	icalproperty *summary = NULL;
 
 	if (cal == NULL) {
-		lprintf(3, "ERROR: trying to reply to NULL event?\n");
+		lprintf(CTDL_ERR, "ERROR: trying to reply to NULL event?\n");
 		return;
 	}
 
@@ -1356,7 +1365,7 @@ void ical_send_out_invitations(icalcomponent *cal) {
 	/* Clone the event */
 	the_request = icalcomponent_new_clone(cal);
 	if (the_request == NULL) {
-		lprintf(3, "ERROR: cannot clone calendar object\n");
+		lprintf(CTDL_ERR, "ERROR: cannot clone calendar object\n");
 		return;
 	}
 
@@ -1379,17 +1388,20 @@ void ical_send_out_invitations(icalcomponent *cal) {
 			strcpy(this_attendee, icalproperty_get_attendee(attendee) );
 			if (!strncasecmp(this_attendee, "MAILTO:", 7)) {
 				strcpy(this_attendee, &this_attendee[7]);
-				snprintf(&attendees_string[strlen(attendees_string)],
-					sizeof(attendees_string) - strlen(attendees_string),
-					"%s, ",
-					this_attendee
-				);
-				++num_attendees;
+
+				if (!CtdlIsMe(this_attendee)) {	/* don't send an invitation to myself! */
+					snprintf(&attendees_string[strlen(attendees_string)],
+						sizeof(attendees_string) - strlen(attendees_string),
+						"%s, ",
+						this_attendee
+					);
+					++num_attendees;
+				}
 			}
 		}
 	}
 
-	lprintf(9, "<%d> attendees: <%s>\n", num_attendees, attendees_string);
+	lprintf(CTDL_DEBUG, "<%d> attendees: <%s>\n", num_attendees, attendees_string);
 
 	/* If there are no attendees, there are no invitations to send, so...
 	 * don't bother putting one together!  Punch out, Maverick!
@@ -1402,7 +1414,7 @@ void ical_send_out_invitations(icalcomponent *cal) {
 	/* Encapsulate the VEVENT component into a complete VCALENDAR */
 	encaps = icalcomponent_new_vcalendar();
 	if (encaps == NULL) {
-		lprintf(3, "Error at %s:%d - could not allocate component!\n",
+		lprintf(CTDL_DEBUG, "Error at %s:%d - could not allocate component!\n",
 			__FILE__, __LINE__);
 		icalcomponent_free(the_request);
 		return;
@@ -1427,11 +1439,11 @@ void ical_send_out_invitations(icalcomponent *cal) {
 	icalcomponent_add_component(encaps, the_request);
 
 	/* Serialize it */
-	serialized_request = strdoop(icalcomponent_as_ical_string(encaps));
+	serialized_request = strdup(icalcomponent_as_ical_string(encaps));
 	icalcomponent_free(encaps);	/* Don't need this anymore. */
 	if (serialized_request == NULL) return;
 
-	request_message_text = mallok(strlen(serialized_request) + SIZ);
+	request_message_text = malloc(strlen(serialized_request) + SIZ);
 	if (request_message_text != NULL) {
 		sprintf(request_message_text,
 			"Content-type: text/calendar\r\n\r\n%s\r\n",
@@ -1451,7 +1463,7 @@ void ical_send_out_invitations(icalcomponent *cal) {
 			CtdlFreeMessage(msg);
 		}
 	}
-	phree(serialized_request);
+	free(serialized_request);
 }
 
 
@@ -1466,9 +1478,12 @@ void ical_saving_vevent(icalcomponent *cal) {
 	icalproperty *organizer = NULL;
 	char organizer_string[SIZ];
 
+	/* Don't send out invitations unless the client wants us to. */
+	if (CIT_ICAL->server_generated_invitations == 0) {
+		return;
+	}
+
 	/* Don't send out invitations if we've been asked not to. */
-	lprintf(9, "CIT_ICAL->avoid_sending_invitations = %d\n",
-		CIT_ICAL->avoid_sending_invitations);
 	if (CIT_ICAL->avoid_sending_invitations > 0) {
 		return;
 	}
@@ -1619,21 +1634,21 @@ int ical_obj_beforesave(struct CtdlMessage *msg)
 				);
 				if (strlen(imm.uid) > 0) {
 					if (msg->cm_fields['E'] != NULL) {
-						phree(msg->cm_fields['E']);
+						free(msg->cm_fields['E']);
 					}
-					msg->cm_fields['E'] = strdoop(imm.uid);
+					msg->cm_fields['E'] = strdup(imm.uid);
 				}
 				if (strlen(imm.subject) > 0) {
 					if (msg->cm_fields['U'] != NULL) {
-						phree(msg->cm_fields['U']);
+						free(msg->cm_fields['U']);
 					}
-					msg->cm_fields['U'] = strdoop(imm.subject);
+					msg->cm_fields['U'] = strdup(imm.subject);
 				}
 				if (imm.dtstart > 0) {
 					if (msg->cm_fields['T'] != NULL) {
-						phree(msg->cm_fields['T']);
+						free(msg->cm_fields['T']);
 					}
-					msg->cm_fields['T'] = strdoop("000000000000000000");
+					msg->cm_fields['T'] = strdup("000000000000000000");
 					sprintf(msg->cm_fields['T'], "%ld", imm.dtstart);
 				}
 				return 0;
@@ -1646,7 +1661,7 @@ int ical_obj_beforesave(struct CtdlMessage *msg)
 	}
 	
 	/* Oops!  No Content-Type in this message!  How'd that happen? */
-	lprintf(7, "RFC822 message with no Content-Type header!\n");
+	lprintf(CTDL_ERR, "RFC822 message with no Content-Type header!\n");
 	return 1;
 }
 
@@ -1723,7 +1738,7 @@ int ical_obj_aftersave(struct CtdlMessage *msg)
 	}
 	
 	/* Oops!  No Content-Type in this message!  How'd that happen? */
-	lprintf(7, "RFC822 message with no Content-Type header!\n");
+	lprintf(CTDL_ERR, "RFC822 message with no Content-Type header!\n");
 	return 1;
 }
 
@@ -1748,5 +1763,5 @@ char *serv_calendar_init(void)
 	CtdlRegisterProtoHook(cmd_ical, "ICAL", "Citadel iCal commands");
 	CtdlRegisterSessionHook(ical_session_startup, EVT_START);
 #endif
-	return "$Id: serv_calendar.c,v 613.0 2003/12/15 16:37:03 ajc Exp $";
+	return "$Id: serv_calendar.c,v 620.0 2004/03/27 02:33:32 ajc Exp $";
 }
