@@ -1,5 +1,5 @@
 /*
- * $Id: serv_chat.c,v 613.0 2003/12/15 16:37:03 ajc Exp $
+ * $Id: serv_chat.c,v 620.0 2004/03/27 02:33:32 ajc Exp $
  * 
  * This module handles all "real time" communication between users.  The
  * modes of communication currently supported are Chat and Paging.
@@ -32,7 +32,6 @@
 #include "citadel.h"
 #include "server.h"
 #include "serv_extensions.h"
-#include <syslog.h>
 #include "serv_chat.h"
 #include "sysdep_decls.h"
 #include "citserver.h"
@@ -50,8 +49,11 @@
 struct ChatLine *ChatQueue = NULL;
 int ChatLastMsg = 0;
 
-
-
+/*
+ * This message can be set to anything you want, but it is
+ * checked for consistency so don't move it away from here.
+ */
+#define KICKEDMSG "You have been kicked out of this room."
 
 void allwrite(char *cmdbuf, int flag, char *username)
 {
@@ -71,6 +73,8 @@ void allwrite(char *cmdbuf, int flag, char *username)
 		snprintf(bcast, sizeof bcast, "%s|%s", un, cmdbuf);
 	} else if (flag == 2) {
 		snprintf(bcast, sizeof bcast, ":|<%s whispers %s>", un, cmdbuf);
+	} else if (flag == 3) {
+		snprintf(bcast, sizeof bcast, ":|%s", KICKEDMSG);
 	}
 	if ((strcasecmp(cmdbuf, "NOOP")) && (flag != 2)) {
 		fp = fopen(CHATLOG, "a");
@@ -78,7 +82,7 @@ void allwrite(char *cmdbuf, int flag, char *username)
 			fprintf(fp, "%s\n", bcast);
 		fclose(fp);
 	}
-	clnew = (struct ChatLine *) mallok(sizeof(struct ChatLine));
+	clnew = (struct ChatLine *) malloc(sizeof(struct ChatLine));
 	memset(clnew, 0, sizeof(struct ChatLine));
 	if (clnew == NULL) {
 		fprintf(stderr, "citserver: cannot alloc chat line: %s\n",
@@ -116,7 +120,7 @@ void allwrite(char *cmdbuf, int flag, char *username)
 	while ((ChatQueue != NULL) && (now - ChatQueue->chat_time >= 120L)) {
 		clptr = ChatQueue;
 		ChatQueue = ChatQueue->next;
-		phree(clptr);
+		free(clptr);
 	}
 	end_critical_section(S_CHATQUEUE);
 }
@@ -311,6 +315,9 @@ void cmd_chat(char *argbuf)
 					cprintf(":|/whobbs (list users in chat -and- elsewhere) \n");
 					cprintf(":|/me     ('action' line, ala irc) \n");
 					cprintf(":|/msg    (send private message, ala irc) \n");
+					if (is_room_aide()) {
+						cprintf(":|/kick   (kick another user out of this room) \n");
+					}
 					cprintf(":|/quit   (return to the BBS) \n");
 					cprintf(":|\n");
 					ok_cmd = 1;
@@ -338,6 +345,21 @@ void cmd_chat(char *argbuf)
 						cprintf(":|User not found.\n");
 					cprintf("\n");
 				}
+				/* The /kick function is implemented by sending a specific
+				 * message to the kicked-out user's context.  When that message
+				 * is processed by the read loop, that context will exit.
+				 */
+				if ( (!strncasecmp(cmdbuf, "/kick ", 6)) && (is_room_aide()) ) {
+					ok_cmd = 1;
+					strptr1 = &cmdbuf[6];
+					strcat(strptr1, " ");
+					if ((t_context = find_context(&strptr1))) {
+						if (strcasecmp(CC->curr_user, t_context->curr_user))
+							allwrite(strptr1, 3, t_context->curr_user);
+					} else
+						cprintf(":|User not found.\n");
+					cprintf("\n");
+				}
 				if ((cmdbuf[0] != '/') && (strlen(cmdbuf) > 0)) {
 					ok_cmd = 1;
 					allwrite(cmdbuf, 0, NULL);
@@ -359,7 +381,22 @@ void cmd_chat(char *argbuf)
 			for (clptr = ChatQueue; clptr != NULL; clptr = clptr->next) {
 				if ((clptr->chat_seq > MyLastMsg) && ((!clptr->chat_username[0]) || (!strncasecmp(un, clptr->chat_username, 32)))) {
 					if ((!clptr->chat_room[0]) || (!strncasecmp(CC->room.QRname, clptr->chat_room, ROOMNAMELEN))) {
+						/* Output new chat data */
 						cprintf("%s\n", clptr->chat_text);
+
+						/* See if we've been force-quitted (kicked etc.) */
+						if (!strcmp(&clptr->chat_text[2], KICKEDMSG)) {
+							allwrite("<kicked out of this room>", 0, NULL);
+							cprintf("000\n");
+							CC->cs_flags = CC->cs_flags - CS_CHAT;
+
+							/* Kick user out of room */
+							CtdlInvtKick(CC->user.fullname, 0);
+
+							/* And return to the Lobby */
+							usergoto(config.c_baseroom, 0, 0, NULL, NULL);
+							return;
+						}
 					}
 				}
 			}
@@ -380,8 +417,8 @@ void delete_express_messages(void) {
 	while (CC->FirstExpressMessage != NULL) {
 		ptr = CC->FirstExpressMessage->next;
 		if (CC->FirstExpressMessage->text != NULL)
-			phree(CC->FirstExpressMessage->text);
-		phree(CC->FirstExpressMessage);
+			free(CC->FirstExpressMessage->text);
+		free(CC->FirstExpressMessage);
 		CC->FirstExpressMessage = ptr;
 		}
 	end_critical_section(S_SESSION_TABLE);
@@ -398,7 +435,7 @@ void cmd_pexp(char *argbuf)
 	struct ExpressMessage *ptr, *holdptr;
 
 	if (CC->FirstExpressMessage == NULL) {
-		cprintf("%d No express messages waiting.\n", ERROR);
+		cprintf("%d No express messages waiting.\n", ERROR + MESSAGE_NOT_FOUND);
 		return;
 	}
 	begin_critical_section(S_SESSION_TABLE);
@@ -421,8 +458,8 @@ void cmd_pexp(char *argbuf)
 			memfmout(80, ptr->text, 0, "\n");
 
 		holdptr = ptr->next;
-		if (ptr->text != NULL) phree(ptr->text);
-		phree(ptr);
+		if (ptr->text != NULL) free(ptr->text);
+		free(ptr);
 		ptr = holdptr;
 	}
 	cprintf("000\n");
@@ -436,7 +473,7 @@ void cmd_gexp(char *argbuf) {
 	struct ExpressMessage *ptr;
 
 	if (CC->FirstExpressMessage == NULL) {
-		cprintf("%d No express messages waiting.\n", ERROR);
+		cprintf("%d No express messages waiting.\n", ERROR + MESSAGE_NOT_FOUND);
 		return;
 	}
 
@@ -455,10 +492,10 @@ void cmd_gexp(char *argbuf) {
 	if (ptr->text != NULL) {
 		memfmout(80, ptr->text, 0, "\n");
 		if (ptr->text[strlen(ptr->text)-1] != '\n') cprintf("\n");
-		phree(ptr->text);
+		free(ptr->text);
 		}
 	cprintf("000\n");
-	phree(ptr);
+	free(ptr);
 }
 
 /*
@@ -540,7 +577,7 @@ int send_express_message(char *lun, char *x_user, char *x_msg)
 		    || (CC->user.axlevel >= 6)) ) {
 			if (do_send) {
 				newmsg = (struct ExpressMessage *)
-					mallok(sizeof (struct ExpressMessage));
+					malloc(sizeof (struct ExpressMessage));
 				memset(newmsg, 0,
 					sizeof (struct ExpressMessage));
 				time(&(newmsg->timestamp));
@@ -548,7 +585,7 @@ int send_express_message(char *lun, char *x_user, char *x_msg)
 					    sizeof newmsg->sender);
 				if (!strcasecmp(x_user, "broadcast"))
 					newmsg->flags |= EM_BROADCAST;
-				newmsg->text = strdoop(x_msg);
+				newmsg->text = strdup(x_msg);
 
 				add_xmsg_to_context(ccptr, newmsg);
 
@@ -571,16 +608,16 @@ int send_express_message(char *lun, char *x_user, char *x_msg)
 	/* Log the page to disk if configured to do so  */
 	if ( (do_send) && (message_sent) ) {
 
-		logmsg = mallok(sizeof(struct CtdlMessage));
+		logmsg = malloc(sizeof(struct CtdlMessage));
 		memset(logmsg, 0, sizeof(struct CtdlMessage));
 		logmsg->cm_magic = CTDLMESSAGE_MAGIC;
 		logmsg->cm_anon_type = MES_NORMAL;
 		logmsg->cm_format_type = 0;
-		logmsg->cm_fields['A'] = strdoop(lun);
-		logmsg->cm_fields['N'] = strdoop(NODENAME);
-		logmsg->cm_fields['O'] = strdoop(PAGELOGROOM);
-		logmsg->cm_fields['R'] = strdoop(x_user);
-		logmsg->cm_fields['M'] = strdoop(x_msg);
+		logmsg->cm_fields['A'] = strdup(lun);
+		logmsg->cm_fields['N'] = strdup(NODENAME);
+		logmsg->cm_fields['O'] = strdup(PAGELOGROOM);
+		logmsg->cm_fields['R'] = strdup(x_user);
+		logmsg->cm_fields['M'] = strdup(x_msg);
 
 
 		/* Save a copy of the message in the sender's log room,
@@ -604,7 +641,7 @@ int send_express_message(char *lun, char *x_user, char *x_msg)
 			create_room(sl->roomname, 5, "", 0, 1, 1);
 			CtdlSaveMsgPointerInRoom(sl->roomname, msgnum, 0);
 			sptr = sl->next;
-			phree(sl);
+			free(sl);
 			sl = sptr;
 		}
 
@@ -639,7 +676,7 @@ void cmd_sexp(char *argbuf)
 	extract(x_msg, argbuf, 1);
 
 	if (!x_user[0]) {
-		cprintf("%d You were not previously paged.\n", ERROR);
+		cprintf("%d You were not previously paged.\n", ERROR + NO_SUCH_USER);
 		return;
 	}
 	if ((!strcasecmp(x_user, "broadcast")) && (CC->user.axlevel < 6)) {
@@ -653,15 +690,15 @@ void cmd_sexp(char *argbuf)
 		if (message_sent == 0) {
 			cprintf("%d '%s' is not logged in "
 				"or is not accepting pages.\n",
-				ERROR, x_user);
+				ERROR + NO_SUCH_USER, x_user);
 			return;
 		}
 		cprintf("%d Transmit message (will deliver to %d users)\n",
 			SEND_LISTING, message_sent);
-		x_big_msgbuf = mallok(SIZ);
+		x_big_msgbuf = malloc(SIZ);
 		memset(x_big_msgbuf, 0, SIZ);
 		while (client_gets(x_msg), strcmp(x_msg, "000")) {
-			x_big_msgbuf = reallok(x_big_msgbuf,
+			x_big_msgbuf = realloc(x_big_msgbuf,
 			       strlen(x_big_msgbuf) + strlen(x_msg) + 4);
 			if (strlen(x_big_msgbuf) > 0)
 			   if (x_big_msgbuf[strlen(x_big_msgbuf)] != '\n')
@@ -669,7 +706,7 @@ void cmd_sexp(char *argbuf)
 			strcat(x_big_msgbuf, x_msg);
 		}
 		PerformXmsgHooks(lun, x_user, x_big_msgbuf);
-		phree(x_big_msgbuf);
+		free(x_big_msgbuf);
 
 		/* This loop handles inline pages */
 	} else {
@@ -686,7 +723,7 @@ void cmd_sexp(char *argbuf)
 		} else {
 			cprintf("%d '%s' is not logged in "
 				"or is not accepting pages.\n",
-				ERROR, x_user);
+				ERROR + NO_SUCH_USER, x_user);
 		}
 
 
@@ -730,14 +767,14 @@ void cmd_reqt(char *argbuf) {
 		if ((ccptr->cs_pid == which_session) || (which_session == 0)) {
 
 			newmsg = (struct ExpressMessage *)
-				mallok(sizeof (struct ExpressMessage));
+				malloc(sizeof (struct ExpressMessage));
 			memset(newmsg, 0,
 				sizeof (struct ExpressMessage));
 			time(&(newmsg->timestamp));
 			safestrncpy(newmsg->sender, CC->user.fullname,
 				    sizeof newmsg->sender);
 			newmsg->flags |= EM_GO_AWAY;
-			newmsg->text = strdoop("Automatic logoff requested.");
+			newmsg->text = strdup("Automatic logoff requested.");
 
 			add_xmsg_to_context(ccptr, newmsg);
 			++sessions;
@@ -750,7 +787,7 @@ void cmd_reqt(char *argbuf) {
 
 
 
-char *serv_inithat_init(void)
+char *serv_chat_init(void)
 {
 	CtdlRegisterProtoHook(cmd_chat, "CHAT", "Begin real-time chat");
 	CtdlRegisterProtoHook(cmd_pexp, "PEXP", "Poll for express messages");
@@ -761,6 +798,6 @@ char *serv_inithat_init(void)
 	CtdlRegisterSessionHook(cmd_gexp_async, EVT_CMD);
 	CtdlRegisterSessionHook(delete_express_messages, EVT_STOP);
 	CtdlRegisterXmsgHook(send_express_message, XMSG_PRI_LOCAL);
-	return "$Id: serv_chat.c,v 613.0 2003/12/15 16:37:03 ajc Exp $";
+	return "$Id: serv_chat.c,v 620.0 2004/03/27 02:33:32 ajc Exp $";
 }
 
