@@ -1,5 +1,5 @@
 /*
- * $Id: serv_expire.c,v 613.0 2003/12/15 16:37:03 ajc Exp $
+ * $Id: serv_expire.c,v 620.0 2004/03/27 02:33:32 ajc Exp $
  *
  * This module handles the expiry of old messages and the purging of old users.
  *
@@ -51,7 +51,6 @@
 #include <limits.h>
 #include "citadel.h"
 #include "server.h"
-#include <syslog.h>
 #include "sysdep_decls.h"
 #include "citserver.h"
 #include "support.h"
@@ -141,7 +140,7 @@ void GatherPurgeMessages(struct ctdlroom *qrbuf, void *data) {
         cdbfr = cdb_fetch(CDB_MSGLISTS, &qrbuf->QRnumber, sizeof(long));
 
         if (cdbfr != NULL) {
-        	msglist = mallok(cdbfr->len);
+        	msglist = malloc(cdbfr->len);
         	memcpy(msglist, cdbfr->ptr, cdbfr->len);
         	num_msgs = cdbfr->len / sizeof(long);
         	cdb_free(cdbfr);
@@ -149,7 +148,7 @@ void GatherPurgeMessages(struct ctdlroom *qrbuf, void *data) {
 
 	/* Nothing to do if there aren't any messages */
 	if (num_msgs == 0) {
-		if (msglist != NULL) phree(msglist);
+		if (msglist != NULL) free(msglist);
 		return;
 	}
 
@@ -184,7 +183,7 @@ void GatherPurgeMessages(struct ctdlroom *qrbuf, void *data) {
 		}
 	}
 
-	if (msglist != NULL) phree(msglist);
+	if (msglist != NULL) free(msglist);
 }
 
 
@@ -217,12 +216,12 @@ void DoPurgeMessages(FILE *purgelist) {
 void PurgeMessages(void) {
 	FILE *purgelist;
 
-	lprintf(5, "PurgeMessages() called\n");
+	lprintf(CTDL_DEBUG, "PurgeMessages() called\n");
 	messages_purged = 0;
 
 	purgelist = tmpfile();
 	if (purgelist == NULL) {
-		lprintf(3, "Can't create purgelist temp file: %s\n",
+		lprintf(CTDL_CRIT, "Can't create purgelist temp file: %s\n",
 			strerror(errno));
 		return;
 	}
@@ -236,7 +235,7 @@ void PurgeMessages(void) {
 void AddValidUser(struct ctdluser *usbuf, void *data) {
 	struct ValidUser *vuptr;
 
-	vuptr = (struct ValidUser *)mallok(sizeof(struct ValidUser));
+	vuptr = (struct ValidUser *)malloc(sizeof(struct ValidUser));
 	vuptr->next = ValidUserList;
 	vuptr->vu_usernum = usbuf->usernum;
 	ValidUserList = vuptr;
@@ -245,7 +244,7 @@ void AddValidUser(struct ctdluser *usbuf, void *data) {
 void AddValidRoom(struct ctdlroom *qrbuf, void *data) {
 	struct ValidRoom *vrptr;
 
-	vrptr = (struct ValidRoom *)mallok(sizeof(struct ValidRoom));
+	vrptr = (struct ValidRoom *)malloc(sizeof(struct ValidRoom));
 	vrptr->next = ValidRoomList;
 	vrptr->vr_roomnum = qrbuf->QRnumber;
 	vrptr->vr_roomgen = qrbuf->QRgen;
@@ -289,12 +288,12 @@ void DoPurgeRooms(struct ctdlroom *qrbuf, void *data) {
 		age = time(NULL) - (qrbuf->QRmtime);
 		purge_secs = (time_t)config.c_roompurge * (time_t)86400;
 		if (purge_secs <= (time_t)0) return;
-		lprintf(9, "<%s> is <%ld> seconds old\n", qrbuf->QRname, (long)age);
+		lprintf(CTDL_DEBUG, "<%s> is <%ld> seconds old\n", qrbuf->QRname, (long)age);
 		if (age > purge_secs) do_purge = 1;
 	} /* !QR_MAILBOX */
 
 	if (do_purge) {
-		pptr = (struct PurgeList *) mallok(sizeof(struct PurgeList));
+		pptr = (struct PurgeList *) malloc(sizeof(struct PurgeList));
 		pptr->next = RoomPurgeList;
 		strcpy(pptr->name, qrbuf->QRname);
 		RoomPurgeList = pptr;
@@ -311,7 +310,7 @@ int PurgeRooms(void) {
 	struct ValidUser *vuptr;
 	char *transcript = NULL;
 
-	lprintf(5, "PurgeRooms() called\n");
+	lprintf(CTDL_DEBUG, "PurgeRooms() called\n");
 
 
 	/* Load up a table full of valid user numbers so we can delete
@@ -324,35 +323,61 @@ int PurgeRooms(void) {
 	/* Free the valid user list */
 	while (ValidUserList != NULL) {
 		vuptr = ValidUserList->next;
-		phree(ValidUserList);
+		free(ValidUserList);
 		ValidUserList = vuptr;
 	}
 
 
-	transcript = mallok(SIZ);
+	transcript = malloc(SIZ);
 	strcpy(transcript, "The following rooms have been auto-purged:\n");
 
 	while (RoomPurgeList != NULL) {
 		if (getroom(&qrbuf, RoomPurgeList->name) == 0) {
-			transcript=reallok(transcript, strlen(transcript)+SIZ);
+			transcript=realloc(transcript, strlen(transcript)+SIZ);
 			snprintf(&transcript[strlen(transcript)], SIZ, " %s\n",
 				qrbuf.QRname);
 			delete_room(&qrbuf);
 		}
 		pptr = RoomPurgeList->next;
-		phree(RoomPurgeList);
+		free(RoomPurgeList);
 		RoomPurgeList = pptr;
 		++num_rooms_purged;
 	}
 
 	if (num_rooms_purged > 0) aide_message(transcript);
-	phree(transcript);
+	free(transcript);
 
-	lprintf(5, "Purged %d rooms.\n", num_rooms_purged);
+	lprintf(CTDL_DEBUG, "Purged %d rooms.\n", num_rooms_purged);
 	return(num_rooms_purged);
 }
 
 
+/*
+ * Back end function to check user accounts for associated Unix accounts
+ * which no longer exist.
+ */
+void do_uid_user_purge(struct ctdluser *us, void *data) {
+#ifdef ENABLE_AUTOLOGIN
+	struct PurgeList *pptr;
+
+	if ((us->uid != (-1)) && (us->uid != BBSUID)) {
+		if (getpwuid(us->uid) == NULL) {
+			pptr = (struct PurgeList *)
+				malloc(sizeof(struct PurgeList));
+			pptr->next = UserPurgeList;
+			strcpy(pptr->name, us->fullname);
+			UserPurgeList = pptr;
+		}
+	}
+
+#endif /* ENABLE_AUTOLOGIN */
+}
+
+
+
+/*
+ * Back end function to check user accounts for expiration.
+ */
 void do_user_purge(struct ctdluser *us, void *data) {
 	int purge;
 	time_t now;
@@ -366,7 +391,7 @@ void do_user_purge(struct ctdluser *us, void *data) {
 	MailboxName(mailboxname, us, MAILROOM);
 	create_room(mailboxname, 4, "", 0, 1, 1);
 	if (getroom(&qrbuf, mailboxname) != 0) return;
-	lprintf(9, "Got %s\n", qrbuf.QRname);
+	lprintf(CTDL_DEBUG, "Got %s\n", qrbuf.QRname);
 	 */
 
 
@@ -408,7 +433,7 @@ void do_user_purge(struct ctdluser *us, void *data) {
 	if (us->timescalled == 0) purge = 1;
 
 	if (purge == 1) {
-		pptr = (struct PurgeList *) mallok(sizeof(struct PurgeList));
+		pptr = (struct PurgeList *) malloc(sizeof(struct PurgeList));
 		pptr->next = UserPurgeList;
 		strcpy(pptr->name, us->fullname);
 		UserPurgeList = pptr;
@@ -423,29 +448,30 @@ int PurgeUsers(void) {
 	int num_users_purged = 0;
 	char *transcript = NULL;
 
-	lprintf(5, "PurgeUsers() called\n");
+	lprintf(CTDL_DEBUG, "PurgeUsers() called\n");
 	if (config.c_userpurge > 0) {
 		ForEachUser(do_user_purge, NULL);
 	}
+	ForEachUser(do_uid_user_purge, NULL);
 
-	transcript = mallok(SIZ);
+	transcript = malloc(SIZ);
 	strcpy(transcript, "The following users have been auto-purged:\n");
 
 	while (UserPurgeList != NULL) {
-		transcript=reallok(transcript, strlen(transcript)+SIZ);
+		transcript=realloc(transcript, strlen(transcript)+SIZ);
 		snprintf(&transcript[strlen(transcript)], SIZ, " %s\n",
 			UserPurgeList->name);
 		purge_user(UserPurgeList->name);
 		pptr = UserPurgeList->next;
-		phree(UserPurgeList);
+		free(UserPurgeList);
 		UserPurgeList = pptr;
 		++num_users_purged;
 	}
 
 	if (num_users_purged > 0) aide_message(transcript);
-	phree(transcript);
+	free(transcript);
 
-	lprintf(5, "Purged %d users.\n", num_users_purged);
+	lprintf(CTDL_DEBUG, "Purged %d users.\n", num_users_purged);
 	return(num_users_purged);
 }
 
@@ -508,7 +534,7 @@ int PurgeVisits(void) {
 		/* Put the record on the purge list if it's dead */
 		if ((RoomIsValid==0) || (UserIsValid==0)) {
 			vptr = (struct VPurgeList *)
-				mallok(sizeof(struct VPurgeList));
+				malloc(sizeof(struct VPurgeList));
 			vptr->next = VisitPurgeList;
 			vptr->vp_roomnum = vbuf.v_roomnum;
 			vptr->vp_roomgen = vbuf.v_roomgen;
@@ -521,14 +547,14 @@ int PurgeVisits(void) {
 	/* Free the valid room/gen combination list */
 	while (ValidRoomList != NULL) {
 		vrptr = ValidRoomList->next;
-		phree(ValidRoomList);
+		free(ValidRoomList);
 		ValidRoomList = vrptr;
 	}
 
 	/* Free the valid user list */
 	while (ValidUserList != NULL) {
 		vuptr = ValidUserList->next;
-		phree(ValidUserList);
+		free(ValidUserList);
 		ValidUserList = vuptr;
 	}
 
@@ -540,7 +566,7 @@ int PurgeVisits(void) {
 				VisitPurgeList->vp_usernum);
 		cdb_delete(CDB_VISIT, IndexBuf, IndexLen);
 		vptr = VisitPurgeList->next;
-		phree(VisitPurgeList);
+		free(VisitPurgeList);
 		VisitPurgeList = vptr;
 		++purged;
 	}
@@ -560,7 +586,7 @@ int PurgeUseTable(void) {
 	struct UPurgeList *uptr; 
 
 	/* Phase 1: traverse through the table, discovering old records... */
-	lprintf(9, "Purge use table: phase 1\n");
+	lprintf(CTDL_DEBUG, "Purge use table: phase 1\n");
 	cdb_rewind(CDB_USETABLE);
 	while(cdbut = cdb_next_item(CDB_USETABLE), cdbut != NULL) {
 
@@ -570,7 +596,7 @@ int PurgeUseTable(void) {
                 cdb_free(cdbut);
 
 		if ( (time(NULL) - ut.ut_timestamp) > USETABLE_RETAIN ) {
-			uptr = (struct UPurgeList *) mallok(sizeof(struct UPurgeList));
+			uptr = (struct UPurgeList *) malloc(sizeof(struct UPurgeList));
 			if (uptr != NULL) {
 				uptr->next = ul;
 				safestrncpy(uptr->up_key, ut.ut_msgid, SIZ);
@@ -582,15 +608,15 @@ int PurgeUseTable(void) {
 	}
 
 	/* Phase 2: delete the records */
-	lprintf(9, "Purge use table: phase 2\n");
+	lprintf(CTDL_DEBUG, "Purge use table: phase 2\n");
 	while (ul != NULL) {
 		cdb_delete(CDB_USETABLE, ul->up_key, strlen(ul->up_key));
 		uptr = ul->next;
-		phree(ul);
+		free(ul);
 		ul = uptr;
 	}
 
-	lprintf(9, "Purge use table: finished (purged %d records)\n", purged);
+	lprintf(CTDL_DEBUG, "Purge use table: finished (purged %d records)\n", purged);
 	return(purged);
 }
 
@@ -610,24 +636,24 @@ void purge_databases(void) {
 	if (tm.tm_hour != config.c_purge_hour) return;
 	if ((now - last_purge) < 43200) return;
 
-	lprintf(3, "Auto-purger: starting.\n");
+	lprintf(CTDL_INFO, "Auto-purger: starting.\n");
 
 	retval = PurgeUsers();
-	lprintf(3, "Purged %d users.\n", retval);
+	lprintf(CTDL_NOTICE, "Purged %d users.\n", retval);
 
 	PurgeMessages();
-	lprintf(3, "Expired %d messages.\n", messages_purged);
+	lprintf(CTDL_NOTICE, "Expired %d messages.\n", messages_purged);
 
 	retval = PurgeRooms();
-	lprintf(3, "Expired %d rooms.\n", retval);
+	lprintf(CTDL_NOTICE, "Expired %d rooms.\n", retval);
 
 	retval = PurgeVisits();
-	lprintf(3, "Purged %d visits.\n", retval);
+	lprintf(CTDL_NOTICE, "Purged %d visits.\n", retval);
 
 	retval = PurgeUseTable();
-	lprintf(3, "Purged %d entries from the use table.\n", retval);
+	lprintf(CTDL_NOTICE, "Purged %d entries from the use table.\n", retval);
 
-	lprintf(3, "Auto-purger: finished.\n");
+	lprintf(CTDL_INFO, "Auto-purger: finished.\n");
 
 	last_purge = now;	/* So we don't do it again soon */
 }
@@ -638,7 +664,7 @@ void purge_databases(void) {
 void do_fsck_msg(long msgnum, void *userdata) {
 	struct ctdlroomref *ptr;
 
-	ptr = (struct ctdlroomref *)mallok(sizeof(struct ctdlroomref));
+	ptr = (struct ctdlroomref *)malloc(sizeof(struct ctdlroomref));
 	ptr->next = rr;
 	ptr->msgnum = msgnum;
 	rr = ptr;
@@ -664,7 +690,7 @@ void cmd_fsck(char *argbuf) {
 
 	/* Lame way of checking whether anyone else is doing this now */
 	if (rr != NULL) {
-		cprintf("%d Another FSCK is already running.\n", ERROR);
+		cprintf("%d Another FSCK is already running.\n", ERROR + RESOURCE_BUSY);
 		return;
 	}
 
@@ -706,7 +732,7 @@ void cmd_fsck(char *argbuf) {
 	cprintf("Freeing memory...\n");
 	while (rr != NULL) {
 		ptr = rr->next;
-		phree(rr);
+		free(rr);
 		rr = ptr;
 	}
 
@@ -724,5 +750,5 @@ char *serv_expire_init(void)
 {
 	CtdlRegisterSessionHook(purge_databases, EVT_TIMER);
 	CtdlRegisterProtoHook(cmd_fsck, "FSCK", "Check message ref counts");
-	return "$Id: serv_expire.c,v 613.0 2003/12/15 16:37:03 ajc Exp $";
+	return "$Id: serv_expire.c,v 620.0 2004/03/27 02:33:32 ajc Exp $";
 }
