@@ -1,5 +1,5 @@
 /*
- * $Id: serv_imap.c,v 613.0 2003/12/15 16:37:03 ajc Exp $ 
+ * $Id: serv_imap.c,v 620.0 2004/03/27 02:33:32 ajc Exp $ 
  *
  * IMAP server for the Citadel/UX system
  * Copyright (C) 2000-2002 by Art Cancro and others.
@@ -34,13 +34,6 @@
 #include <ctype.h>
 #include <string.h>
 #include <limits.h>
-
-#ifdef HAVE_OPENSSL_XXX /* temporarily disabled due to bugs */
-#include <openssl/ssl.h>
-#include <openssl/err.h>
-#include <openssl/rand.h>
-#endif
-
 #include "citadel.h"
 #include "server.h"
 #include "sysdep_decls.h"
@@ -62,7 +55,7 @@
 #include "imap_store.h"
 #include "imap_misc.h"
 
-#ifdef HAVE_OPENSSL_XXX /* temporarily disabled due to bugs */
+#ifdef HAVE_OPENSSL
 #include "serv_crypto.h"
 #endif
 
@@ -75,7 +68,7 @@ struct irl {
 };
 
 /* Data which is passed between imap_rename() and imap_rename_backend() */
-struct irlparms { 
+struct irlparms {
 	char *oldname;
 	char *newname;
 	struct irl **irl;
@@ -85,14 +78,15 @@ struct irlparms {
 /*
  * If there is a message ID map in memory, free it
  */
-void imap_free_msgids(void) {
+void imap_free_msgids(void)
+{
 	if (IMAP->msgids != NULL) {
-		phree(IMAP->msgids);
+		free(IMAP->msgids);
 		IMAP->msgids = NULL;
 		IMAP->num_msgs = 0;
 	}
 	if (IMAP->flags != NULL) {
-		phree(IMAP->flags);
+		free(IMAP->flags);
 		IMAP->flags = NULL;
 	}
 }
@@ -101,9 +95,10 @@ void imap_free_msgids(void) {
 /*
  * If there is a transmitted message in memory, free it
  */
-void imap_free_transmitted_message(void) {
+void imap_free_transmitted_message(void)
+{
 	if (IMAP->transmitted_message != NULL) {
-		phree(IMAP->transmitted_message);
+		free(IMAP->transmitted_message);
 		IMAP->transmitted_message = NULL;
 		IMAP->transmitted_length = 0;
 	}
@@ -113,17 +108,19 @@ void imap_free_transmitted_message(void) {
 /*
  * Set the \\Seen flag for messages which aren't new
  */
-void imap_set_seen_flags(void) {
+void imap_set_seen_flags(void)
+{
 	struct visit vbuf;
 	int i;
 
 	CtdlGetRelationship(&vbuf, &CC->user, &CC->room);
 	if (IMAP->num_msgs > 0) {
-		for (i=0; i<IMAP->num_msgs; ++i) {
+		for (i = 0; i < IMAP->num_msgs; ++i) {
 			if (is_msg_in_mset(vbuf.v_seen, IMAP->msgids[i])) {
 				IMAP->flags[i] |= IMAP_SEEN;
 			}
-			if (is_msg_in_mset(vbuf.v_answered, IMAP->msgids[i])) {
+			if (is_msg_in_mset
+			    (vbuf.v_answered, IMAP->msgids[i])) {
 				IMAP->flags[i] |= IMAP_ANSWERED;
 			}
 		}
@@ -140,24 +137,25 @@ void imap_set_seen_flags(void) {
  * allocate space in the list for REALLOC_INCREMENT messages at a time.  This
  * allows the mapping to proceed much faster.
  */
-void imap_add_single_msgid(long msgnum, void *userdata) {
-	
+void imap_add_single_msgid(long msgnum, void *userdata)
+{
+
 	IMAP->num_msgs = IMAP->num_msgs + 1;
 	if (IMAP->msgids == NULL) {
-		IMAP->msgids = mallok(IMAP->num_msgs * sizeof(long)
-					* REALLOC_INCREMENT);
-	}
-	else if (IMAP->num_msgs % REALLOC_INCREMENT == 0) {
-		IMAP->msgids = reallok(IMAP->msgids,
-			(IMAP->num_msgs + REALLOC_INCREMENT) * sizeof(long));
+		IMAP->msgids = malloc(IMAP->num_msgs * sizeof(long)
+				      * REALLOC_INCREMENT);
+	} else if (IMAP->num_msgs % REALLOC_INCREMENT == 0) {
+		IMAP->msgids = realloc(IMAP->msgids,
+				       (IMAP->num_msgs +
+					REALLOC_INCREMENT) * sizeof(long));
 	}
 	if (IMAP->flags == NULL) {
-		IMAP->flags = mallok(IMAP->num_msgs * sizeof(long)
-					* REALLOC_INCREMENT);
-	}
-	else if (IMAP->num_msgs % REALLOC_INCREMENT == 0) {
-		IMAP->flags = reallok(IMAP->flags,
-			(IMAP->num_msgs + REALLOC_INCREMENT) * sizeof(long));
+		IMAP->flags = malloc(IMAP->num_msgs * sizeof(long)
+				     * REALLOC_INCREMENT);
+	} else if (IMAP->num_msgs % REALLOC_INCREMENT == 0) {
+		IMAP->flags = realloc(IMAP->flags,
+				      (IMAP->num_msgs +
+				       REALLOC_INCREMENT) * sizeof(long));
 	}
 	IMAP->msgids[IMAP->num_msgs - 1] = msgnum;
 	IMAP->flags[IMAP->num_msgs - 1] = 0;
@@ -168,27 +166,31 @@ void imap_add_single_msgid(long msgnum, void *userdata) {
 /*
  * Set up a message ID map for the current room (folder)
  */
-void imap_load_msgids(void) {
-	 
+void imap_load_msgids(void)
+{
+
 	if (IMAP->selected == 0) {
-		lprintf(5, "imap_load_msgids() can't run; no room selected\n");
+		lprintf(CTDL_ERR,
+			"imap_load_msgids() can't run; no room selected\n");
 		return;
 	}
 
 	imap_free_msgids();	/* If there was already a map, free it */
 
 	CtdlForEachMessage(MSGS_ALL, 0L, NULL, NULL,
-		imap_add_single_msgid, NULL);
+			   imap_add_single_msgid, NULL);
 
 	imap_set_seen_flags();
-	lprintf(9, "imap_load_msgids() mapped %d messages\n", IMAP->num_msgs);
+	lprintf(CTDL_DEBUG, "imap_load_msgids() mapped %d messages\n",
+		IMAP->num_msgs);
 }
 
 
 /*
  * Re-scan the selected room (folder) and see if it's been changed at all
  */
-void imap_rescan_msgids(void) {
+void imap_rescan_msgids(void)
+{
 
 	int original_num_msgs = 0;
 	long original_highest = 0L;
@@ -200,7 +202,8 @@ void imap_rescan_msgids(void) {
 
 
 	if (IMAP->selected == 0) {
-		lprintf(5, "imap_load_msgids() can't run; no room selected\n");
+		lprintf(CTDL_ERR,
+			"imap_load_msgids() can't run; no room selected\n");
 		return;
 	}
 
@@ -209,12 +212,11 @@ void imap_rescan_msgids(void) {
 	 */
 	cdbfr = cdb_fetch(CDB_MSGLISTS, &CC->room.QRnumber, sizeof(long));
 	if (cdbfr != NULL) {
-		msglist = mallok(cdbfr->len);
+		msglist = malloc(cdbfr->len);
 		memcpy(msglist, cdbfr->ptr, cdbfr->len);
 		num_msgs = cdbfr->len / sizeof(long);
 		cdb_free(cdbfr);
-	}
-	else {
+	} else {
 		num_msgs = 0;
 	}
 
@@ -222,52 +224,57 @@ void imap_rescan_msgids(void) {
 	 * Check to see if any of the messages we know about have been expunged
 	 */
 	if (IMAP->num_msgs > 0)
-	 for (i=0; i<IMAP->num_msgs; ++i) {
+		for (i = 0; i < IMAP->num_msgs; ++i) {
 
-		message_still_exists = 0;
-		if (num_msgs > 0) for (j = 0; j < num_msgs; ++j) {
-			if (msglist[j] == IMAP->msgids[i]) {
-				message_still_exists = 1;
+			message_still_exists = 0;
+			if (num_msgs > 0)
+				for (j = 0; j < num_msgs; ++j) {
+					if (msglist[j] == IMAP->msgids[i]) {
+						message_still_exists = 1;
+					}
+				}
+
+			if (message_still_exists == 0) {
+				cprintf("* %d EXPUNGE\r\n", i + 1);
+
+				/* Here's some nice stupid nonsense.  When a message
+				 * is expunged, we have to slide all the existing
+				 * messages up in the message array.
+				 */
+				--IMAP->num_msgs;
+				memcpy(&IMAP->msgids[i],
+				       &IMAP->msgids[i + 1],
+				       (sizeof(long) *
+					(IMAP->num_msgs - i)));
+				memcpy(&IMAP->flags[i],
+				       &IMAP->flags[i + 1],
+				       (sizeof(long) *
+					(IMAP->num_msgs - i)));
+
+				--i;
 			}
+
 		}
 
-		if (message_still_exists == 0) {
-			cprintf("* %d EXPUNGE\r\n", i+1);
-
-			/* Here's some nice stupid nonsense.  When a message
-			 * is expunged, we have to slide all the existing
-			 * messages up in the message array.
-			 */
-			--IMAP->num_msgs;
-			memcpy(&IMAP->msgids[i], &IMAP->msgids[i+1],
-				(sizeof(long)*(IMAP->num_msgs-i)) );
-			memcpy(&IMAP->flags[i], &IMAP->flags[i+1],
-				(sizeof(long)*(IMAP->num_msgs-i)) );
-
-			--i;
-		}
-
-	}
-
 	/*
 	 * Remember how many messages were here before we re-scanned.
 	 */
 	original_num_msgs = IMAP->num_msgs;
 	if (IMAP->num_msgs > 0) {
 		original_highest = IMAP->msgids[IMAP->num_msgs - 1];
-	}
-	else {
+	} else {
 		original_highest = 0L;
 	}
 
 	/*
 	 * Now peruse the room for *new* messages only.
 	 */
-	if (num_msgs > 0) for (j=0; j<num_msgs; ++j) {
-		if (msglist[j] > original_highest) {
-			imap_add_single_msgid(msglist[j], NULL);
+	if (num_msgs > 0)
+		for (j = 0; j < num_msgs; ++j) {
+			if (msglist[j] > original_highest) {
+				imap_add_single_msgid(msglist[j], NULL);
+			}
 		}
-	}
 	imap_set_seen_flags();
 
 	/*
@@ -277,7 +284,8 @@ void imap_rescan_msgids(void) {
 		cprintf("* %d EXISTS\r\n", IMAP->num_msgs);
 	}
 
-	if (num_msgs != 0) phree(msglist);
+	if (num_msgs != 0)
+		free(msglist);
 }
 
 
@@ -290,15 +298,30 @@ void imap_rescan_msgids(void) {
  * This cleanup function blows away the temporary memory and files used by
  * the IMAP server.
  */
-void imap_cleanup_function(void) {
+void imap_cleanup_function(void)
+{
 
 	/* Don't do this stuff if this is not a IMAP session! */
-	if (CC->h_command_function != imap_command_loop) return;
+	if (CC->h_command_function != imap_command_loop)
+		return;
 
-	lprintf(9, "Performing IMAP cleanup hook\n");
+	lprintf(CTDL_DEBUG, "Performing IMAP cleanup hook\n");
 	imap_free_msgids();
 	imap_free_transmitted_message();
-	lprintf(9, "Finished IMAP cleanup hook\n");
+
+	if (IMAP->cached_fetch != NULL) {
+		fclose(IMAP->cached_fetch);
+		IMAP->cached_fetch = NULL;
+		IMAP->cached_msgnum = (-1);
+	}
+
+	if (IMAP->cached_body != NULL) {
+		fclose(IMAP->cached_body);
+		IMAP->cached_body = NULL;
+		IMAP->cached_bodymsgnum = (-1);
+	}
+
+	lprintf(CTDL_DEBUG, "Finished IMAP cleanup hook\n");
 }
 
 
@@ -306,11 +329,14 @@ void imap_cleanup_function(void) {
 /*
  * Here's where our IMAP session begins its happy day.
  */
-void imap_greeting(void) {
+void imap_greeting(void)
+{
 
 	strcpy(CC->cs_clientname, "IMAP session");
 	CtdlAllocUserData(SYM_IMAP, sizeof(struct citimap));
 	IMAP->authstate = imap_as_normal;
+	IMAP->cached_fetch = NULL;
+	IMAP->cached_msgnum = (-1);
 
 	cprintf("* OK %s Citadel/UX IMAP4rev1 server ready\r\n",
 		config.c_fqdn);
@@ -320,13 +346,14 @@ void imap_greeting(void) {
 /*
  * implements the LOGIN command (ordinary username/password login)
  */
-void imap_login(int num_parms, char *parms[]) {
+void imap_login(int num_parms, char *parms[])
+{
 	if (CtdlLoginExistingUser(parms[2]) == login_ok) {
 		if (CtdlTryPassword(parms[3]) == pass_ok) {
 			cprintf("%s OK login successful\r\n", parms[0]);
 			return;
 		}
-        }
+	}
 
 	cprintf("%s BAD Login incorrect\r\n", parms[0]);
 }
@@ -335,11 +362,18 @@ void imap_login(int num_parms, char *parms[]) {
 /*
  * Implements the AUTHENTICATE command
  */
-void imap_authenticate(int num_parms, char *parms[]) {
+void imap_authenticate(int num_parms, char *parms[])
+{
 	char buf[SIZ];
 
 	if (num_parms != 3) {
-		cprintf("%s BAD incorrect number of parameters\r\n", parms[0]);
+		cprintf("%s BAD incorrect number of parameters\r\n",
+			parms[0]);
+		return;
+	}
+
+	if (CC->logged_in) {
+		cprintf("%s BAD Already logged in.\r\n", parms[0]);
 		return;
 	}
 
@@ -357,7 +391,8 @@ void imap_authenticate(int num_parms, char *parms[]) {
 	}
 }
 
-void imap_auth_login_user(char *cmd) {
+void imap_auth_login_user(char *cmd)
+{
 	char buf[SIZ];
 
 	CtdlDecodeBase64(buf, cmd, SIZ);
@@ -368,14 +403,15 @@ void imap_auth_login_user(char *cmd) {
 	return;
 }
 
-void imap_auth_login_pass(char *cmd) {
+void imap_auth_login_pass(char *cmd)
+{
 	char buf[SIZ];
 
 	CtdlDecodeBase64(buf, cmd, SIZ);
 	if (CtdlTryPassword(buf) == pass_ok) {
-		cprintf("%s OK authentication succeeded\r\n", IMAP->authseq);
-	}
-	else {
+		cprintf("%s OK authentication succeeded\r\n",
+			IMAP->authseq);
+	} else {
 		cprintf("%s NO authentication failed\r\n", IMAP->authseq);
 	}
 	IMAP->authstate = imap_as_normal;
@@ -383,14 +419,14 @@ void imap_auth_login_pass(char *cmd) {
 }
 
 
-
 /*
  * implements the CAPABILITY command
  */
-void imap_capability(int num_parms, char *parms[]) {
-	cprintf("* CAPABILITY IMAP4 IMAP4REV1 AUTH=LOGIN");
+void imap_capability(int num_parms, char *parms[])
+{
+	cprintf("* CAPABILITY IMAP4 IMAP4REV1 NAMESPACE AUTH=LOGIN");
 
-#ifdef HAVE_OPENSSL_XXX /* temporarily disabled due to bugs */
+#ifdef HAVE_OPENSSL
 	cprintf(" STARTTLS");
 #endif
 
@@ -399,66 +435,36 @@ void imap_capability(int num_parms, char *parms[]) {
 }
 
 
+
 /*
- * implements the STARTTLS command
+ * implements the STARTTLS command (Citadel API version)
  */
-#ifdef HAVE_OPENSSL_XXX /* temporarily disabled due to bugs */
-void imap_starttls(int num_parms, char *parms[]) {
-	int retval, bits, alg_bits;
+#ifdef HAVE_OPENSSL
+void imap_starttls(int num_parms, char *parms[])
+{
+	char ok_response[SIZ];
+	char nosup_response[SIZ];
+	char error_response[SIZ];
 
-	if (!ssl_ctx) {
-		cprintf("%s NO No SSL_CTX available\r\n", parms[0]);
-		return;
-	}
-	if (!(CC->ssl = SSL_new(ssl_ctx))) {
-		lprintf(2, "SSL_new failed: %s\n",
-				ERR_reason_error_string(ERR_peek_error()));
-		cprintf("%s NO SSL_new: %s\r\n", parms[0],
-				ERR_reason_error_string(ERR_get_error()));
-		return;
-	}
-	if (!(SSL_set_fd(CC->ssl, CC->client_socket))) {
-		lprintf(2, "SSL_set_fd failed: %s\n",
-				ERR_reason_error_string(ERR_peek_error()));
-		SSL_free(CC->ssl);
-		CC->ssl = NULL;
-		cprintf("%s NO SSL_set_fd: %s\r\n", parms[0],
-				ERR_reason_error_string(ERR_get_error()));
-		return;
-	}
-	cprintf("%s OK begin TLS negotiation now\r\n", parms[0]);
-	retval = SSL_accept(CC->ssl);
-	if (retval < 1) {
-		/*
-		 * Can't notify the client of an error here; they will
-		 * discover the problem at the SSL layer and should
-		 * revert to unencrypted communications.
-		 */
-		long errval;
-
-		errval = SSL_get_error(CC->ssl, retval);
-		lprintf(2, "SSL_accept failed: %s\n",
-				ERR_reason_error_string(ERR_get_error()));
-		SSL_free(CC->ssl);
-		CC->ssl = NULL;
-		return;
-	}
-	BIO_set_close(CC->ssl->rbio, BIO_NOCLOSE);
-	bits = SSL_CIPHER_get_bits(SSL_get_current_cipher(CC->ssl), &alg_bits);
-	lprintf(3, "SSL/TLS using %s on %s (%d of %d bits)\n",
-			SSL_CIPHER_get_name(SSL_get_current_cipher(CC->ssl)),
-			SSL_CIPHER_get_version(SSL_get_current_cipher(CC->ssl)),
-			bits, alg_bits);
-	CC->redirect_ssl = 1;
+	sprintf(ok_response,
+		"%s OK begin TLS negotiation now\r\n",
+		parms[0]);
+	sprintf(nosup_response,
+		"%s NO TLS not supported here\r\n",
+		parms[0]);
+	sprintf(error_response,
+		"%s BAD Internal error\r\n",
+		parms[0]);
+	CtdlStartTLS(ok_response, nosup_response, error_response);
 }
 #endif
 
 
-
 /*
  * implements the SELECT command
  */
-void imap_select(int num_parms, char *parms[]) {
+void imap_select(int num_parms, char *parms[])
+{
 	char towhere[SIZ];
 	char augmented_roomname[ROOMNAMELEN];
 	int c = 0;
@@ -480,26 +486,26 @@ void imap_select(int num_parms, char *parms[]) {
 	floornum = (i & 0x00ff);
 	roomflags = (i & 0xff00);
 
-        /* First try a regular match */
-        c = getroom(&QRscratch, towhere);
+	/* First try a regular match */
+	c = getroom(&QRscratch, towhere);
 
-        /* Then try a mailbox name match */
-        if (c != 0) {
-                MailboxName(augmented_roomname, sizeof augmented_roomname,
+	/* Then try a mailbox name match */
+	if (c != 0) {
+		MailboxName(augmented_roomname, sizeof augmented_roomname,
 			    &CC->user, towhere);
-                c = getroom(&QRscratch, augmented_roomname);
-                if (c == 0)
-                        strcpy(towhere, augmented_roomname);
-        }
+		c = getroom(&QRscratch, augmented_roomname);
+		if (c == 0)
+			strcpy(towhere, augmented_roomname);
+	}
 
 	/* If the room exists, check security/access */
-        if (c == 0) {
-                /* See if there is an existing user/room relationship */
-                ra = CtdlRoomAccess(&QRscratch, &CC->user);
+	if (c == 0) {
+		/* See if there is an existing user/room relationship */
+		ra = CtdlRoomAccess(&QRscratch, &CC->user);
 
-                /* normal clients have to pass through security */
-                if (ra & UA_KNOWN) {
-                        ok = 1;
+		/* normal clients have to pass through security */
+		if (ra & UA_KNOWN) {
+			ok = 1;
 		}
 	}
 
@@ -507,10 +513,13 @@ void imap_select(int num_parms, char *parms[]) {
 	if (!ok) {
 		cprintf("%s NO ... no such room, or access denied\r\n",
 			parms[0]);
-		IMAP->selected = 0;
+		/* IMAP->selected = 0; */
 		return;
 	}
 
+	/* If we already had some other folder selected, auto-expunge it */
+	imap_do_expunge();
+
 	/*
 	 * usergoto() formally takes us to the desired room, happily returning
 	 * the number of messages and number of new messages.
@@ -521,8 +530,7 @@ void imap_select(int num_parms, char *parms[]) {
 
 	if (!strcasecmp(parms[1], "EXAMINE")) {
 		IMAP->readonly = 1;
-	}
-	else {
+	} else {
 		IMAP->readonly = 0;
 	}
 
@@ -530,14 +538,19 @@ void imap_select(int num_parms, char *parms[]) {
 
 	cprintf("* %d EXISTS\r\n", msgs);
 	cprintf("* %d RECENT\r\n", new);
+
+	/* Note that \Deleted is a valid flag, but not a permanent flag,
+	 * because we don't maintain its state across sessions.  Citadel
+	 * automatically expunges mailboxes when they are de-selected.
+	 */
 	cprintf("* FLAGS (\\Deleted \\Seen \\Answered)\r\n");
-	cprintf("* OK [PERMANENTFLAGS (\\Deleted \\Seen \\Answered)] "
+	cprintf("* OK [PERMANENTFLAGS (\\Seen \\Answered)] "
 		"permanent flags\r\n");
+
 	cprintf("* OK [UIDVALIDITY 0] UIDs valid\r\n");
 	cprintf("%s OK [%s] %s completed\r\n",
 		parms[0],
-		(IMAP->readonly ? "READ-ONLY" : "READ-WRITE"),
-		parms[1]);
+		(IMAP->readonly ? "READ-ONLY" : "READ-WRITE"), parms[1]);
 }
 
 
@@ -545,34 +558,42 @@ void imap_select(int num_parms, char *parms[]) {
 /*
  * does the real work for expunge
  */
-int imap_do_expunge(void) {
+int imap_do_expunge(void)
+{
 	int i;
 	int num_expunged = 0;
 
-	if (IMAP->num_msgs > 0) for (i=0; i<IMAP->num_msgs; ++i) {
-		if (IMAP->flags[i] & IMAP_DELETED) {
-			CtdlDeleteMessages(CC->room.QRname,
-					IMAP->msgids[i], "");
-			++num_expunged;
-			lprintf(9, "%ld ... deleted\n", IMAP->msgids[i]);
+	lprintf(CTDL_DEBUG, "imap_do_expunge() called\n");
+	if (IMAP->selected == 0)
+		return (0);
+
+	if (IMAP->num_msgs > 0)
+		for (i = 0; i < IMAP->num_msgs; ++i) {
+			if (IMAP->flags[i] & IMAP_DELETED) {
+				CtdlDeleteMessages(CC->room.QRname,
+						   IMAP->msgids[i], "");
+				++num_expunged;
+				lprintf(CTDL_DEBUG, "%ld ... deleted\n",
+					IMAP->msgids[i]);
+			} else {
+				lprintf(CTDL_DEBUG, "%ld ... not deleted\n",
+					IMAP->msgids[i]);
+			}
 		}
-		else {
-			lprintf(9, "%ld ... not deleted\n", IMAP->msgids[i]);
-		}
-	}
 
 	if (num_expunged > 0) {
 		imap_rescan_msgids();
 	}
 
-	return(num_expunged);
+	return (num_expunged);
 }
 
 
 /*
  * implements the EXPUNGE command syntax
  */
-void imap_expunge(int num_parms, char *parms[]) {
+void imap_expunge(int num_parms, char *parms[])
+{
 	int num_expunged = 0;
 
 	num_expunged = imap_do_expunge();
@@ -583,7 +604,8 @@ void imap_expunge(int num_parms, char *parms[]) {
 /*
  * implements the CLOSE command
  */
-void imap_close(int num_parms, char *parms[]) {
+void imap_close(int num_parms, char *parms[])
+{
 
 	/* Yes, we always expunge on close. */
 	imap_do_expunge();
@@ -595,19 +617,59 @@ void imap_close(int num_parms, char *parms[]) {
 }
 
 
+/*
+ * Implements the NAMESPACE command.
+ */
+void imap_namespace(int num_parms, char *parms[])
+{
+	int i;
+	struct floor *fl;
+	int floors = 0;
+	char buf[SIZ];
+
+	cprintf("* NAMESPACE ");
+
+	/* All personal folders are subordinate to INBOX. */
+	cprintf("((\"INBOX|\" \"|\")) ");
+
+	/* Other users' folders ... coming soon! FIXME */
+	cprintf("NIL ");
+
+	/* Show all floors as shared namespaces.  Neato! */
+	cprintf("(");
+	for (i = 0; i < MAXFLOORS; ++i) {
+		fl = cgetfloor(i);
+		if (fl->f_flags & F_INUSE) {
+			if (floors > 0) cprintf(" ");
+			cprintf("(");
+			sprintf(buf, "%s|", fl->f_name);
+			imap_strout(buf);
+			cprintf(" \"|\")");
+			++floors;
+		}
+	}
+	cprintf(")");
+
+	/* Wind it up with a newline and a completion message. */
+	cprintf("\r\n");
+	cprintf("%s OK NAMESPACE completed\r\n", parms[0]);
+}
+
 
 
 /*
  * Used by LIST and LSUB to show the floors in the listing
  */
-void imap_list_floors(char *cmd, char *pattern) {
+void imap_list_floors(char *cmd, char *pattern)
+{
 	int i;
 	struct floor *fl;
 
-	for (i=0; i<MAXFLOORS; ++i) {
+	for (i = 0; i < MAXFLOORS; ++i) {
 		fl = cgetfloor(i);
 		if (fl->f_flags & F_INUSE) {
-			if (imap_mailbox_matches_pattern(pattern, fl->f_name)) {
+			if (imap_mailbox_matches_pattern
+			    (pattern, fl->f_name)) {
 				cprintf("* %s (\\NoSelect) \"|\" ", cmd);
 				imap_strout(fl->f_name);
 				cprintf("\r\n");
@@ -624,12 +686,13 @@ void imap_list_floors(char *cmd, char *pattern) {
  * IMAP "subscribed folder" is equivocated to Citadel "known rooms."  This
  * may or may not be the desired behavior in the future.
  */
-void imap_lsub_listroom(struct ctdlroom *qrbuf, void *data) {
+void imap_lsub_listroom(struct ctdlroom *qrbuf, void *data)
+{
 	char buf[SIZ];
 	int ra;
 	char *pattern;
 
-	pattern = (char *)data;
+	pattern = (char *) data;
 
 	/* Only list rooms to which the user has access!! */
 	ra = CtdlRoomAccess(qrbuf, &CC->user);
@@ -647,7 +710,8 @@ void imap_lsub_listroom(struct ctdlroom *qrbuf, void *data) {
 /*
  * Implements the LSUB command
  */
-void imap_lsub(int num_parms, char *parms[]) {
+void imap_lsub(int num_parms, char *parms[])
+{
 	char pattern[SIZ];
 	if (num_parms < 4) {
 		cprintf("%s BAD arguments invalid\r\n", parms[0]);
@@ -655,7 +719,7 @@ void imap_lsub(int num_parms, char *parms[]) {
 	}
 	snprintf(pattern, sizeof pattern, "%s%s", parms[2], parms[3]);
 
-	if (strlen(parms[3])==0) {
+	if (strlen(parms[3]) == 0) {
 		cprintf("* LIST (\\Noselect) \"|\" \"\"\r\n");
 	}
 
@@ -672,17 +736,18 @@ void imap_lsub(int num_parms, char *parms[]) {
 /*
  * Back end for imap_list()
  */
-void imap_list_listroom(struct ctdlroom *qrbuf, void *data) {
+void imap_list_listroom(struct ctdlroom *qrbuf, void *data)
+{
 	char buf[SIZ];
 	int ra;
 	char *pattern;
 
-	pattern = (char *)data;
+	pattern = (char *) data;
 
 	/* Only list rooms to which the user has access!! */
 	ra = CtdlRoomAccess(qrbuf, &CC->user);
-	if ( (ra & UA_KNOWN) 
-	  || ((ra & UA_GOTOALLOWED) && (ra & UA_ZAPPED))) {
+	if ((ra & UA_KNOWN)
+	    || ((ra & UA_GOTOALLOWED) && (ra & UA_ZAPPED))) {
 		imap_mailboxname(buf, sizeof buf, qrbuf);
 		if (imap_mailbox_matches_pattern(pattern, buf)) {
 			cprintf("* LIST () \"|\" ");
@@ -696,7 +761,8 @@ void imap_list_listroom(struct ctdlroom *qrbuf, void *data) {
 /*
  * Implements the LIST command
  */
-void imap_list(int num_parms, char *parms[]) {
+void imap_list(int num_parms, char *parms[])
+{
 	char pattern[SIZ];
 	if (num_parms < 4) {
 		cprintf("%s BAD arguments invalid\r\n", parms[0]);
@@ -704,7 +770,7 @@ void imap_list(int num_parms, char *parms[]) {
 	}
 	snprintf(pattern, sizeof pattern, "%s%s", parms[2], parms[3]);
 
-	if (strlen(parms[3])==0) {
+	if (strlen(parms[3]) == 0) {
 		cprintf("* LIST (\\Noselect) \"|\" \"\"\r\n");
 	}
 
@@ -722,7 +788,8 @@ void imap_list(int num_parms, char *parms[]) {
  * Implements the CREATE command
  *
  */
-void imap_create(int num_parms, char *parms[]) {
+void imap_create(int num_parms, char *parms[])
+{
 	int ret;
 	char roomname[ROOMNAMELEN];
 	int floornum;
@@ -730,7 +797,9 @@ void imap_create(int num_parms, char *parms[]) {
 	int newroomtype;
 
 	if (strchr(parms[2], '\\') != NULL) {
-		cprintf("%s NO Invalid character in folder name\r\n", parms[0]);
+		cprintf("%s NO Invalid character in folder name\r\n",
+			parms[0]);
+		lprintf(CTDL_DEBUG, "invalid character in folder name\n");
 		return;
 	}
 
@@ -738,36 +807,46 @@ void imap_create(int num_parms, char *parms[]) {
 	if (ret < 0) {
 		cprintf("%s NO Invalid mailbox name or location\r\n",
 			parms[0]);
+		lprintf(CTDL_DEBUG, "invalid mailbox name or location\n");
 		return;
 	}
-	floornum = ( ret & 0x00ff );	/* lower 8 bits = floor number */
-	flags =    ( ret & 0xff00 );	/* upper 8 bits = flags        */
+	floornum = (ret & 0x00ff);	/* lower 8 bits = floor number */
+	flags = (ret & 0xff00);	/* upper 8 bits = flags        */
+
+	if (flags & IR_MAILBOX) {
+		if (strncasecmp(parms[2], "INBOX|", 6)) {
+			cprintf("%s NO Personal folders must be created under INBOX\r\n", parms[0]);
+			lprintf(CTDL_DEBUG, "not subordinate to inbox\n");
+			return;
+		}
+	}
 
 	if (flags & IR_MAILBOX) {
 		newroomtype = 4;	/* private mailbox */
-	}
-	else {
+	} else {
 		newroomtype = 0;	/* public folder */
 	}
 
-	lprintf(7, "Create new room <%s> on floor <%d> with type <%d>\n",
+	lprintf(CTDL_INFO, "Create new room <%s> on floor <%d> with type <%d>\n",
 		roomname, floornum, newroomtype);
 
 	ret = create_room(roomname, newroomtype, "", floornum, 1, 0);
 	if (ret == 0) {
-		cprintf("%s NO Mailbox already exists, or create failed\r\n",
-			parms[0]);
-	}
-	else {
+		cprintf
+		    ("%s NO Mailbox already exists, or create failed\r\n",
+		     parms[0]);
+	} else {
 		cprintf("%s OK CREATE completed\r\n", parms[0]);
 	}
+	lprintf(CTDL_DEBUG, "imap_create() completed\n");
 }
 
 
 /*
  * Locate a room by its IMAP folder name, and check access to it
  */
-int imap_grabroom(char *returned_roomname, char *foldername) {
+int imap_grabroom(char *returned_roomname, char *foldername)
+{
 	int ret;
 	char augmented_roomname[ROOMNAMELEN];
 	char roomname[ROOMNAMELEN];
@@ -778,40 +857,39 @@ int imap_grabroom(char *returned_roomname, char *foldername) {
 
 	ret = imap_roomname(roomname, sizeof roomname, foldername);
 	if (ret < 0) {
-		return(1);
+		return (1);
 	}
 
-        /* First try a regular match */
-        c = getroom(&QRscratch, roomname);
+	/* First try a regular match */
+	c = getroom(&QRscratch, roomname);
 
-        /* Then try a mailbox name match */
-        if (c != 0) {
-                MailboxName(augmented_roomname, sizeof augmented_roomname,
+	/* Then try a mailbox name match */
+	if (c != 0) {
+		MailboxName(augmented_roomname, sizeof augmented_roomname,
 			    &CC->user, roomname);
-                c = getroom(&QRscratch, augmented_roomname);
-                if (c == 0)
-                        strcpy(roomname, augmented_roomname);
-        }
+		c = getroom(&QRscratch, augmented_roomname);
+		if (c == 0)
+			strcpy(roomname, augmented_roomname);
+	}
 
 	/* If the room exists, check security/access */
-        if (c == 0) {
-                /* See if there is an existing user/room relationship */
-                ra = CtdlRoomAccess(&QRscratch, &CC->user);
+	if (c == 0) {
+		/* See if there is an existing user/room relationship */
+		ra = CtdlRoomAccess(&QRscratch, &CC->user);
 
-                /* normal clients have to pass through security */
-                if (ra & UA_KNOWN) {
-                        ok = 1;
+		/* normal clients have to pass through security */
+		if (ra & UA_KNOWN) {
+			ok = 1;
 		}
 	}
 
 	/* Fail here if no such room */
 	if (!ok) {
 		strcpy(returned_roomname, "");
-		return(2);
-	}
-	else {
+		return (2);
+	} else {
 		strcpy(returned_roomname, QRscratch.QRname);
-		return(0);
+		return (0);
 	}
 }
 
@@ -820,7 +898,8 @@ int imap_grabroom(char *returned_roomname, char *foldername) {
  * Implements the STATUS command (sort of)
  *
  */
-void imap_status(int num_parms, char *parms[]) {
+void imap_status(int num_parms, char *parms[])
+{
 	int ret;
 	char roomname[ROOMNAMELEN];
 	char buf[SIZ];
@@ -829,8 +908,9 @@ void imap_status(int num_parms, char *parms[]) {
 
 	ret = imap_grabroom(roomname, parms[2]);
 	if (ret != 0) {
-		cprintf("%s NO Invalid mailbox name or location, or access denied\r\n",
-			parms[0]);
+		cprintf
+		    ("%s NO Invalid mailbox name or location, or access denied\r\n",
+		     parms[0]);
 		return;
 	}
 
@@ -878,7 +958,8 @@ void imap_status(int num_parms, char *parms[]) {
  * Implements the SUBSCRIBE command
  *
  */
-void imap_subscribe(int num_parms, char *parms[]) {
+void imap_subscribe(int num_parms, char *parms[])
+{
 	int ret;
 	char roomname[ROOMNAMELEN];
 	char savedroom[ROOMNAMELEN];
@@ -886,8 +967,9 @@ void imap_subscribe(int num_parms, char *parms[]) {
 
 	ret = imap_grabroom(roomname, parms[2]);
 	if (ret != 0) {
-		cprintf("%s NO Invalid mailbox name or location, or access denied\r\n",
-			parms[0]);
+		cprintf
+		    ("%s NO Invalid mailbox name or location, or access denied\r\n",
+		     parms[0]);
 		return;
 	}
 
@@ -917,7 +999,8 @@ void imap_subscribe(int num_parms, char *parms[]) {
  * Implements the UNSUBSCRIBE command
  *
  */
-void imap_unsubscribe(int num_parms, char *parms[]) {
+void imap_unsubscribe(int num_parms, char *parms[])
+{
 	int ret;
 	char roomname[ROOMNAMELEN];
 	char savedroom[ROOMNAMELEN];
@@ -925,8 +1008,9 @@ void imap_unsubscribe(int num_parms, char *parms[]) {
 
 	ret = imap_grabroom(roomname, parms[2]);
 	if (ret != 0) {
-		cprintf("%s NO Invalid mailbox name or location, or access denied\r\n",
-			parms[0]);
+		cprintf
+		    ("%s NO Invalid mailbox name or location, or access denied\r\n",
+		     parms[0]);
 		return;
 	}
 
@@ -943,10 +1027,10 @@ void imap_unsubscribe(int num_parms, char *parms[]) {
 	 */
 	if (CtdlForgetThisRoom() == 0) {
 		cprintf("%s OK UNSUBSCRIBE completed\r\n", parms[0]);
-	}
-	else {
-		cprintf("%s NO You may not unsubscribe from this folder.\r\n",
-			parms[0]);
+	} else {
+		cprintf
+		    ("%s NO You may not unsubscribe from this folder.\r\n",
+		     parms[0]);
 	}
 
 	/*
@@ -964,7 +1048,8 @@ void imap_unsubscribe(int num_parms, char *parms[]) {
  * Implements the DELETE command
  *
  */
-void imap_delete(int num_parms, char *parms[]) {
+void imap_delete(int num_parms, char *parms[])
+{
 	int ret;
 	char roomname[ROOMNAMELEN];
 	char savedroom[ROOMNAMELEN];
@@ -993,8 +1078,7 @@ void imap_delete(int num_parms, char *parms[]) {
 	if (CtdlDoIHavePermissionToDeleteThisRoom(&CC->room)) {
 		cprintf("%s OK DELETE completed\r\n", parms[0]);
 		delete_room(&CC->room);
-	}
-	else {
+	} else {
 		cprintf("%s NO Can't delete this folder.\r\n", parms[0]);
 	}
 
@@ -1011,7 +1095,8 @@ void imap_delete(int num_parms, char *parms[]) {
 /*
  * Back end function for imap_rename()
  */
-void imap_rename_backend(struct ctdlroom *qrbuf, void *data) {
+void imap_rename_backend(struct ctdlroom *qrbuf, void *data)
+{
 	char foldername[SIZ];
 	char newfoldername[SIZ];
 	char newroomname[ROOMNAMELEN];
@@ -1019,23 +1104,24 @@ void imap_rename_backend(struct ctdlroom *qrbuf, void *data) {
 	struct irl *irlp = NULL;	/* scratch pointer */
 	struct irlparms *irlparms;
 
-	irlparms = (struct irlparms *)data;
+	irlparms = (struct irlparms *) data;
 	imap_mailboxname(foldername, sizeof foldername, qrbuf);
 
 	/* Rename subfolders */
-	if ( (!strncasecmp(foldername, irlparms->oldname,
-	   strlen(irlparms->oldname))
-	   && (foldername[strlen(irlparms->oldname)] == '|')) ) {
+	if ((!strncasecmp(foldername, irlparms->oldname,
+			  strlen(irlparms->oldname))
+	     && (foldername[strlen(irlparms->oldname)] == '|'))) {
 
 		sprintf(newfoldername, "%s|%s",
 			irlparms->newname,
-			&foldername[strlen(irlparms->oldname)+1]
-		);
+			&foldername[strlen(irlparms->oldname) + 1]
+		    );
 
 		newfloor = imap_roomname(newroomname,
-			sizeof newroomname, newfoldername) & 0xFF;
+					 sizeof newroomname,
+					 newfoldername) & 0xFF;
 
-		irlp = (struct irl *) mallok(sizeof(struct irl));
+		irlp = (struct irl *) malloc(sizeof(struct irl));
 		strcpy(irlp->irl_newroom, newroomname);
 		strcpy(irlp->irl_oldroom, qrbuf->QRname);
 		irlp->irl_newfloor = newfloor;
@@ -1043,24 +1129,26 @@ void imap_rename_backend(struct ctdlroom *qrbuf, void *data) {
 		*(irlparms->irl) = irlp;
 	}
 }
-	
+
 
 /*
  * Implements the RENAME command
  *
  */
-void imap_rename(int num_parms, char *parms[]) {
+void imap_rename(int num_parms, char *parms[])
+{
 	char old_room[ROOMNAMELEN];
 	char new_room[ROOMNAMELEN];
 	int oldr, newr;
 	int new_floor;
 	int r;
-	struct irl *irl = NULL;		/* the list */
+	struct irl *irl = NULL;	/* the list */
 	struct irl *irlp = NULL;	/* scratch pointer */
 	struct irlparms irlparms;
 
 	if (strchr(parms[3], '\\') != NULL) {
-		cprintf("%s NO Invalid character in folder name\r\n", parms[0]);
+		cprintf("%s NO Invalid character in folder name\r\n",
+			parms[0]);
 		return;
 	}
 
@@ -1071,7 +1159,8 @@ void imap_rename(int num_parms, char *parms[]) {
 	r = CtdlRenameRoom(old_room, new_room, new_floor);
 
 	if (r == crr_room_not_found) {
-		cprintf("%s NO Could not locate this folder\r\n", parms[0]);
+		cprintf("%s NO Could not locate this folder\r\n",
+			parms[0]);
 		return;
 	}
 	if (r == crr_already_exists) {
@@ -1111,19 +1200,21 @@ void imap_rename(int num_parms, char *parms[]) {
 		irlparms.oldname = parms[2];
 		irlparms.newname = parms[3];
 		irlparms.irl = &irl;
-		ForEachRoom(imap_rename_backend, (void *)&irlparms);
+		ForEachRoom(imap_rename_backend, (void *) &irlparms);
 
 		/* ... and now rename them. */
 		while (irl != NULL) {
 			r = CtdlRenameRoom(irl->irl_oldroom,
-				irl->irl_newroom, irl->irl_newfloor);
+					   irl->irl_newroom,
+					   irl->irl_newfloor);
 			if (r != crr_ok) {
 				/* FIXME handle error returns better */
-				lprintf(5, "CtdlRenameRoom() error %d\n", r);
+				lprintf(CTDL_ERR, "CtdlRenameRoom() error %d\n",
+					r);
 			}
 			irlp = irl;
 			irl = irl->next;
-			phree(irlp);
+			free(irlp);
 		}
 	}
 
@@ -1136,26 +1227,30 @@ void imap_rename(int num_parms, char *parms[]) {
 /* 
  * Main command loop for IMAP sessions.
  */
-void imap_command_loop(void) {
+void imap_command_loop(void)
+{
 	char cmdbuf[SIZ];
 	char *parms[SIZ];
 	int num_parms;
 
 	time(&CC->lastcmd);
-	memset(cmdbuf, 0, sizeof cmdbuf); /* Clear it, just in case */
+	memset(cmdbuf, 0, sizeof cmdbuf);	/* Clear it, just in case */
 	if (client_gets(cmdbuf) < 1) {
-		lprintf(3, "IMAP socket is broken.  Ending session.\r\n");
+		lprintf(CTDL_ERR, "IMAP socket is broken.  Ending session.\r\n");
 		CC->kill_me = 1;
 		return;
 	}
 
-	lprintf(5, "IMAP: %s\r\n", cmdbuf);
-	while (strlen(cmdbuf) < 5) strcat(cmdbuf, " ");
+	lprintf(CTDL_INFO, "IMAP: %s\r\n", cmdbuf);
+	while (strlen(cmdbuf) < 5)
+		strcat(cmdbuf, " ");
 
 
 	/* strip off l/t whitespace and CRLF */
-	if (cmdbuf[strlen(cmdbuf)-1]=='\n') cmdbuf[strlen(cmdbuf)-1]=0;
-	if (cmdbuf[strlen(cmdbuf)-1]=='\r') cmdbuf[strlen(cmdbuf)-1]=0;
+	if (cmdbuf[strlen(cmdbuf) - 1] == '\n')
+		cmdbuf[strlen(cmdbuf) - 1] = 0;
+	if (cmdbuf[strlen(cmdbuf) - 1] == '\r')
+		cmdbuf[strlen(cmdbuf) - 1] = 0;
 	striplt(cmdbuf);
 
 	/* If we're in the middle of a multi-line command, handle that */
@@ -1194,15 +1289,17 @@ void imap_command_loop(void) {
 
 	/* The commands below may be executed in any state */
 
-	else if ( (!strcasecmp(parms[1], "NOOP"))
-	   || (!strcasecmp(parms[1], "CHECK")) ) {
+	else if ((!strcasecmp(parms[1], "NOOP"))
+		 || (!strcasecmp(parms[1], "CHECK"))) {
 		cprintf("%s OK This command successfully did nothing.\r\n",
 			parms[0]);
 	}
 
 	else if (!strcasecmp(parms[1], "LOGOUT")) {
+		imap_do_expunge();	/* yes, we auto-expunge */
 		cprintf("* BYE %s logging out\r\n", config.c_fqdn);
-		cprintf("%s OK thank you for using Citadel IMAP\r\n", parms[0]);
+		cprintf("%s OK thank you for using Citadel IMAP\r\n",
+			parms[0]);
 		CC->kill_me = 1;
 		return;
 	}
@@ -1218,13 +1315,11 @@ void imap_command_loop(void) {
 	else if (!strcasecmp(parms[1], "CAPABILITY")) {
 		imap_capability(num_parms, parms);
 	}
-
-#ifdef HAVE_OPENSSL_XXX /* temporarily disabled due to bugs */
+#ifdef HAVE_OPENSSL
 	else if (!strcasecmp(parms[1], "STARTTLS")) {
 		imap_starttls(num_parms, parms);
 	}
 #endif
-
 	else if (!CC->logged_in) {
 		cprintf("%s BAD Not logged in.\r\n", parms[0]);
 	}
@@ -1275,6 +1370,10 @@ void imap_command_loop(void) {
 		imap_append(num_parms, parms);
 	}
 
+	else if (!strcasecmp(parms[1], "NAMESPACE")) {
+		imap_namespace(num_parms, parms);
+	}
+
 	else if (IMAP->selected == 0) {
 		cprintf("%s BAD no folder selected\r\n", parms[0]);
 	}
@@ -1285,8 +1384,8 @@ void imap_command_loop(void) {
 		imap_fetch(num_parms, parms);
 	}
 
-	else if ( (!strcasecmp(parms[1], "UID"))
-		&& (!strcasecmp(parms[2], "FETCH")) ) {
+	else if ((!strcasecmp(parms[1], "UID"))
+		 && (!strcasecmp(parms[2], "FETCH"))) {
 		imap_uidfetch(num_parms, parms);
 	}
 
@@ -1294,8 +1393,8 @@ void imap_command_loop(void) {
 		imap_search(num_parms, parms);
 	}
 
-	else if ( (!strcasecmp(parms[1], "UID"))
-		&& (!strcasecmp(parms[2], "SEARCH")) ) {
+	else if ((!strcasecmp(parms[1], "UID"))
+		 && (!strcasecmp(parms[2], "SEARCH"))) {
 		imap_uidsearch(num_parms, parms);
 	}
 
@@ -1303,8 +1402,8 @@ void imap_command_loop(void) {
 		imap_store(num_parms, parms);
 	}
 
-	else if ( (!strcasecmp(parms[1], "UID"))
-		&& (!strcasecmp(parms[2], "STORE")) ) {
+	else if ((!strcasecmp(parms[1], "UID"))
+		 && (!strcasecmp(parms[2], "STORE"))) {
 		imap_uidstore(num_parms, parms);
 	}
 
@@ -1312,8 +1411,8 @@ void imap_command_loop(void) {
 		imap_copy(num_parms, parms);
 	}
 
-	else if ( (!strcasecmp(parms[1], "UID"))
-		&& (!strcasecmp(parms[2], "COPY")) ) {
+	else if ((!strcasecmp(parms[1], "UID"))
+		 && (!strcasecmp(parms[2], "COPY"))) {
 		imap_uidcopy(num_parms, parms);
 	}
 
@@ -1345,9 +1444,7 @@ void imap_command_loop(void) {
 char *serv_imap_init(void)
 {
 	CtdlRegisterServiceHook(config.c_imap_port,
-				NULL,
-				imap_greeting,
-				imap_command_loop);
+				NULL, imap_greeting, imap_command_loop);
 	CtdlRegisterSessionHook(imap_cleanup_function, EVT_STOP);
-	return "$Id: serv_imap.c,v 613.0 2003/12/15 16:37:03 ajc Exp $";
+	return "$Id: serv_imap.c,v 620.0 2004/03/27 02:33:32 ajc Exp $";
 }
