@@ -1,4 +1,4 @@
-/* $Id: serv_imap.h,v 613.0 2003/12/15 16:37:03 ajc Exp $ 
+/* $Id: serv_imap.h,v 620.0 2004/03/27 02:33:32 ajc Exp $ 
  */
 
 
@@ -7,7 +7,7 @@ void imap_greeting(void);
 void imap_command_loop(void);
 int imap_grabroom(char *returned_roomname, char *foldername);
 void imap_free_transmitted_message(void);
-
+int imap_do_expunge(void);
 
 
 struct citimap {
@@ -20,6 +20,13 @@ struct citimap {
 	unsigned int *flags;
 	char *transmitted_message;	/* for APPEND command... */
 	size_t transmitted_length;
+
+	FILE *cached_fetch;		/* cache our most recent RFC822 FETCH */
+	long cached_msgnum;		/* because the client might ask for it in pieces */
+
+	FILE *cached_body;		/* cache our most recent BODY FETCH */
+	char cached_bodypart[SIZ];	/* because the client might ask for it in pieces */
+	long cached_bodymsgnum;
 };
 
 /*
