@@ -1,5 +1,5 @@
 /*
- * $Id: serv_network.c,v 613.0 2003/12/15 16:37:03 ajc Exp $ 
+ * $Id: serv_network.c,v 620.0 2004/03/27 02:33:34 ajc Exp $ 
  *
  * This module handles shared rooms, inter-Citadel mail, and outbound
  * mailing list processing.
@@ -85,6 +85,8 @@ struct RoomProcList *rplist = NULL;
  */
 struct NetMap *the_netmap = NULL;
 
+char *ignetcfg = NULL;
+
 /*
  * Keep track of what messages to reject
  */
@@ -101,7 +103,7 @@ struct FilterList *load_filter_list(void) {
 	/* Use the string tokenizer to grab one line at a time */
 	for (i=0; i<num_tokens(serialized_list, '\n'); ++i) {
 		extract_token(buf, serialized_list, i, '\n');
-		nptr = (struct FilterList *) mallok(sizeof(struct FilterList));
+		nptr = (struct FilterList *) malloc(sizeof(struct FilterList));
 		extract(nptr->fl_user, buf, 0);
 		striplt(nptr->fl_user);
 		extract(nptr->fl_room, buf, 1);
@@ -114,7 +116,7 @@ struct FilterList *load_filter_list(void) {
 		 */
 		if (strlen(nptr->fl_user) + strlen(nptr->fl_room)
 		   + strlen(nptr->fl_node) == 0) {
-			phree(nptr);
+			free(nptr);
 		}
 		else {
 			nptr->next = newlist;
@@ -122,7 +124,7 @@ struct FilterList *load_filter_list(void) {
 		}
 	}
 
-	phree(serialized_list);
+	free(serialized_list);
 	return newlist;
 }
 
@@ -130,7 +132,7 @@ struct FilterList *load_filter_list(void) {
 void free_filter_list(struct FilterList *fl) {
 	if (fl == NULL) return;
 	free_filter_list(fl->next);
-	phree(fl);
+	free(fl);
 }
 
 
@@ -200,7 +202,7 @@ void read_network_map(void) {
 	/* Use the string tokenizer to grab one line at a time */
 	for (i=0; i<num_tokens(serialized_map, '\n'); ++i) {
 		extract_token(buf, serialized_map, i, '\n');
-		nmptr = (struct NetMap *) mallok(sizeof(struct NetMap));
+		nmptr = (struct NetMap *) malloc(sizeof(struct NetMap));
 		extract(nmptr->nodename, buf, 0);
 		nmptr->lastcontact = extract_long(buf, 1);
 		extract(nmptr->nexthop, buf, 2);
@@ -208,7 +210,7 @@ void read_network_map(void) {
 		the_netmap = nmptr;
 	}
 
-	phree(serialized_map);
+	free(serialized_map);
 }
 
 
@@ -219,11 +221,11 @@ void write_network_map(void) {
 	char *serialized_map = NULL;
 	struct NetMap *nmptr;
 
-	serialized_map = strdoop("");
+	serialized_map = strdup("");
 
 	if (the_netmap != NULL) {
 		for (nmptr = the_netmap; nmptr != NULL; nmptr = nmptr->next) {
-			serialized_map = reallok(serialized_map,
+			serialized_map = realloc(serialized_map,
 						(strlen(serialized_map)+SIZ) );
 			if (strlen(nmptr->nodename) > 0) {
 				snprintf(&serialized_map[strlen(serialized_map)],
@@ -237,12 +239,12 @@ void write_network_map(void) {
 	}
 
 	CtdlPutSysConfig(IGNETMAP, serialized_map);
-	phree(serialized_map);
+	free(serialized_map);
 
 	/* Now free the list */
 	while (the_netmap != NULL) {
 		nmptr = the_netmap->next;
-		phree(the_netmap);
+		free(the_netmap);
 		the_netmap = nmptr;
 	}
 }
@@ -256,7 +258,6 @@ void write_network_map(void) {
  * shared secret.
  */
 int is_valid_node(char *nexthop, char *secret, char *node) {
-	char *ignetcfg = NULL;
 	int i;
 	char linebuf[SIZ];
 	char buf[SIZ];
@@ -270,7 +271,6 @@ int is_valid_node(char *nexthop, char *secret, char *node) {
 	/*
 	 * First try the neighbor nodes
 	 */
-	ignetcfg = CtdlGetSysConfig(IGNETCFG);
 	if (ignetcfg == NULL) {
 		if (nexthop != NULL) {
 			strcpy(nexthop, "");
@@ -298,7 +298,6 @@ int is_valid_node(char *nexthop, char *secret, char *node) {
 		}
 	}
 
-	phree(ignetcfg);
 	if (retval == 0) {
 		return(retval);		/* yup, it's a direct neighbor */
 	}
@@ -320,7 +319,7 @@ int is_valid_node(char *nexthop, char *secret, char *node) {
 	/*
 	 * If we get to this point, the supplied node name is bogus.
 	 */
-	lprintf(5, "Invalid node name <%s>\n", node);
+	lprintf(CTDL_ERR, "Invalid node name <%s>\n", node);
 	return(-1);
 }
 
@@ -365,7 +364,7 @@ void cmd_snet(char *argbuf) {
 	fp = fopen(tempfilename, "w");
 	if (fp == NULL) {
 		cprintf("%d Cannot open %s: %s\n",
-			ERROR+INTERNAL_ERROR,
+			ERROR + INTERNAL_ERROR,
 			tempfilename,
 			strerror(errno));
 	}
@@ -430,10 +429,10 @@ void network_spool_msg(long msgnum, void *userdata) {
 		/*
 	 	 * allocate...
 	 	 */
-		lprintf(9, "Generating delivery instructions\n");
-		instr = mallok(instr_len);
+		lprintf(CTDL_DEBUG, "Generating delivery instructions\n");
+		instr = malloc(instr_len);
 		if (instr == NULL) {
-			lprintf(1, "Cannot allocate %ld bytes for instr...\n",
+			lprintf(CTDL_EMERG, "Cannot allocate %ld bytes for instr...\n",
 				(long)instr_len);
 			abort();
 		}
@@ -452,12 +451,12 @@ void network_spool_msg(long msgnum, void *userdata) {
 		/*
 	 	 * Generate a message from the instructions
 	 	 */
-       		imsg = mallok(sizeof(struct CtdlMessage));
+       		imsg = malloc(sizeof(struct CtdlMessage));
 		memset(imsg, 0, sizeof(struct CtdlMessage));
 		imsg->cm_magic = CTDLMESSAGE_MAGIC;
 		imsg->cm_anon_type = MES_NORMAL;
 		imsg->cm_format_type = FMT_RFC822;
-		imsg->cm_fields['A'] = strdoop("Citadel");
+		imsg->cm_fields['A'] = strdup("Citadel");
 		imsg->cm_fields['M'] = instr;
 	
 		/* Save delivery instructions in spoolout room */
@@ -491,14 +490,14 @@ void network_spool_msg(long msgnum, void *userdata) {
 			 * sending a message to another IGnet node
 			 */
 			if (msg->cm_fields['P'] == NULL) {
-				msg->cm_fields['P'] = strdoop("username");
+				msg->cm_fields['P'] = strdup("username");
 			}
 			newpath_len = strlen(msg->cm_fields['P']) +
 				 strlen(config.c_nodename) + 2;
-			newpath = mallok(newpath_len);
+			newpath = malloc(newpath_len);
 			snprintf(newpath, newpath_len, "%s!%s",
 				 config.c_nodename, msg->cm_fields['P']);
-			phree(msg->cm_fields['P']);
+			free(msg->cm_fields['P']);
 			msg->cm_fields['P'] = newpath;
 
 			/*
@@ -506,9 +505,9 @@ void network_spool_msg(long msgnum, void *userdata) {
 			 * on the far end by setting the C field correctly
 			 */
 			if (msg->cm_fields['C'] != NULL) {
-				phree(msg->cm_fields['C']);
+				free(msg->cm_fields['C']);
 			}
-			msg->cm_fields['C'] = strdoop(CC->room.QRname);
+			msg->cm_fields['C'] = strdup(CC->room.QRname);
 
 			/*
 			 * Determine if this message is set to be deleted
@@ -534,13 +533,13 @@ void network_spool_msg(long msgnum, void *userdata) {
 
 				/* Check for valid node name */
 				if (is_valid_node(NULL,NULL,nptr->name) != 0) {
-					lprintf(3, "Invalid node <%s>\n",
+					lprintf(CTDL_ERR, "Invalid node <%s>\n",
 						nptr->name);
 					send = 0;
 				}
 
 				/* Check for split horizon */
-				lprintf(9, "Path is %s\n", msg->cm_fields['P']);
+				lprintf(CTDL_DEBUG, "Path is %s\n", msg->cm_fields['P']);
 				bang = num_tokens(msg->cm_fields['P'], '!');
 				if (bang > 1) for (i=0; i<(bang-1); ++i) {
 					extract_token(buf, msg->cm_fields['P'],
@@ -563,7 +562,7 @@ void network_spool_msg(long msgnum, void *userdata) {
 					}
 				}
 			}
-			phree(sermsg.ser);
+			free(sermsg.ser);
 			CtdlFreeMessage(msg);
 		}
 	}
@@ -599,27 +598,27 @@ void network_deliver_digest(struct SpoolControl *sc) {
 		return;
 	}
 
-	msg = mallok(sizeof(struct CtdlMessage));
+	msg = malloc(sizeof(struct CtdlMessage));
 	memset(msg, 0, sizeof(struct CtdlMessage));
 	msg->cm_magic = CTDLMESSAGE_MAGIC;
 	msg->cm_format_type = FMT_RFC822;
 	msg->cm_anon_type = MES_NORMAL;
 
 	sprintf(buf, "%ld", time(NULL));
-	msg->cm_fields['T'] = strdoop(buf);
-	msg->cm_fields['A'] = strdoop(CC->room.QRname);
-	msg->cm_fields['U'] = strdoop(CC->room.QRname);
+	msg->cm_fields['T'] = strdup(buf);
+	msg->cm_fields['A'] = strdup(CC->room.QRname);
+	msg->cm_fields['U'] = strdup(CC->room.QRname);
 	sprintf(buf, "room_%s@%s", CC->room.QRname, config.c_fqdn);
 	for (i=0; i<strlen(buf); ++i) {
 		if (isspace(buf[i])) buf[i]='_';
 		buf[i] = tolower(buf[i]);
 	}
-	msg->cm_fields['F'] = strdoop(buf);
+	msg->cm_fields['F'] = strdup(buf);
 
 	fseek(sc->digestfp, 0L, SEEK_END);
 	msglen = ftell(sc->digestfp);
 
-	msg->cm_fields['M'] = mallok(msglen + 1);
+	msg->cm_fields['M'] = malloc(msglen + 1);
 	fseek(sc->digestfp, 0L, SEEK_SET);
 	fread(msg->cm_fields['M'], (size_t)msglen, 1, sc->digestfp);
 	msg->cm_fields['M'][msglen] = 0;
@@ -642,10 +641,10 @@ void network_deliver_digest(struct SpoolControl *sc) {
 	/*
  	 * allocate...
  	 */
-	lprintf(9, "Generating delivery instructions\n");
-	instr = mallok(instr_len);
+	lprintf(CTDL_DEBUG, "Generating delivery instructions\n");
+	instr = malloc(instr_len);
 	if (instr == NULL) {
-		lprintf(1, "Cannot allocate %ld bytes for instr...\n",
+		lprintf(CTDL_EMERG, "Cannot allocate %ld bytes for instr...\n",
 			(long)instr_len);
 		abort();
 	}
@@ -664,12 +663,12 @@ void network_deliver_digest(struct SpoolControl *sc) {
 	/*
  	 * Generate a message from the instructions
  	 */
-  	imsg = mallok(sizeof(struct CtdlMessage));
+  	imsg = malloc(sizeof(struct CtdlMessage));
 	memset(imsg, 0, sizeof(struct CtdlMessage));
 	imsg->cm_magic = CTDLMESSAGE_MAGIC;
 	imsg->cm_anon_type = MES_NORMAL;
 	imsg->cm_format_type = FMT_RFC822;
-	imsg->cm_fields['A'] = strdoop("Citadel");
+	imsg->cm_fields['A'] = strdup("Citadel");
 	imsg->cm_fields['M'] = instr;
 
 	/* Save delivery instructions in spoolout room */
@@ -687,14 +686,14 @@ void network_spoolout_room(char *room_to_spool) {
 	char instr[SIZ];
 	FILE *fp;
 	struct SpoolControl sc;
-	struct namelist *nptr;
+	struct namelist *nptr = NULL;
 	size_t miscsize = 0;
 	size_t linesize = 0;
 	int skipthisline = 0;
 	int i;
 
 	if (getroom(&CC->room, room_to_spool) != 0) {
-		lprintf(1, "ERROR: cannot load <%s>\n", room_to_spool);
+		lprintf(CTDL_CRIT, "ERROR: cannot load <%s>\n", room_to_spool);
 		return;
 	}
 
@@ -710,7 +709,7 @@ void network_spoolout_room(char *room_to_spool) {
 		return;
 	}
 
-	lprintf(5, "Networking started for <%s>\n", CC->room.QRname);
+	lprintf(CTDL_INFO, "Networking started for <%s>\n", CC->room.QRname);
 
 	while (fgets(buf, sizeof buf, fp) != NULL) {
 		buf[strlen(buf)-1] = 0;
@@ -721,21 +720,21 @@ void network_spoolout_room(char *room_to_spool) {
 		}
 		else if (!strcasecmp(instr, "listrecp")) {
 			nptr = (struct namelist *)
-				mallok(sizeof(struct namelist));
+				malloc(sizeof(struct namelist));
 			nptr->next = sc.listrecps;
 			extract(nptr->name, buf, 1);
 			sc.listrecps = nptr;
 		}
 		else if (!strcasecmp(instr, "digestrecp")) {
 			nptr = (struct namelist *)
-				mallok(sizeof(struct namelist));
+				malloc(sizeof(struct namelist));
 			nptr->next = sc.digestrecps;
 			extract(nptr->name, buf, 1);
 			sc.digestrecps = nptr;
 		}
 		else if (!strcasecmp(instr, "ignet_push_share")) {
 			nptr = (struct namelist *)
-				mallok(sizeof(struct namelist));
+				malloc(sizeof(struct namelist));
 			nptr->next = sc.ignet_push_shares;
 			extract(nptr->name, buf, 1);
 			sc.ignet_push_shares = nptr;
@@ -802,7 +801,7 @@ void network_spoolout_room(char *room_to_spool) {
 	/* Now rewrite the config file */
 	fp = fopen(filename, "w");
 	if (fp == NULL) {
-		lprintf(1, "ERROR: cannot open %s: %s\n",
+		lprintf(CTDL_CRIT, "ERROR: cannot open %s: %s\n",
 			filename, strerror(errno));
 	}
 	else {
@@ -814,27 +813,33 @@ void network_spoolout_room(char *room_to_spool) {
 		while (sc.listrecps != NULL) {
 			fprintf(fp, "listrecp|%s\n", sc.listrecps->name);
 			nptr = sc.listrecps->next;
-			phree(sc.listrecps);
+			free(sc.listrecps);
 			sc.listrecps = nptr;
 		}
 		/* Do the same for digestrecps */
 		while (sc.digestrecps != NULL) {
 			fprintf(fp, "digestrecp|%s\n", sc.digestrecps->name);
 			nptr = sc.digestrecps->next;
-			phree(sc.digestrecps);
+			free(sc.digestrecps);
 			sc.digestrecps = nptr;
 		}
 		while (sc.ignet_push_shares != NULL) {
-			fprintf(fp, "ignet_push_share|%s\n",
-				sc.ignet_push_shares->name);
+			/* by checking each node's validity, we automatically
+			 * purge nodes which do not exist from room network
+			 * configurations at this time.
+			 */
+			if (is_valid_node(NULL, NULL, nptr->name) == 0) {
+				fprintf(fp, "ignet_push_share|%s\n",
+					sc.ignet_push_shares->name);
+			}
 			nptr = sc.ignet_push_shares->next;
-			phree(sc.ignet_push_shares);
+			free(sc.ignet_push_shares);
 			sc.ignet_push_shares = nptr;
 		}
 		if (sc.misc != NULL) {
 			fwrite(sc.misc, strlen(sc.misc), 1, fp);
 		}
-		phree(sc.misc);
+		free(sc.misc);
 
 		fclose(fp);
 	}
@@ -842,13 +847,61 @@ void network_spoolout_room(char *room_to_spool) {
 }
 
 
+
+/*
+ * Send the *entire* contents of the current room to one specific network node,
+ * ignoring anything we know about which messages have already undergone
+ * network processing.  This can be used to bring a new node into sync.
+ */
+int network_sync_to(char *target_node) {
+	struct SpoolControl sc;
+	int num_spooled = 0;
+
+	/* Concise syntax because we don't need a full linked-list */
+	memset(&sc, 0, sizeof(struct SpoolControl));
+	sc.ignet_push_shares = (struct namelist *)
+		malloc(sizeof(struct namelist));
+	sc.ignet_push_shares->next = NULL;
+	safestrncpy(sc.ignet_push_shares->name,
+		target_node,
+		sizeof sc.ignet_push_shares->name);
+
+	/* Send ALL messages */
+	num_spooled = CtdlForEachMessage(MSGS_ALL, 0L, NULL, NULL,
+		network_spool_msg, &sc);
+
+	/* Concise cleanup because we know there's only one node in the sc */
+	free(sc.ignet_push_shares);
+
+	lprintf(CTDL_INFO, "Synchronized %d messages to <%s>\n",
+		num_spooled, target_node);
+	return(num_spooled);
+}
+
+
+/*
+ * Implements the NSYN command
+ */
+void cmd_nsyn(char *argbuf) {
+	int num_spooled;
+	char target_node[SIZ];
+
+	if (CtdlAccessCheck(ac_aide)) return;
+
+	extract(target_node, argbuf, 0);
+	num_spooled = network_sync_to(target_node);
+	cprintf("%d Spooled %d messages.\n", CIT_OK, num_spooled);
+}
+
+
+
 /*
  * Batch up and send all outbound traffic from the current room
  */
 void network_queue_room(struct ctdlroom *qrbuf, void *data) {
 	struct RoomProcList *ptr;
 
-	ptr = (struct RoomProcList *) mallok(sizeof (struct RoomProcList));
+	ptr = (struct RoomProcList *) malloc(sizeof (struct RoomProcList));
 	if (ptr == NULL) return;
 
 	safestrncpy(ptr->name, qrbuf->QRname, sizeof ptr->name);
@@ -876,7 +929,7 @@ void network_learn_topology(char *node, char *path) {
 	}
 
 	/* If we got here then it's not in the map, so add it. */
-	nmptr = (struct NetMap *) mallok(sizeof (struct NetMap));
+	nmptr = (struct NetMap *) malloc(sizeof (struct NetMap));
 	strcpy(nmptr->nodename, node);
 	nmptr->lastcontact = time(NULL);
 	extract_token(nmptr->nexthop, path, 0, '!');
@@ -900,7 +953,7 @@ void network_bounce(struct CtdlMessage *msg, char *reason) {
 	static int serialnum = 0;
 	size_t size;
 
-	lprintf(9, "entering network_bounce()\n");
+	lprintf(CTDL_DEBUG, "entering network_bounce()\n");
 
 	if (msg == NULL) return;
 
@@ -910,46 +963,46 @@ void network_bounce(struct CtdlMessage *msg, char *reason) {
 	 * Give it a fresh message ID
 	 */
 	if (msg->cm_fields['I'] != NULL) {
-		phree(msg->cm_fields['I']);
+		free(msg->cm_fields['I']);
 	}
 	snprintf(buf, sizeof buf, "%ld.%04lx.%04x@%s",
 		(long)time(NULL), (long)getpid(), ++serialnum, config.c_fqdn);
-	msg->cm_fields['I'] = strdoop(buf);
+	msg->cm_fields['I'] = strdup(buf);
 
 	/*
 	 * FIXME ... right now we're just sending a bounce; we really want to
 	 * include the text of the bounced message.
 	 */
 	if (msg->cm_fields['M'] != NULL) {
-		phree(msg->cm_fields['M']);
+		free(msg->cm_fields['M']);
 	}
-	msg->cm_fields['M'] = strdoop(reason);
+	msg->cm_fields['M'] = strdup(reason);
 	msg->cm_format_type = 0;
 
 	/*
 	 * Turn the message around
 	 */
 	if (msg->cm_fields['R'] == NULL) {
-		phree(msg->cm_fields['R']);
+		free(msg->cm_fields['R']);
 	}
 
 	if (msg->cm_fields['D'] == NULL) {
-		phree(msg->cm_fields['D']);
+		free(msg->cm_fields['D']);
 	}
 
 	snprintf(recipient, sizeof recipient, "%s@%s",
 		msg->cm_fields['A'], msg->cm_fields['N']);
 
 	if (msg->cm_fields['A'] == NULL) {
-		phree(msg->cm_fields['A']);
+		free(msg->cm_fields['A']);
 	}
 
 	if (msg->cm_fields['N'] == NULL) {
-		phree(msg->cm_fields['N']);
+		free(msg->cm_fields['N']);
 	}
 
-	msg->cm_fields['A'] = strdoop(BOUNCESOURCE);
-	msg->cm_fields['N'] = strdoop(config.c_nodename);
+	msg->cm_fields['A'] = strdup(BOUNCESOURCE);
+	msg->cm_fields['N'] = strdup(config.c_nodename);
 	
 
 	/* prepend our node to the path */
@@ -958,17 +1011,17 @@ void network_bounce(struct CtdlMessage *msg, char *reason) {
 		msg->cm_fields['P'] = NULL;
 	}
 	else {
-		oldpath = strdoop("unknown_user");
+		oldpath = strdup("unknown_user");
 	}
 	size = strlen(oldpath) + SIZ;
-	msg->cm_fields['P'] = mallok(size);
+	msg->cm_fields['P'] = malloc(size);
 	snprintf(msg->cm_fields['P'], size, "%s!%s", config.c_nodename, oldpath);
-	phree(oldpath);
+	free(oldpath);
 
 	/* Now submit the message */
 	valid = validate_recipients(recipient);
 	if (valid != NULL) if (valid->num_error > 0) {
-		phree(valid);
+		free(valid);
 		valid = NULL;
 	}
 	if ( (valid == NULL) || (!strcasecmp(recipient, bouncesource)) ) {
@@ -983,9 +1036,9 @@ void network_bounce(struct CtdlMessage *msg, char *reason) {
 	CtdlSubmitMsg(msg, valid, force_room);
 
 	/* Clean up */
-	if (valid != NULL) phree(valid);
+	if (valid != NULL) free(valid);
 	CtdlFreeMessage(msg);
-	lprintf(9, "leaving network_bounce()\n");
+	lprintf(CTDL_DEBUG, "leaving network_bounce()\n");
 }
 
 
@@ -1005,14 +1058,25 @@ void network_process_buffer(char *buffer, long size) {
 	char *oldpath = NULL;
 	char filename[SIZ];
 	FILE *fp;
-	char buf[SIZ];
 	char nexthop[SIZ];
+	unsigned char firstbyte;
+	unsigned char lastbyte;
+
+	/* Validate just a little bit.  First byte should be FF and
+	 * last byte should be 00.
+	 */
+	memcpy(&firstbyte, &buffer[0], 1);
+	memcpy(&lastbyte, &buffer[size-1], 1);
+	if ( (firstbyte != 255) || (lastbyte != 0) ) {
+		lprintf(CTDL_ERR, "Corrupt message!  Ignoring.\n");
+		return;
+	}
 
 	/* Set default target room to trash */
 	strcpy(target_room, TWITROOM);
 
 	/* Load the message into memory */
-        msg = (struct CtdlMessage *) mallok(sizeof(struct CtdlMessage));
+        msg = (struct CtdlMessage *) malloc(sizeof(struct CtdlMessage));
         memset(msg, 0, sizeof(struct CtdlMessage));
         msg->cm_magic = CTDLMESSAGE_MAGIC;
         msg->cm_anon_type = buffer[1];
@@ -1020,7 +1084,7 @@ void network_process_buffer(char *buffer, long size) {
 
 	for (pos = 3; pos < size; ++pos) {
 		field = buffer[pos];
-		msg->cm_fields[field] = strdoop(&buffer[pos+1]);
+		msg->cm_fields[field] = strdup(&buffer[pos+1]);
 		pos = pos + strlen(&buffer[(int)pos]);
 	}
 
@@ -1039,13 +1103,13 @@ void network_process_buffer(char *buffer, long size) {
 					msg->cm_fields['P'] = NULL;
 				}
 				else {
-					oldpath = strdoop("unknown_user");
+					oldpath = strdup("unknown_user");
 				}
 				size = strlen(oldpath) + SIZ;
-				msg->cm_fields['P'] = mallok(size);
+				msg->cm_fields['P'] = malloc(size);
 				snprintf(msg->cm_fields['P'], size, "%s!%s",
 					config.c_nodename, oldpath);
-				phree(oldpath);
+				free(oldpath);
 
 				/* serialize the message */
 				serialize_message(&sermsg, msg);
@@ -1062,7 +1126,7 @@ void network_process_buffer(char *buffer, long size) {
 						sermsg.len, 1, fp);
 					fclose(fp);
 				}
-				phree(sermsg.ser);
+				free(sermsg.ser);
 				CtdlFreeMessage(msg);
 				return;
 			}
@@ -1080,20 +1144,12 @@ void network_process_buffer(char *buffer, long size) {
 	}
 
 	/*
-	 * Check to see if we already have a copy of this message
+	 * Check to see if we already have a copy of this message, and
+	 * abort its processing if so.  (We used to post a warning to Aide>
+	 * every time this happened, but the network is now so densely
+	 * connected that it's inevitable.)
 	 */
 	if (network_usetable(msg) != 0) {
-		snprintf(buf, sizeof buf,
-			"Loopzapper rejected message <%s> "
-			"from <%s> in <%s> @ <%s>\n",
-			((msg->cm_fields['I']!=NULL)?(msg->cm_fields['I']):""),
-			((msg->cm_fields['A']!=NULL)?(msg->cm_fields['A']):""),
-			((msg->cm_fields['O']!=NULL)?(msg->cm_fields['O']):""),
-			((msg->cm_fields['N']!=NULL)?(msg->cm_fields['N']):"")
-		);
-		aide_message(buf);
-		CtdlFreeMessage(msg);
-		msg = NULL;
 		return;
 	}
 
@@ -1111,7 +1167,7 @@ void network_process_buffer(char *buffer, long size) {
 "A message you sent could not be delivered due to an invalid address.\n"
 "Please check the address and try sending the message again.\n");
 			msg = NULL;
-			phree(recp);
+			free(recp);
 			return;
                 }
 		strcpy(target_room, "");	/* no target room if mail */
@@ -1131,11 +1187,11 @@ void network_process_buffer(char *buffer, long size) {
 
 	/* Strip out fields that are only relevant during transit */
 	if (msg->cm_fields['D'] != NULL) {
-		phree(msg->cm_fields['D']);
+		free(msg->cm_fields['D']);
 		msg->cm_fields['D'] = NULL;
 	}
 	if (msg->cm_fields['C'] != NULL) {
-		phree(msg->cm_fields['C']);
+		free(msg->cm_fields['C']);
 		msg->cm_fields['C'] = NULL;
 	}
 
@@ -1145,7 +1201,7 @@ void network_process_buffer(char *buffer, long size) {
         	CtdlSubmitMsg(msg, recp, target_room);
 	}
 	CtdlFreeMessage(msg);
-	phree(recp);
+	free(recp);
 }
 
 
@@ -1159,12 +1215,12 @@ void network_process_message(FILE *fp, long msgstart, long msgend) {
 
 	hold_pos = ftell(fp);
 	size = msgend - msgstart + 1;
-	buffer = mallok(size);
+	buffer = malloc(size);
 	if (buffer != NULL) {
 		fseek(fp, msgstart, SEEK_SET);
 		fread(buffer, size, 1, fp);
 		network_process_buffer(buffer, size);
-		phree(buffer);
+		free(buffer);
 	}
 
 	fseek(fp, hold_pos, SEEK_SET);
@@ -1181,15 +1237,16 @@ void network_process_file(char *filename) {
 	long msgcur = 0L;
 	int ch;
 
-	lprintf(7, "network: processing <%s>\n", filename);
 
 	fp = fopen(filename, "rb");
 	if (fp == NULL) {
-		lprintf(5, "Error opening %s: %s\n",
+		lprintf(CTDL_CRIT, "Error opening %s: %s\n",
 			filename, strerror(errno));
 		return;
 	}
 
+	lprintf(CTDL_INFO, "network: processing <%s>\n", filename);
+
 	/* Look for messages in the data stream and break them out */
 	while (ch = getc(fp), ch >= 0) {
 	
@@ -1226,7 +1283,8 @@ void network_do_spoolin(void) {
 	if (dp == NULL) return;
 
 	while (d = readdir(dp), d != NULL) {
-		snprintf(filename, sizeof filename, "./network/spoolin/%s", d->d_name);
+		snprintf(filename, sizeof filename,
+			"./network/spoolin/%s", d->d_name);
 		network_process_file(filename);
 	}
 
@@ -1235,6 +1293,35 @@ void network_do_spoolin(void) {
 }
 
 
+/*
+ * Delete any files in the outbound queue that were intended
+ * to be sent to nodes which no nlonger exist.
+ */
+void network_purge_spoolout(void) {
+	DIR *dp;
+	struct dirent *d;
+	char filename[SIZ];
+	char nexthop[SIZ];
+	int i;
+
+	dp = opendir("./network/spoolout");
+	if (dp == NULL) return;
+
+	while (d = readdir(dp), d != NULL) {
+		snprintf(filename, sizeof filename,
+			"./network/spoolout/%s", d->d_name);
+
+		strcpy(nexthop, "");
+		i = is_valid_node(nexthop, NULL, d->d_name);
+	
+		if ( (i != 0) || (strlen(nexthop) > 0) ) {
+			unlink(filename);
+		}
+	}
+
+
+	closedir(dp);
+}
 
 
 
@@ -1253,7 +1340,7 @@ void receive_spool(int sock, char *remote_nodename) {
 	strcpy(tempfilename, tmpnam(NULL));
 	if (sock_puts(sock, "NDOP") < 0) return;
 	if (sock_gets(sock, buf) < 0) return;
-	lprintf(9, "<%s\n", buf);
+	lprintf(CTDL_DEBUG, "<%s\n", buf);
 	if (buf[0] != '2') {
 		return;
 	}
@@ -1262,7 +1349,7 @@ void receive_spool(int sock, char *remote_nodename) {
 	bytes_received = 0L;
 	fp = fopen(tempfilename, "w");
 	if (fp == NULL) {
-		lprintf(9, "cannot open download file locally: %s\n",
+		lprintf(CTDL_CRIT, "cannot open download file locally: %s\n",
 			strerror(errno));
 		return;
 	}
@@ -1303,7 +1390,7 @@ void receive_spool(int sock, char *remote_nodename) {
 		unlink(tempfilename);
 		return;
 	}
-	lprintf(9, "%s\n", buf);
+	lprintf(CTDL_DEBUG, "%s\n", buf);
 	snprintf(buf, sizeof buf, "mv %s ./network/spoolin/%s.%ld",
 		tempfilename, remote_nodename, (long) getpid());
 	system(buf);
@@ -1325,7 +1412,7 @@ void transmit_spool(int sock, char *remote_nodename)
 
 	if (sock_puts(sock, "NUOP") < 0) return;
 	if (sock_gets(sock, buf) < 0) return;
-	lprintf(9, "<%s\n", buf);
+	lprintf(CTDL_DEBUG, "<%s\n", buf);
 	if (buf[0] != '2') {
 		return;
 	}
@@ -1334,9 +1421,9 @@ void transmit_spool(int sock, char *remote_nodename)
 	fd = open(sfname, O_RDONLY);
 	if (fd < 0) {
 		if (errno == ENOENT) {
-			lprintf(9, "Nothing to send.\n");
+			lprintf(CTDL_INFO, "Nothing to send.\n");
 		} else {
-			lprintf(5, "cannot open upload file locally: %s\n",
+			lprintf(CTDL_CRIT, "cannot open upload file locally: %s\n",
 				strerror(errno));
 		}
 		return;
@@ -1371,7 +1458,7 @@ ABORTUPL:
 	close(fd);
 	if (sock_puts(sock, "UCLS 1") < 0) return;
 	if (sock_gets(sock, buf) < 0) return;
-	lprintf(9, "<%s\n", buf);
+	lprintf(CTDL_DEBUG, "<%s\n", buf);
 	if (buf[0] == '2') {
 		unlink(sfname);
 	}
@@ -1388,27 +1475,27 @@ void network_poll_node(char *node, char *secret, char *host, char *port) {
 
 	if (network_talking_to(node, NTT_CHECK)) return;
 	network_talking_to(node, NTT_ADD);
-	lprintf(5, "Polling node <%s> at %s:%s\n", node, host, port);
+	lprintf(CTDL_INFO, "Polling node <%s> at %s:%s\n", node, host, port);
 
 	sock = sock_connect(host, port, "tcp");
 	if (sock < 0) {
-		lprintf(7, "Could not connect: %s\n", strerror(errno));
+		lprintf(CTDL_ERR, "Could not connect: %s\n", strerror(errno));
 		network_talking_to(node, NTT_REMOVE);
 		return;
 	}
 	
-	lprintf(9, "Connected!\n");
+	lprintf(CTDL_DEBUG, "Connected!\n");
 
 	/* Read the server greeting */
 	if (sock_gets(sock, buf) < 0) goto bail;
-	lprintf(9, ">%s\n", buf);
+	lprintf(CTDL_DEBUG, ">%s\n", buf);
 
 	/* Identify ourselves */
 	snprintf(buf, sizeof buf, "NETP %s|%s", config.c_nodename, secret);
-	lprintf(9, "<%s\n", buf);
+	lprintf(CTDL_DEBUG, "<%s\n", buf);
 	if (sock_puts(sock, buf) <0) goto bail;
 	if (sock_gets(sock, buf) < 0) goto bail;
-	lprintf(9, ">%s\n", buf);
+	lprintf(CTDL_DEBUG, ">%s\n", buf);
 	if (buf[0] != '2') goto bail;
 
 	/* At this point we are authenticated. */
@@ -1428,7 +1515,6 @@ bail:	sock_close(sock);
  * only nodes to which we have data to send.
  */
 void network_poll_other_citadel_nodes(int full_poll) {
-	char *ignetcfg = NULL;
 	int i;
 	char linebuf[SIZ];
 	char node[SIZ];
@@ -1438,7 +1524,6 @@ void network_poll_other_citadel_nodes(int full_poll) {
 	int poll = 0;
 	char spoolfile[SIZ];
 
-	ignetcfg = CtdlGetSysConfig(IGNETCFG);
 	if (ignetcfg == NULL) return; 	/* no nodes defined */
 
 	/* Use the string tokenizer to grab one line at a time */
@@ -1464,7 +1549,6 @@ void network_poll_other_citadel_nodes(int full_poll) {
 		}
 	}
 
-	phree(ignetcfg);
 }
 
 
@@ -1500,6 +1584,11 @@ void network_do_queue(void) {
 	if (doing_queue) return;
 	doing_queue = 1;
 
+	/* Load the IGnet Configuration into memory */
+	if (ignetcfg == NULL) {
+		ignetcfg = CtdlGetSysConfig(IGNETCFG);
+	}
+
 	/*
 	 * Poll other Citadel nodes.  Maybe.  If "full_processing" is set
 	 * then we poll everyone.  Otherwise we only poll nodes we have stuff
@@ -1517,19 +1606,19 @@ void network_do_queue(void) {
 	 * Go ahead and run the queue
 	 */
 	if (full_processing) {
-		lprintf(7, "network: loading outbound queue\n");
+		lprintf(CTDL_INFO, "network: loading outbound queue\n");
 		ForEachRoom(network_queue_room, NULL);
 
-		lprintf(7, "network: running outbound queue\n");
+		lprintf(CTDL_INFO, "network: running outbound queue\n");
 		while (rplist != NULL) {
 			network_spoolout_room(rplist->name);
 			ptr = rplist;
 			rplist = rplist->next;
-			phree(ptr);
+			free(ptr);
 		}
 	}
 
-	lprintf(7, "network: processing inbound queue\n");
+	lprintf(CTDL_INFO, "network: processing inbound queue\n");
 	network_do_spoolin();
 
 	/* Save the network map back to disk */
@@ -1539,7 +1628,9 @@ void network_do_queue(void) {
 	free_filter_list(filterlist);
 	filterlist = NULL;
 
-	lprintf(7, "network: queue run completed\n");
+	network_purge_spoolout();
+
+	lprintf(CTDL_INFO, "network: queue run completed\n");
 
 	if (full_processing) {
 		last_run = time(NULL);
@@ -1562,7 +1653,7 @@ void cmd_netp(char *cmdbuf)
 	char nexthop[SIZ];
 
 	if (doing_queue) {
-		cprintf("%d spooling - try again in a few minutes\n", ERROR);
+		cprintf("%d spooling - try again in a few minutes\n", ERROR + RESOURCE_BUSY);
 		return;
 	}
 
@@ -1570,17 +1661,17 @@ void cmd_netp(char *cmdbuf)
 	extract(pass, cmdbuf, 1);
 
 	if (is_valid_node(nexthop, secret, node) != 0) {
-		cprintf("%d authentication failed\n", ERROR);
+		cprintf("%d authentication failed\n", ERROR + PASSWORD_REQUIRED);
 		return;
 	}
 
 	if (strcasecmp(pass, secret)) {
-		cprintf("%d authentication failed\n", ERROR);
+		cprintf("%d authentication failed\n", ERROR + PASSWORD_REQUIRED);
 		return;
 	}
 
 	if (network_talking_to(node, NTT_CHECK)) {
-		cprintf("%d Already talking to %s right now\n", ERROR, node);
+		cprintf("%d Already talking to %s right now\n", ERROR + RESOURCE_BUSY, node);
 		return;
 	}
 
@@ -1591,6 +1682,41 @@ void cmd_netp(char *cmdbuf)
 }
 
 
+/*
+ * This handler detects changes being made to the system's IGnet
+ * configuration.
+ */
+int netconfig_aftersave(struct CtdlMessage *msg) {
+	char *ptr;
+	int linelen;
+
+	/* If this isn't the configuration room, or if this isn't a MIME
+	 * message, don't bother.
+	 */
+	if (strcasecmp(msg->cm_fields['O'], SYSCONFIGROOM)) return(0);
+	if (msg->cm_format_type != 4) return(0);
+
+	ptr = msg->cm_fields['M'];
+	while (ptr != NULL) {
+	
+		linelen = strcspn(ptr, "\n");
+		if (linelen == 0) return(0);	/* end of headers */	
+		
+		if (!strncasecmp(ptr, "Content-type: ", 14)) {
+			if (!strncasecmp(&ptr[14], IGNETCFG,
+		   	   strlen(IGNETCFG))) {
+				if (ignetcfg != NULL) free(ignetcfg);
+				ignetcfg = NULL;
+			}
+		}
+
+		ptr = strchr((char *)ptr, '\n');
+		if (ptr != NULL) ++ptr;
+	}
+
+	return(0);
+}
+
 
 
 
@@ -1602,6 +1728,8 @@ char *serv_network_init(void)
 	CtdlRegisterProtoHook(cmd_gnet, "GNET", "Get network config");
 	CtdlRegisterProtoHook(cmd_snet, "SNET", "Set network config");
 	CtdlRegisterProtoHook(cmd_netp, "NETP", "Identify as network poller");
+	CtdlRegisterProtoHook(cmd_nsyn, "NSYN", "Synchronize room to node");
 	CtdlRegisterSessionHook(network_do_queue, EVT_TIMER);
-	return "$Id: serv_network.c,v 613.0 2003/12/15 16:37:03 ajc Exp $";
+	CtdlRegisterMessageHook(netconfig_aftersave, EVT_AFTERSAVE);
+	return "$Id: serv_network.c,v 620.0 2004/03/27 02:33:34 ajc Exp $";
 }
