@@ -1,5 +1,5 @@
 /*
- * $Id: serv_smtp.c,v 613.0 2003/12/15 16:37:03 ajc Exp $
+ * $Id: serv_smtp.c,v 620.0 2004/03/27 02:33:34 ajc Exp $
  *
  * This module is an SMTP and ESMTP implementation for the Citadel system.
  * It is compliant with all of the following:
@@ -14,6 +14,7 @@
  * RFC 2033 - Local Mail Transfer Protocol
  * RFC 2034 - SMTP Service Extension for Returning Enhanced Error Codes
  * RFC 2197 - SMTP Service Extension for Command Pipelining
+ * RFC 2487 - SMTP Service Extension for Secure SMTP over TLS
  * RFC 2554 - SMTP Service Extension for Authentication
  * RFC 2821 - Simple Mail Transfer Protocol
  * RFC 2822 - Internet Message Format
@@ -69,6 +70,11 @@
 #include "clientsocket.h"
 #include "locate_host.h"
 
+#ifdef HAVE_OPENSSL
+#include "serv_crypto.h"
+#endif
+
+
 
 #ifndef HAVE_SNPRINTF
 #include "snprintf.h"
@@ -141,6 +147,17 @@ void lmtp_greeting(void) {
 }
 
 
+/*
+ * Login greeting common to all auth methods
+ */
+void smtp_auth_greeting(void) {
+		cprintf("235 2.0.0 Hello, %s\r\n", CC->user.fullname);
+		lprintf(CTDL_NOTICE, "SMTP authenticated %s\n", CC->user.fullname);
+		CC->internal_pgm = 0;
+		CC->cs_flags &= ~CS_STEALTH;
+}
+
+
 /*
  * Implement HELO and EHLO commands.
  *
@@ -161,19 +178,46 @@ void smtp_hello(char *argbuf, int which_command) {
 	}
 
 	if (which_command == 0) {
-		cprintf("250 Greetings and joyous salutations.\r\n");
+		cprintf("250 Hello %s (%s [%s])\r\n",
+			SMTP->helo_node,
+			CC->cs_host,
+			CC->cs_addr
+		);
 	}
 	else {
-		cprintf("250-Greetings and joyous salutations.\r\n");
+		if (which_command == 1) {
+			cprintf("250-Hello %s (%s [%s])\r\n",
+				SMTP->helo_node,
+				CC->cs_host,
+				CC->cs_addr
+			);
+		}
+		else {
+			cprintf("250-Greetings and joyous salutations.\r\n");
+		}
 		cprintf("250-HELP\r\n");
 		cprintf("250-SIZE %ld\r\n", config.c_maxmsglen);
-		cprintf("250-PIPELINING\r\n");
-		cprintf("250-AUTH=LOGIN\r\n");
+
+		/* Only offer the PIPELINING command if TLS is inactive, because
+		 * of flow control issues.  Also, avoid offering TLS if TLS is
+		 * already active.
+		 */
+		if (!CC->redirect_ssl) {
+			cprintf("250-PIPELINING\r\n");
+#ifdef HAVE_OPENSSL
+			cprintf("250-STARTTLS\r\n");
+#endif
+		}
+
+		cprintf("250-AUTH LOGIN PLAIN\r\n");
+		cprintf("250-AUTH=LOGIN PLAIN\r\n");
+
 		cprintf("250 ENHANCEDSTATUSCODES\r\n");
 	}
 }
 
 
+
 /*
  * Implement HELP command.
  */
@@ -202,7 +246,7 @@ void smtp_get_user(char *argbuf) {
 	char username[SIZ];
 
 	CtdlDecodeBase64(username, argbuf, SIZ);
-	lprintf(9, "Trying <%s>\n", username);
+	lprintf(CTDL_DEBUG, "Trying <%s>\n", username);
 	if (CtdlLoginExistingUser(username) == login_ok) {
 		CtdlEncodeBase64(buf, "Password:", 9);
 		cprintf("334 %s\r\n", buf);
@@ -222,12 +266,9 @@ void smtp_get_pass(char *argbuf) {
 	char password[SIZ];
 
 	CtdlDecodeBase64(password, argbuf, SIZ);
-	lprintf(9, "Trying <%s>\n", password);
+	lprintf(CTDL_DEBUG, "Trying <%s>\n", password);
 	if (CtdlTryPassword(password) == pass_ok) {
-		cprintf("235 2.0.0 Hello, %s\r\n", CC->user.fullname);
-		lprintf(9, "SMTP authenticated login successful\n");
-		CC->internal_pgm = 0;
-		CC->cs_flags &= ~CS_STEALTH;
+		smtp_auth_greeting();
 	}
 	else {
 		cprintf("535 5.7.0 Authentication failed.\r\n");
@@ -241,21 +282,55 @@ void smtp_get_pass(char *argbuf) {
  */
 void smtp_auth(char *argbuf) {
 	char buf[SIZ];
+	char method[SIZ];
+	char encoded_authstring[SIZ];
+	char decoded_authstring[SIZ];
+	char ident[SIZ];
+	char user[SIZ];
+	char pass[SIZ];
 
-	if (strncasecmp(argbuf, "login", 5) ) {
-		cprintf("504 5.7.4 We only support LOGIN authentication.\r\n");
+	if (CC->logged_in) {
+		cprintf("504 5.7.4 Already logged in.\r\n");
 		return;
 	}
 
-	if (strlen(argbuf) >= 7) {
-		smtp_get_user(&argbuf[6]);
+	extract_token(method, argbuf, 0, ' ');
+
+	if (!strncasecmp(method, "login", 5) ) {
+		if (strlen(argbuf) >= 7) {
+			smtp_get_user(&argbuf[6]);
+		}
+		else {
+			CtdlEncodeBase64(buf, "Username:", 9);
+			cprintf("334 %s\r\n", buf);
+			SMTP->command_state = smtp_user;
+		}
+		return;
 	}
 
-	else {
-		CtdlEncodeBase64(buf, "Username:", 9);
-		cprintf("334 %s\r\n", buf);
-		SMTP->command_state = smtp_user;
+	if (!strncasecmp(method, "plain", 5) ) {
+		extract_token(encoded_authstring, argbuf, 1, ' ');
+		CtdlDecodeBase64(decoded_authstring,
+				encoded_authstring,
+				strlen(encoded_authstring) );
+		strcpy(ident, decoded_authstring);
+		strcpy(user, &decoded_authstring[strlen(ident) + 1] );
+		strcpy(pass, &decoded_authstring[strlen(ident) + strlen(user) + 2] );
+
+		if (CtdlLoginExistingUser(user) == login_ok) {
+			if (CtdlTryPassword(pass) == pass_ok) {
+				smtp_auth_greeting();
+				return;
+			}
+		}
+		cprintf("504 5.7.4 Authentication failed.\r\n");
 	}
+
+	if (strncasecmp(method, "login", 5) ) {
+		cprintf("504 5.7.4 Unknown authentication method.\r\n");
+		return;
+	}
+
 }
 
 
@@ -342,10 +417,12 @@ void smtp_expn(char *argbuf) {
 /*
  * Implements the RSET (reset state) command.
  * Currently this just zeroes out the state buffer.  If pointers to data
- * allocated with mallok() are ever placed in the state buffer, we have to
- * be sure to phree() them first!
+ * allocated with malloc() are ever placed in the state buffer, we have to
+ * be sure to free() them first!
+ *
+ * Set do_response to nonzero to output the SMTP RSET response code.
  */
-void smtp_rset(void) {
+void smtp_rset(int do_response) {
 	int is_lmtp;
 
 	/*
@@ -373,7 +450,9 @@ void smtp_rset(void) {
 	 */
 	SMTP->is_lmtp = is_lmtp;
 
-	cprintf("250 2.0.0 Zap!\r\n");
+	if (do_response) {
+		cprintf("250 2.0.0 Zap!\r\n");
+	}
 }
 
 /*
@@ -487,7 +566,7 @@ void smtp_rcpt(char *argbuf) {
 	   && (!SMTP->is_lmtp) ) {
 		if (rbl_check(message_to_spammer)) {
 			cprintf("550 %s\r\n", message_to_spammer);
-			/* no need to phree(valid), it's not allocated yet */
+			/* no need to free(valid), it's not allocated yet */
 			return;
 		}
 	}
@@ -495,7 +574,7 @@ void smtp_rcpt(char *argbuf) {
 	valid = validate_recipients(recp);
 	if (valid->num_error > 0) {
 		cprintf("599 5.1.1 Error: %s\r\n", valid->errormsg);
-		phree(valid);
+		free(valid);
 		return;
 	}
 
@@ -503,7 +582,7 @@ void smtp_rcpt(char *argbuf) {
 		if ( (SMTP->message_originated_locally == 0)
 		   && (SMTP->is_lmtp == 0) ) {
 			cprintf("551 5.7.1 <%s> - relaying denied\r\n", recp);
-			phree(valid);
+			free(valid);
 			return;
 		}
 	}
@@ -545,7 +624,7 @@ void smtp_data(void) {
 	cprintf("354 Transmit message now - terminate with '.' by itself\r\n");
 	
 	datestring(nowstamp, sizeof nowstamp, time(NULL), DATESTRING_RFC822);
-	body = mallok(4096);
+	body = malloc(4096);
 
 	if (body != NULL) snprintf(body, 4096,
 		"Received: from %s (%s [%s])\n"
@@ -563,7 +642,7 @@ void smtp_data(void) {
 		return;
 	}
 
-	lprintf(9, "Converting message...\n");
+	lprintf(CTDL_DEBUG, "Converting message...\n");
 	msg = convert_internet_message(body);
 
 	/* If the user is locally authenticated, FORCE the From: header to
@@ -577,16 +656,16 @@ void smtp_data(void) {
 	 * is read with a Citadel client.
 	 */
 	if ( (CC->logged_in) && (config.c_rfc822_strict_from == 0) ) {
-		if (msg->cm_fields['A'] != NULL) phree(msg->cm_fields['A']);
-		if (msg->cm_fields['N'] != NULL) phree(msg->cm_fields['N']);
-		if (msg->cm_fields['H'] != NULL) phree(msg->cm_fields['H']);
-		if (msg->cm_fields['F'] != NULL) phree(msg->cm_fields['F']);
-		if (msg->cm_fields['O'] != NULL) phree(msg->cm_fields['O']);
-		msg->cm_fields['A'] = strdoop(CC->user.fullname);
-		msg->cm_fields['N'] = strdoop(config.c_nodename);
-		msg->cm_fields['H'] = strdoop(config.c_humannode);
-		msg->cm_fields['F'] = strdoop(CC->cs_inet_email);
-        	msg->cm_fields['O'] = strdoop(MAILROOM);
+		if (msg->cm_fields['A'] != NULL) free(msg->cm_fields['A']);
+		if (msg->cm_fields['N'] != NULL) free(msg->cm_fields['N']);
+		if (msg->cm_fields['H'] != NULL) free(msg->cm_fields['H']);
+		if (msg->cm_fields['F'] != NULL) free(msg->cm_fields['F']);
+		if (msg->cm_fields['O'] != NULL) free(msg->cm_fields['O']);
+		msg->cm_fields['A'] = strdup(CC->user.fullname);
+		msg->cm_fields['N'] = strdup(config.c_nodename);
+		msg->cm_fields['H'] = strdup(config.c_humannode);
+		msg->cm_fields['F'] = strdup(CC->cs_inet_email);
+        	msg->cm_fields['O'] = strdup(MAILROOM);
 	}
 
 	/* Submit the message into the Citadel system. */
@@ -601,7 +680,7 @@ void smtp_data(void) {
 	if (scan_errors > 0) {	/* We don't want this message! */
 
 		if (msg->cm_fields['0'] == NULL) {
-			msg->cm_fields['0'] = strdoop(
+			msg->cm_fields['0'] = strdup(
 				"5.7.1 Message rejected by filter");
 		}
 
@@ -634,11 +713,32 @@ void smtp_data(void) {
 	}
 
 	CtdlFreeMessage(msg);
-	phree(valid);
+	free(valid);
 	smtp_data_clear();	/* clear out the buffers now */
 }
 
 
+/*
+ * implements the STARTTLS command (Citadel API version)
+ */
+#ifdef HAVE_OPENSSL
+void smtp_starttls(void)
+{
+	char ok_response[SIZ];
+	char nosup_response[SIZ];
+	char error_response[SIZ];
+
+	sprintf(ok_response,
+		"200 2.0.0 Begin TLS negotiation now\r\n");
+	sprintf(nosup_response,
+		"554 5.7.3 TLS not supported here\r\n");
+	sprintf(error_response,
+		"554 5.7.3 Internal error\r\n");
+	CtdlStartTLS(ok_response, nosup_response, error_response);
+	smtp_rset(0);
+}
+#endif
+
 
 
 /* 
@@ -650,15 +750,13 @@ void smtp_command_loop(void) {
 	time(&CC->lastcmd);
 	memset(cmdbuf, 0, sizeof cmdbuf); /* Clear it, just in case */
 	if (client_gets(cmdbuf) < 1) {
-		lprintf(3, "SMTP socket is broken.  Ending session.\n");
+		lprintf(CTDL_CRIT, "SMTP socket is broken.  Ending session.\n");
 		CC->kill_me = 1;
 		return;
 	}
-	lprintf(5, "SMTP: %s\n", cmdbuf);
+	lprintf(CTDL_INFO, "SMTP: %s\n", cmdbuf);
 	while (strlen(cmdbuf) < 5) strcat(cmdbuf, " ");
 
-	lprintf(9, "CC->logged_in = %d\n", CC->logged_in);
-
 	if (SMTP->command_state == smtp_user) {
 		smtp_get_user(cmdbuf);
 	}
@@ -714,9 +812,13 @@ void smtp_command_loop(void) {
 	}
 
 	else if (!strncasecmp(cmdbuf, "RSET", 4)) {
-		smtp_rset();
+		smtp_rset(1);
 	}
-
+#ifdef HAVE_OPENSSL
+	else if (!strcasecmp(cmdbuf, "STARTTLS")) {
+		smtp_starttls();
+	}
+#endif
 	else if (!strncasecmp(cmdbuf, "VRFY", 4)) {
 		smtp_vrfy(&cmdbuf[5]);
 	}
@@ -725,6 +827,7 @@ void smtp_command_loop(void) {
 		cprintf("502 5.0.0 I'm afraid I can't do that.\r\n");
 	}
 
+
 }
 
 
@@ -762,7 +865,7 @@ void smtp_try(const char *key, const char *addr, int *status,
 	/* Parse out the host portion of the recipient address */
 	process_rfc822_addr(addr, user, node, name);
 
-	lprintf(9, "Attempting SMTP delivery to <%s> @ <%s> (%s)\n",
+	lprintf(CTDL_DEBUG, "Attempting SMTP delivery to <%s> @ <%s> (%s)\n",
 		user, node, name);
 
 	/* Load the message out of the database into a temp file */
@@ -827,7 +930,7 @@ void smtp_try(const char *key, const char *addr, int *status,
 
 	/* Figure out what mail exchanger host we have to connect to */
 	num_mxhosts = getmx(mxhosts, node);
-	lprintf(9, "Number of MX hosts for <%s> is %d\n", node, num_mxhosts);
+	lprintf(CTDL_DEBUG, "Number of MX hosts for <%s> is %d\n", node, num_mxhosts);
 	if (num_mxhosts < 1) {
 		*status = 5;
 		snprintf(dsn, SIZ, "No MX hosts found for <%s>", node);
@@ -837,10 +940,10 @@ void smtp_try(const char *key, const char *addr, int *status,
 	sock = (-1);
 	for (mx=0; (mx<num_mxhosts && sock < 0); ++mx) {
 		extract(buf, mxhosts, mx);
-		lprintf(9, "Trying <%s>\n", buf);
+		lprintf(CTDL_DEBUG, "Trying <%s>\n", buf);
 		sock = sock_connect(buf, "25", "tcp");
 		snprintf(dsn, SIZ, "Could not connect: %s", strerror(errno));
-		if (sock >= 0) lprintf(9, "Connected!\n");
+		if (sock >= 0) lprintf(CTDL_DEBUG, "Connected!\n");
 		if (sock < 0) snprintf(dsn, SIZ, "%s", strerror(errno));
 	}
 
@@ -855,7 +958,7 @@ void smtp_try(const char *key, const char *addr, int *status,
 		strcpy(dsn, "Connection broken during SMTP conversation");
 		goto bail;
 	}
-	lprintf(9, "<%s\n", buf);
+	lprintf(CTDL_DEBUG, "<%s\n", buf);
 	if (buf[0] != '2') {
 		if (buf[0] == '4') {
 			*status = 4;
@@ -873,14 +976,14 @@ void smtp_try(const char *key, const char *addr, int *status,
 
 	/* Do a HELO command */
 	snprintf(buf, sizeof buf, "HELO %s\r\n", config.c_fqdn);
-	lprintf(9, ">%s", buf);
+	lprintf(CTDL_DEBUG, ">%s", buf);
 	sock_write(sock, buf, strlen(buf));
 	if (ml_sock_gets(sock, buf) < 0) {
 		*status = 4;
 		strcpy(dsn, "Connection broken during SMTP HELO");
 		goto bail;
 	}
-	lprintf(9, "<%s\n", buf);
+	lprintf(CTDL_DEBUG, "<%s\n", buf);
 	if (buf[0] != '2') {
 		if (buf[0] == '4') {
 			*status = 4;
@@ -897,14 +1000,14 @@ void smtp_try(const char *key, const char *addr, int *status,
 
 	/* HELO succeeded, now try the MAIL From: command */
 	snprintf(buf, sizeof buf, "MAIL From: <%s>\r\n", mailfrom);
-	lprintf(9, ">%s", buf);
+	lprintf(CTDL_DEBUG, ">%s", buf);
 	sock_write(sock, buf, strlen(buf));
 	if (ml_sock_gets(sock, buf) < 0) {
 		*status = 4;
 		strcpy(dsn, "Connection broken during SMTP MAIL");
 		goto bail;
 	}
-	lprintf(9, "<%s\n", buf);
+	lprintf(CTDL_DEBUG, "<%s\n", buf);
 	if (buf[0] != '2') {
 		if (buf[0] == '4') {
 			*status = 4;
@@ -921,14 +1024,14 @@ void smtp_try(const char *key, const char *addr, int *status,
 
 	/* MAIL succeeded, now try the RCPT To: command */
 	snprintf(buf, sizeof buf, "RCPT To: <%s>\r\n", addr);
-	lprintf(9, ">%s", buf);
+	lprintf(CTDL_DEBUG, ">%s", buf);
 	sock_write(sock, buf, strlen(buf));
 	if (ml_sock_gets(sock, buf) < 0) {
 		*status = 4;
 		strcpy(dsn, "Connection broken during SMTP RCPT");
 		goto bail;
 	}
-	lprintf(9, "<%s\n", buf);
+	lprintf(CTDL_DEBUG, "<%s\n", buf);
 	if (buf[0] != '2') {
 		if (buf[0] == '4') {
 			*status = 4;
@@ -944,14 +1047,14 @@ void smtp_try(const char *key, const char *addr, int *status,
 
 
 	/* RCPT succeeded, now try the DATA command */
-	lprintf(9, ">DATA\n");
+	lprintf(CTDL_DEBUG, ">DATA\n");
 	sock_write(sock, "DATA\r\n", 6);
 	if (ml_sock_gets(sock, buf) < 0) {
 		*status = 4;
 		strcpy(dsn, "Connection broken during SMTP DATA");
 		goto bail;
 	}
-	lprintf(9, "<%s\n", buf);
+	lprintf(CTDL_DEBUG, "<%s\n", buf);
 	if (buf[0] != '3') {
 		if (buf[0] == '4') {
 			*status = 3;
@@ -975,8 +1078,8 @@ void smtp_try(const char *key, const char *addr, int *status,
 		msg_size -= blocksize;
 	}
 	if (buf[blocksize-1] != 10) {
-		lprintf(5, "Possible problem: message did not correctly "
-			"terminate. (expecting 0x10, got 0x%02x)\n",
+		lprintf(CTDL_WARNING, "Possible problem: message did not "
+			"correctly terminate. (expecting 0x10, got 0x%02x)\n",
 				buf[blocksize-1]);
 	}
 
@@ -986,7 +1089,7 @@ void smtp_try(const char *key, const char *addr, int *status,
 		strcpy(dsn, "Connection broken during SMTP message transmit");
 		goto bail;
 	}
-	lprintf(9, "%s\n", buf);
+	lprintf(CTDL_DEBUG, "%s\n", buf);
 	if (buf[0] != '2') {
 		if (buf[0] == '4') {
 			*status = 4;
@@ -1004,10 +1107,12 @@ void smtp_try(const char *key, const char *addr, int *status,
 	safestrncpy(dsn, &buf[4], 1023);
 	*status = 2;
 
-	lprintf(9, ">QUIT\n");
+	lprintf(CTDL_DEBUG, ">QUIT\n");
 	sock_write(sock, "QUIT\r\n", 6);
 	ml_sock_gets(sock, buf);
-	lprintf(9, "<%s\n", buf);
+	lprintf(CTDL_DEBUG, "<%s\n", buf);
+	lprintf(CTDL_INFO, "SMTP delivery to <%s> @ <%s> (%s) succeeded\n",
+		user, node, name);
 
 bail:	if (msg_fp != NULL) fclose(msg_fp);
 	sock_close(sock);
@@ -1039,7 +1144,7 @@ void smtp_do_bounce(char *instr) {
 	struct recptypes *valid;
 	int successful_bounce = 0;
 
-	lprintf(9, "smtp_do_bounce() called\n");
+	lprintf(CTDL_DEBUG, "smtp_do_bounce() called\n");
 	strcpy(bounceto, "");
 
 	lines = num_tokens(instr, '\n');
@@ -1061,24 +1166,24 @@ void smtp_do_bounce(char *instr) {
 
 
 
-	bmsg = (struct CtdlMessage *) mallok(sizeof(struct CtdlMessage));
+	bmsg = (struct CtdlMessage *) malloc(sizeof(struct CtdlMessage));
 	if (bmsg == NULL) return;
 	memset(bmsg, 0, sizeof(struct CtdlMessage));
 
         bmsg->cm_magic = CTDLMESSAGE_MAGIC;
         bmsg->cm_anon_type = MES_NORMAL;
         bmsg->cm_format_type = 1;
-        bmsg->cm_fields['A'] = strdoop("Citadel");
-        bmsg->cm_fields['O'] = strdoop(MAILROOM);
-        bmsg->cm_fields['N'] = strdoop(config.c_nodename);
+        bmsg->cm_fields['A'] = strdup("Citadel");
+        bmsg->cm_fields['O'] = strdup(MAILROOM);
+        bmsg->cm_fields['N'] = strdup(config.c_nodename);
 
-	if (give_up) bmsg->cm_fields['M'] = strdoop(
+	if (give_up) bmsg->cm_fields['M'] = strdup(
 "A message you sent could not be delivered to some or all of its recipients\n"
 "due to prolonged unavailability of its destination(s).\n"
 "Giving up on the following addresses:\n\n"
 );
 
-        else bmsg->cm_fields['M'] = strdoop(
+        else bmsg->cm_fields['M'] = strdup(
 "A message you sent could not be delivered to some or all of its recipients.\n"
 "The following addresses were undeliverable:\n\n"
 );
@@ -1094,7 +1199,7 @@ void smtp_do_bounce(char *instr) {
 		extract(dsn, buf, 3);
 		bounce_this = 0;
 
-		lprintf(9, "key=<%s> addr=<%s> status=%d dsn=<%s>\n",
+		lprintf(CTDL_DEBUG, "key=<%s> addr=<%s> status=%d dsn=<%s>\n",
 			key, addr, status, dsn);
 
 		if (!strcasecmp(key, "bounceto")) {
@@ -1115,11 +1220,11 @@ void smtp_do_bounce(char *instr) {
 			++num_bounces;
 
 			if (bmsg->cm_fields['M'] == NULL) {
-				lprintf(2, "ERROR ... M field is null "
+				lprintf(CTDL_ERR, "ERROR ... M field is null "
 					"(%s:%d)\n", __FILE__, __LINE__);
 			}
 
-			bmsg->cm_fields['M'] = reallok(bmsg->cm_fields['M'],
+			bmsg->cm_fields['M'] = realloc(bmsg->cm_fields['M'],
 				strlen(bmsg->cm_fields['M']) + 1024 );
 			strcat(bmsg->cm_fields['M'], addr);
 			strcat(bmsg->cm_fields['M'], ": ");
@@ -1133,13 +1238,13 @@ void smtp_do_bounce(char *instr) {
 	}
 
 	/* Deliver the bounce if there's anything worth mentioning */
-	lprintf(9, "num_bounces = %d\n", num_bounces);
+	lprintf(CTDL_DEBUG, "num_bounces = %d\n", num_bounces);
 	if (num_bounces > 0) {
 
 		/* First try the user who sent the message */
-		lprintf(9, "bounce to user? <%s>\n", bounceto);
+		lprintf(CTDL_DEBUG, "bounce to user? <%s>\n", bounceto);
 		if (strlen(bounceto) == 0) {
-			lprintf(7, "No bounce address specified\n");
+			lprintf(CTDL_ERR, "No bounce address specified\n");
 			bounce_msgid = (-1L);
 		}
 
@@ -1159,12 +1264,12 @@ void smtp_do_bounce(char *instr) {
 
 		/* Free up the memory we used */
 		if (valid != NULL) {
-			phree(valid);
+			free(valid);
 		}
 	}
 
 	CtdlFreeMessage(bmsg);
-	lprintf(9, "Done processing bounces\n");
+	lprintf(CTDL_DEBUG, "Done processing bounces\n");
 }
 
 
@@ -1238,15 +1343,15 @@ void smtp_do_procmsg(long msgnum, void *userdata) {
 	time_t last_attempted = 0L;
 	time_t retry = SMTP_RETRY_INTERVAL;
 
-	lprintf(9, "smtp_do_procmsg(%ld)\n", msgnum);
+	lprintf(CTDL_DEBUG, "smtp_do_procmsg(%ld)\n", msgnum);
 
 	msg = CtdlFetchMessage(msgnum);
 	if (msg == NULL) {
-		lprintf(3, "SMTP: tried %ld but no such message!\n", msgnum);
+		lprintf(CTDL_ERR, "SMTP: tried %ld but no such message!\n", msgnum);
 		return;
 	}
 
-	instr = strdoop(msg->cm_fields['M']);
+	instr = strdup(msg->cm_fields['M']);
 	CtdlFreeMessage(msg);
 
 	/* Strip out the headers amd any other non-instruction line */
@@ -1287,8 +1392,8 @@ void smtp_do_procmsg(long msgnum, void *userdata) {
 	 * Postpone delivery if we've already tried recently.
 	 */
 	if (((time(NULL) - last_attempted) < retry) && (run_queue_now == 0)) {
-		lprintf(7, "Retry time not yet reached.\n");
-		phree(instr);
+		lprintf(CTDL_DEBUG, "Retry time not yet reached.\n");
+		free(instr);
 		return;
 	}
 
@@ -1297,8 +1402,8 @@ void smtp_do_procmsg(long msgnum, void *userdata) {
 	 * Bail out if there's no actual message associated with this
 	 */
 	if (text_msgid < 0L) {
-		lprintf(3, "SMTP: no 'msgid' directive found!\n");
-		phree(instr);
+		lprintf(CTDL_ERR, "SMTP: no 'msgid' directive found!\n");
+		free(instr);
 		return;
 	}
 
@@ -1327,15 +1432,15 @@ void smtp_do_procmsg(long msgnum, void *userdata) {
 
 			--i;
 			--lines;
-			lprintf(9, "SMTP: Trying <%s>\n", addr);
+			lprintf(CTDL_DEBUG, "SMTP: Trying <%s>\n", addr);
 			smtp_try(key, addr, &status, dsn, sizeof dsn, text_msgid);
 			if (status != 2) {
 				if (results == NULL) {
-					results = mallok(1024);
+					results = malloc(1024);
 					memset(results, 0, 1024);
 				}
 				else {
-					results = reallok(results,
+					results = realloc(results,
 						strlen(results) + 1024);
 				}
 				snprintf(&results[strlen(results)], 1024,
@@ -1346,9 +1451,9 @@ void smtp_do_procmsg(long msgnum, void *userdata) {
 	}
 
 	if (results != NULL) {
-		instr = reallok(instr, strlen(instr) + strlen(results) + 2);
+		instr = realloc(instr, strlen(instr) + strlen(results) + 2);
 		strcat(instr, results);
-		phree(results);
+		free(results);
 	}
 
 
@@ -1376,7 +1481,7 @@ void smtp_do_procmsg(long msgnum, void *userdata) {
 	 */
 	if (incomplete_deliveries_remaining > 0) {
 		CtdlDeleteMessages(SMTP_SPOOLOUT_ROOM, msgnum, "");
-        	msg = mallok(sizeof(struct CtdlMessage));
+        	msg = malloc(sizeof(struct CtdlMessage));
 		memset(msg, 0, sizeof(struct CtdlMessage));
 		msg->cm_magic = CTDLMESSAGE_MAGIC;
 		msg->cm_anon_type = MES_NORMAL;
@@ -1388,7 +1493,7 @@ void smtp_do_procmsg(long msgnum, void *userdata) {
 			"attempted|%ld\n"
 			"retry|%ld\n",
 			SPOOLMIME, instr, (long)time(NULL), (long)retry );
-		phree(instr);
+		free(instr);
 		CtdlSubmitMsg(msg, NULL, SMTP_SPOOLOUT_ROOM);
 		CtdlFreeMessage(msg);
 	}
@@ -1417,16 +1522,16 @@ void smtp_do_queue(void) {
 	/* 
 	 * Go ahead and run the queue
 	 */
-	lprintf(7, "SMTP: processing outbound queue\n");
+	lprintf(CTDL_INFO, "SMTP: processing outbound queue\n");
 
 	if (getroom(&CC->room, SMTP_SPOOLOUT_ROOM) != 0) {
-		lprintf(3, "Cannot find room <%s>\n", SMTP_SPOOLOUT_ROOM);
+		lprintf(CTDL_ERR, "Cannot find room <%s>\n", SMTP_SPOOLOUT_ROOM);
 		return;
 	}
 	CtdlForEachMessage(MSGS_ALL, 0L,
 		SPOOLMIME, NULL, smtp_do_procmsg, NULL);
 
-	lprintf(7, "SMTP: queue run completed\n");
+	lprintf(CTDL_INFO, "SMTP: queue run completed\n");
 	run_queue_now = 0;
 	doing_queue = 0;
 }
@@ -1468,7 +1573,7 @@ void cmd_smtp(char *argbuf) {
 	}
 
 	else {
-		cprintf("%d Invalid command.\n", ERROR+ILLEGAL_VALUE);
+		cprintf("%d Invalid command.\n", ERROR + ILLEGAL_VALUE);
 	}
 
 }
@@ -1519,5 +1624,5 @@ char *serv_smtp_init(void)
 	smtp_init_spoolout();
 	CtdlRegisterSessionHook(smtp_do_queue, EVT_TIMER);
 	CtdlRegisterProtoHook(cmd_smtp, "SMTP", "SMTP utility commands");
-	return "$Id: serv_smtp.c,v 613.0 2003/12/15 16:37:03 ajc Exp $";
+	return "$Id: serv_smtp.c,v 620.0 2004/03/27 02:33:34 ajc Exp $";
 }
