@@ -1,5 +1,5 @@
 /*
- * $Id: serv_vcard.c,v 613.0 2003/12/15 16:37:03 ajc Exp $
+ * $Id: serv_vcard.c,v 620.0 2004/03/27 02:33:34 ajc Exp $
  * 
  * A server-side module for Citadel which supports address book information
  * using the standard vCard format.
@@ -63,6 +63,7 @@
 #include "internet_addressing.h"
 #include "tools.h"
 #include "vcard.h"
+#include "serv_ldap.h"
 
 struct vcard_internal_info {
 	long msgnum;
@@ -112,14 +113,14 @@ void vcard_extract_internet_addresses(struct CtdlMessage *msg,
 	do {
 		s = vcard_get_prop(v, "email;internet", 0, instance++, 0);
 		if (s != NULL) {
-			addr = strdoop(s);
+			addr = strdup(s);
 			striplt(addr);
 			if (strlen(addr) > 0) {
 				if (callback != NULL) {
 					callback(addr, citadel_address);
 				}
 			}
-			phree(addr);
+			free(addr);
 			found_something = 1;
 		}
 		else {
@@ -145,18 +146,19 @@ void vcard_directory_add_user(char *internet_addr, char *citadel_addr) {
 	 * probably just the networker or something.
 	 */
 	if (CC->logged_in) {
-		lprintf(9, "Checking for <%s>...\n", internet_addr);
+		lprintf(CTDL_DEBUG, "Checking for <%s>...\n", internet_addr);
 		if (CtdlDirectoryLookup(buf, internet_addr) == 0) {
 			if (strcasecmp(buf, citadel_addr)) {
 				/* This address belongs to someone else.
 				 * Bail out silently without saving.
 				 */
-				lprintf(9, "DOOP!\n");
+				lprintf(CTDL_DEBUG, "DOOP!\n");
 				return;
 			}
 		}
 	}
-	lprintf(9, "ADDING!\n");
+	lprintf(CTDL_INFO, "Adding %s (%s) to directory\n",
+			citadel_addr, internet_addr);
 	CtdlDirectoryAddUser(internet_addr, citadel_addr);
 }
 
@@ -172,6 +174,10 @@ void vcard_add_to_directory(long msgnum, void *data) {
 		vcard_extract_internet_addresses(msg, vcard_directory_add_user);
 	}
 
+#ifdef HAVE_LDAP
+	ctdl_vcard_to_ldap(msg, V2L_WRITE);
+#endif
+
 	CtdlFreeMessage(msg);
 }
 
@@ -188,7 +194,7 @@ void cmd_igab(char *argbuf) {
 
         if (getroom(&CC->room, ADDRESS_BOOK_ROOM) != 0) {
                 getroom(&CC->room, hold_rm);
-		cprintf("%d cannot get address book room\n", ERROR);
+		cprintf("%d cannot get address book room\n", ERROR + ROOM_NOT_FOUND);
 		return;
         }
 
@@ -196,7 +202,7 @@ void cmd_igab(char *argbuf) {
 	 */
 	CtdlDirectoryInit();
 
-        /* We want the last (and probably only) vcard in this room */
+        /* We want *all* vCards in this room */
         CtdlForEachMessage(MSGS_ALL, 0, "text/x-vcard",
 		NULL, vcard_add_to_directory, NULL);
 
@@ -223,7 +229,7 @@ void vcard_populate_cs_inet_email(struct vCard *v) {
 		s = vcard_get_prop(v, "email;internet", 0, instance++, 0);
 		if (s != NULL) {
 			continue_searching = 1;
-			addr = strdoop(s);
+			addr = strdup(s);
 			striplt(addr);
 			if (strlen(addr) > 0) {
 				if (IsDirectory(addr)) {
@@ -234,7 +240,7 @@ void vcard_populate_cs_inet_email(struct vCard *v) {
 					);
 				}
 			}
-			phree(addr);
+			free(addr);
 		}
 		else {
 			continue_searching = 0;
@@ -314,16 +320,16 @@ int vcard_upload_beforesave(struct CtdlMessage *msg) {
 			 * replication always works correctly
 			 */
                         if (msg->cm_fields['E'] != NULL)
-                                phree(msg->cm_fields['E']);
+                                free(msg->cm_fields['E']);
 
                         if (msg->cm_fields['A'] != NULL)
-                                phree(msg->cm_fields['A']);
+                                free(msg->cm_fields['A']);
 
-			msg->cm_fields['A'] = strdoop(usbuf.fullname);
+			msg->cm_fields['A'] = strdup(usbuf.fullname);
 
                         snprintf(buf, sizeof buf, VCARD_EXT_FORMAT,
                                 msg->cm_fields['A'], NODENAME);
-                        msg->cm_fields['E'] = strdoop(buf);
+                        msg->cm_fields['E'] = strdup(buf);
 
 			/* Now allow the save to complete. */
 			return(0);
@@ -471,7 +477,7 @@ void vcard_write_user(struct ctdluser *u, struct vCard *v) {
 		fprintf(fp, "begin:vcard\r\nend:vcard\r\n");
 	} else {
 		fwrite(ser, strlen(ser), 1, fp);
-		phree(ser);
+		free(ser);
 	}
         fclose(fp);
 
@@ -512,7 +518,7 @@ void cmd_regi(char *argbuf) {
 	char tmpcountry[SIZ];
 
 	if (!(CC->logged_in)) {
-		cprintf("%d Not logged in.\n",ERROR+NOT_LOGGED_IN);
+		cprintf("%d Not logged in.\n",ERROR + NOT_LOGGED_IN);
 		return;
 	}
 
@@ -568,7 +574,7 @@ void cmd_greg(char *argbuf)
 	extract(who, argbuf, 0);
 
 	if (!(CC->logged_in)) {
-		cprintf("%d Not logged in.\n", ERROR+NOT_LOGGED_IN);
+		cprintf("%d Not logged in.\n", ERROR + NOT_LOGGED_IN);
 		return;
 	}
 
@@ -576,12 +582,12 @@ void cmd_greg(char *argbuf)
 
 	if ((CC->user.axlevel < 6) && (strcasecmp(who,CC->curr_user))) {
 		cprintf("%d Higher access required.\n",
-			ERROR+HIGHER_ACCESS_REQUIRED);
+			ERROR + HIGHER_ACCESS_REQUIRED);
 		return;
 	}
 
 	if (getuser(&usbuf, who) != 0) {
-		cprintf("%d '%s' not found.\n", ERROR+NO_SUCH_USER, who);
+		cprintf("%d '%s' not found.\n", ERROR + NO_SUCH_USER, who);
 		return;
 	}
 
@@ -627,32 +633,107 @@ void cmd_greg(char *argbuf)
 }
 
 
+/*
+ * When a user is being created, create his/her vCard.
+ */
+void vcard_newuser(struct ctdluser *usbuf) {
+	char buf[SIZ];
+	char vname[SIZ];
+
+	char lastname[SIZ];
+	char firstname[SIZ];
+	char middlename[SIZ];
+	char honorific_prefixes[SIZ];
+	char honorific_suffixes[SIZ];
+
+	struct vCard *v;
+	int i;
+
+	/* Try to intelligently convert the screen name to a
+	 * fully expanded vCard name based on the number of
+	 * words in the name
+	 */
+	strcpy(lastname, "");
+	strcpy(firstname, "");
+	strcpy(middlename, "");
+	strcpy(honorific_prefixes, "");
+	strcpy(honorific_suffixes, "");
+
+	strcpy(buf, usbuf->fullname);
+
+	/* Honorific suffixes */
+	if (num_tokens(buf, ',') > 1) {
+		extract_token(honorific_suffixes, buf, (num_tokens(buf, ' ') - 1), ',');
+		remove_token(buf, (num_tokens(buf, ',') - 1), ',');
+	}
+
+	/* Find a last name */
+	extract_token(lastname, buf, (num_tokens(buf, ' ') - 1), ' ');
+	remove_token(buf, (num_tokens(buf, ' ') - 1), ' ');
+
+	/* Find honorific prefixes */
+	if (num_tokens(buf, ' ') > 2) {
+		extract_token(honorific_prefixes, buf, 0, ' ');
+		remove_token(buf, 0, ' ');
+	}
+
+	/* Find a middle name */
+	if (num_tokens(buf, ' ') > 1) {
+		extract_token(middlename, buf, (num_tokens(buf, ' ') - 1), ' ');
+		remove_token(buf, (num_tokens(buf, ' ') - 1), ' ');
+	}
+
+	/* Anything left is probably the first name */
+	strcpy(firstname, buf);
+	striplt(firstname);
+
+	/* Compose the structured name */
+	sprintf(vname, "%s;%s;%s;%s;%s", lastname, firstname, middlename,
+		honorific_prefixes, honorific_suffixes);
+
+	lprintf(CTDL_DEBUG, "Converted <%s> to <%s>\n", usbuf->fullname, vname);
+
+	/* Create and save the vCard */
+        v = vcard_new();
+	if (v == NULL) return;
+	sprintf(buf, "%s@%s", usbuf->fullname, config.c_fqdn);
+	for (i=0; i<strlen(buf); ++i) {
+		if (buf[i] == ' ') buf[i] = '_';
+	}
+	vcard_add_prop(v, "fn", usbuf->fullname);
+	vcard_add_prop(v, "n", vname);
+	vcard_add_prop(v, "email;internet", buf);
+	vcard_write_user(usbuf, v);
+	vcard_free(v);
+}
+
+
 /*
  * When a user is being deleted, we have to remove his/her vCard.
  * This is accomplished by issuing a message with 'CANCEL' in the S (special)
  * field, and the same Extended ID as the existing card.
  */
-void vcard_purge(char *username, long usernum) {
+void vcard_purge(struct ctdluser *usbuf) {
 	struct CtdlMessage *msg;
 	char buf[SIZ];
 
-	msg = (struct CtdlMessage *) mallok(sizeof(struct CtdlMessage));
+	msg = (struct CtdlMessage *) malloc(sizeof(struct CtdlMessage));
 	if (msg == NULL) return;
 	memset(msg, 0, sizeof(struct CtdlMessage));
 
         msg->cm_magic = CTDLMESSAGE_MAGIC;
         msg->cm_anon_type = MES_NORMAL;
         msg->cm_format_type = 0;
-        msg->cm_fields['A'] = strdoop(username);
-        msg->cm_fields['O'] = strdoop(ADDRESS_BOOK_ROOM);
-        msg->cm_fields['N'] = strdoop(NODENAME);
-        msg->cm_fields['M'] = strdoop("Purge this vCard\n");
+        msg->cm_fields['A'] = strdup(usbuf->fullname);
+        msg->cm_fields['O'] = strdup(ADDRESS_BOOK_ROOM);
+        msg->cm_fields['N'] = strdup(NODENAME);
+        msg->cm_fields['M'] = strdup("Purge this vCard\n");
 
         snprintf(buf, sizeof buf, VCARD_EXT_FORMAT,
 			msg->cm_fields['A'], NODENAME);
-        msg->cm_fields['E'] = strdoop(buf);
+        msg->cm_fields['E'] = strdup(buf);
 
-	msg->cm_fields['S'] = strdoop("CANCEL");
+	msg->cm_fields['S'] = strdup("CANCEL");
 
         CtdlSubmitMsg(msg, NULL, ADDRESS_BOOK_ROOM);
         CtdlFreeMessage(msg);
@@ -726,6 +807,9 @@ void vcard_delete_remove(char *room, long msgnum) {
 		 	*/
 			vcard_extract_internet_addresses(msg,
 							CtdlDirectoryDelUser);
+#ifdef HAVE_LDAP
+			ctdl_vcard_to_ldap(msg, V2L_DELETE);
+#endif
 		}
 		ptr = strchr((char *)ptr, '\n');
 		if (ptr != NULL) ++ptr;
@@ -749,7 +833,7 @@ void cmd_qdir(char *argbuf) {
 
 	if (CtdlDirectoryLookup(citadel_addr, internet_addr) != 0) {
 		cprintf("%d %s was not found.\n",
-			ERROR+NO_SUCH_USER, internet_addr);
+			ERROR + NO_SUCH_USER, internet_addr);
 		return;
 	}
 
@@ -770,7 +854,7 @@ void vcard_create_room(void)
 
 	/* Set expiration policy to manual; otherwise objects will be lost! */
 	if (lgetroom(&qr, USERCONTACTSROOM)) {
-		lprintf(3, "Couldn't get the user CONTACTS room!\n");
+		lprintf(CTDL_ERR, "Couldn't get the user CONTACTS room!\n");
 		return;
 	}
 	qr.QRep.expire_mode = EXPIRE_MANUAL;
@@ -814,6 +898,8 @@ void vcard_session_login_hook(void) {
 
 char *serv_vcard_init(void)
 {
+	struct ctdlroom qr;
+
 	CtdlRegisterSessionHook(vcard_session_startup_hook, EVT_START);
 	CtdlRegisterSessionHook(vcard_session_login_hook, EVT_LOGIN);
 	CtdlRegisterMessageHook(vcard_upload_beforesave, EVT_BEFORESAVE);
@@ -824,8 +910,19 @@ char *serv_vcard_init(void)
 	CtdlRegisterProtoHook(cmd_igab, "IGAB",
 					"Initialize Global Address Book");
 	CtdlRegisterProtoHook(cmd_qdir, "QDIR", "Query Directory");
+	CtdlRegisterUserHook(vcard_newuser, EVT_NEWUSER);
 	CtdlRegisterUserHook(vcard_purge, EVT_PURGEUSER);
 	CtdlRegisterNetprocHook(vcard_extract_from_network);
+
+	/* Create the Global ADdress Book room if necessary */
 	create_room(ADDRESS_BOOK_ROOM, 3, "", 0, 1, 0);
-	return "$Id: serv_vcard.c,v 613.0 2003/12/15 16:37:03 ajc Exp $";
+
+	/* Set expiration policy to manual; otherwise objects will be lost! */
+	if (!lgetroom(&qr, ADDRESS_BOOK_ROOM)) {
+		qr.QRep.expire_mode = EXPIRE_MANUAL;
+		qr.QRdefaultview = 2;	/* 2 = address book view */
+		lputroom(&qr);
+	}
+
+	return "$Id: serv_vcard.c,v 620.0 2004/03/27 02:33:34 ajc Exp $";
 }
