@@ -1,5 +1,5 @@
 /*
- * $Id: setup.c,v 613.0 2003/12/15 16:37:03 ajc Exp $
+ * $Id: setup.c,v 620.0 2004/03/27 02:33:34 ajc Exp $
  *
  * Citadel/UX setup utility
  *
@@ -461,7 +461,7 @@ void check_inittab_entry(void)
 		display_error(strerror(errno));
 	} else {
 		fprintf(infp, "# Start the Citadel/UX server...\n");
-		fprintf(infp, "%s:2345:respawn:%s -h%s\n",
+		fprintf(infp, "%s:2345:respawn:%s -h%s -x3 -llocal4\n",
 			entryname, looking_for, setup_directory);
 		fclose(infp);
 		strcpy(init_entry, entryname);
@@ -610,10 +610,8 @@ void set_str_val(int msgpos, char str[])
 	int i;
 #endif
 	char buf[SIZ];
-	char tempfile[PATH_MAX];
 	char setupmsg[SIZ];
 
-	strcpy(tempfile, tmpnam(NULL));
 	strcpy(setupmsg, "");
 
 	switch (setup_type) {
@@ -994,12 +992,19 @@ NEW_INST:
 	write_config_to_disk();
 
 	mkdir("info", 0700);
+	chmod("info", 0700);
 	mkdir("bio", 0700);
+	chmod("bio", 0700);
 	mkdir("userpics", 0700);
+	chmod("userpics", 0700);
 	mkdir("messages", 0700);
+	chmod("messages", 0700);
 	mkdir("help", 0700);
+	chmod("help", 0700);
 	mkdir("images", 0700);
+	chmod("images", 0700);
 	mkdir("netconfigs", 0700);
+	chmod("netconfigs", 0700);
 
 	/* Delete files and directories used by older Citadel versions */
 	system("exec /bin/rm -fr ./rooms ./chatpipes ./expressmsgs ./sessions 2>/dev/null");
