@@ -1,5 +1,5 @@
 /*
- * $Id: sysdep.c,v 613.0 2003/12/15 16:37:03 ajc Exp $
+ * $Id: sysdep.c,v 620.0 2004/03/27 02:33:34 ajc Exp $
  *
  * Citadel/UX "system dependent" stuff.
  * See copyright.txt for copyright information.
@@ -27,6 +27,7 @@
 #include <sys/stat.h>
 #include <sys/wait.h>
 #include <sys/socket.h>
+#include <sys/syslog.h>
 
 #if TIME_WITH_SYS_TIME
 # include <sys/time.h>
@@ -47,7 +48,6 @@
 #include <pwd.h>
 #include <errno.h>
 #include <stdarg.h>
-#include <syslog.h>
 #include <grp.h>
 #ifdef HAVE_PTHREAD_H
 #include <pthread.h>
@@ -72,10 +72,19 @@
 #include "snprintf.h"
 #endif
 
+
 #ifdef DEBUG_MEMORY_LEAKS
-struct TheHeap *heap = NULL;
+struct igheap {
+	struct igheap *next;
+	char file[32];
+	int line;
+	void *block;
+};
+
+struct igheap *igheap = NULL;
 #endif
 
+
 pthread_mutex_t Critters[MAX_SEMAPHORES];	/* Things needing locking */
 pthread_key_t MyConKey;				/* TSD key for MyContext() */
 
@@ -92,6 +101,8 @@ int masterhighest;
 
 pthread_t initial_thread;		/* tid for main() thread */
 
+int syslog_facility = (-1);
+
 
 /*
  * lprintf()  ...   Write logging information
@@ -99,15 +110,26 @@ pthread_t initial_thread;		/* tid for main() thread */
  * Note: the variable "buf" below needs to be large enough to handle any
  * log data sent through this function.  BE CAREFUL!
  */
-void lprintf(int loglevel, const char *format, ...) {   
+void lprintf(enum LogLevel loglevel, const char *format, ...) {   
         va_list arg_ptr;
 	char buf[SIZ];
-  
+ 
         va_start(arg_ptr, format);   
         vsnprintf(buf, sizeof(buf), format, arg_ptr);   
         va_end(arg_ptr);   
 
-	if (loglevel <= verbosity) { 
+	if (syslog_facility >= 0) {
+		if (loglevel <= verbosity) {
+			/* Hackery -IO */
+			if (CC && CC->cs_pid) {
+				memmove(buf + 6, buf, sizeof(buf) - 6);
+				snprintf(buf, 6, "[%3d]", CC->cs_pid);
+				buf[5] = ' ';
+			}
+			syslog(loglevel, buf);
+		}
+	}
+	else if (loglevel <= verbosity) { 
 		struct timeval tv;
 		struct tm *tim;
 		time_t unixtime;
@@ -122,18 +144,37 @@ void lprintf(int loglevel, const char *format, ...) {
 		 * %03ld to %06ld and remove " / 1000" after tv.tv_usec.
 		 */
 		if (CC && CC->cs_pid) {
+#if 0
+			/* Millisecond display */
 			fprintf(stderr,
 				"%04d/%02d/%02d %2d:%02d:%02d.%03ld [%3d] %s",
 				tim->tm_year + 1900, tim->tm_mon + 1,
 				tim->tm_mday, tim->tm_hour, tim->tm_min,
 				tim->tm_sec, (long)tv.tv_usec / 1000,
 				CC->cs_pid, buf);
+#endif
+			/* Microsecond display */
+			fprintf(stderr,
+				"%04d/%02d/%02d %2d:%02d:%02d.%06ld [%3d] %s",
+				tim->tm_year + 1900, tim->tm_mon + 1,
+				tim->tm_mday, tim->tm_hour, tim->tm_min,
+				tim->tm_sec, (long)tv.tv_usec,
+				CC->cs_pid, buf);
 		} else {
+#if 0
+			/* Millisecond display */
 			fprintf(stderr,
 				"%04d/%02d/%02d %2d:%02d:%02d.%03ld %s",
 				tim->tm_year + 1900, tim->tm_mon + 1,
 				tim->tm_mday, tim->tm_hour, tim->tm_min,
 				tim->tm_sec, (long)tv.tv_usec / 1000, buf);
+#endif
+			/* Microsecond display */
+			fprintf(stderr,
+				"%04d/%02d/%02d %2d:%02d:%02d.%06ld %s",
+				tim->tm_year + 1900, tim->tm_mon + 1,
+				tim->tm_mday, tim->tm_hour, tim->tm_min,
+				tim->tm_sec, (long)tv.tv_usec, buf);
 		}
 		fflush(stderr);
 	}
@@ -143,89 +184,6 @@ void lprintf(int loglevel, const char *format, ...) {
 
 
 
-#ifdef DEBUG_MEMORY_LEAKS
-void *tracked_malloc(size_t tsize, char *tfile, int tline) {
-	void *ptr;
-	struct TheHeap *hptr;
-
-	ptr = malloc(tsize);
-	if (ptr == NULL) {
-		lprintf(3, "DANGER!  mallok(%d) at %s:%d failed!\n",
-			tsize, tfile, tline);
-		return(NULL);
-	}
-
-	hptr = (struct TheHeap *) malloc(sizeof(struct TheHeap));
-	strcpy(hptr->h_file, tfile);
-	hptr->h_line = tline;
-	hptr->next = heap;
-	hptr->h_ptr = ptr;
-	heap = hptr;
-	return ptr;
-}
-
-char *tracked_strdup(const char *orig, char *tfile, int tline) {
-	char *s;
-
-	s = tracked_malloc( (strlen(orig)+1), tfile, tline);
-	if (s == NULL) return NULL;
-
-	strcpy(s, orig);
-	return s;
-}
-
-void tracked_free(void *ptr) {
-	struct TheHeap *hptr, *freeme;
-
-	if (heap->h_ptr == ptr) {
-		hptr = heap->next;
-		free(heap);
-		heap = hptr;
-	}
-	else {
-		for (hptr=heap; hptr->next!=NULL; hptr=hptr->next) {
-			if (hptr->next->h_ptr == ptr) {
-				freeme = hptr->next;
-				hptr->next = hptr->next->next;
-				free(freeme);
-			}
-		}
-	}
-
-	free(ptr);
-}
-
-void *tracked_realloc(void *ptr, size_t size) {
-	void *newptr;
-	struct TheHeap *hptr;
-	
-	newptr = realloc(ptr, size);
-
-	for (hptr=heap; hptr!=NULL; hptr=hptr->next) {
-		if (hptr->h_ptr == ptr) hptr->h_ptr = newptr;
-	}
-
-	return newptr;
-}
-
-
-void dump_tracked() {
-	struct TheHeap *hptr;
-
-	cprintf("%d Here's what's allocated...\n", LISTING_FOLLOWS);	
-	for (hptr=heap; hptr!=NULL; hptr=hptr->next) {
-		cprintf("%20s %5d\n",
-			hptr->h_file, hptr->h_line);
-	}
-#ifdef __GNUC__
-        malloc_stats();
-#endif
-
-	cprintf("000\n");
-}
-#endif
-
-
 /*
  * We used to use master_cleanup() as a signal handler to shut down the server.
  * however, master_cleanup() and the functions it calls do some things that
@@ -264,7 +222,7 @@ void init_sysdep(void) {
 	 * session to which the calling thread is currently bound.
 	 */
 	if (pthread_key_create(&MyConKey, NULL) != 0) {
-		lprintf(1, "Can't create TSD key!!  %s\n", strerror(errno));
+		lprintf(CTDL_CRIT, "Can't create TSD key!!  %s\n", strerror(errno));
 	}
 
 	/*
@@ -290,10 +248,19 @@ void init_sysdep(void) {
  */
 void begin_critical_section(int which_one)
 {
-	/* lprintf(9, "begin_critical_section(%d)\n", which_one); */
-	/* ensure nobody ever tries to do a critical section within a
-	   transaction; this could lead to deadlock. */
-	cdb_check_handles();
+	/* lprintf(CTDL_DEBUG, "begin_critical_section(%d)\n", which_one); */
+
+	/* For all types of critical sections except those listed here,
+	 * ensure nobody ever tries to do a critical section within a
+	 * transaction; this could lead to deadlock.
+	 */
+	if (	(which_one != S_FLOORCACHE)
+#ifdef DEBUG_MEMORY_LEAKS
+		&& (which_one != S_DEBUGMEMLEAKS)
+#endif
+	) {
+		cdb_check_handles();
+	}
 	pthread_mutex_lock(&Critters[which_one]);
 }
 
@@ -302,7 +269,6 @@ void begin_critical_section(int which_one)
  */
 void end_critical_section(int which_one)
 {
-	/* lprintf(9, "end_critical_section(%d)\n", which_one); */
 	pthread_mutex_unlock(&Critters[which_one]);
 }
 
@@ -330,7 +296,7 @@ int ig_tcp_server(int port_number, int queue_len)
 	s = socket(PF_INET, SOCK_STREAM, IPPROTO_TCP);
 
 	if (s < 0) {
-		lprintf(1, "citserver: Can't create a socket: %s\n",
+		lprintf(CTDL_EMERG, "citserver: Can't create a socket: %s\n",
 			strerror(errno));
 		return(-1);
 	}
@@ -339,14 +305,14 @@ int ig_tcp_server(int port_number, int queue_len)
 	setsockopt(s, SOL_SOCKET, SO_REUSEADDR, &i, sizeof(i));
 
 	if (bind(s, (struct sockaddr *)&sin, sizeof(sin)) < 0) {
-		lprintf(1, "citserver: Can't bind: %s\n",
+		lprintf(CTDL_EMERG, "citserver: Can't bind: %s\n",
 			strerror(errno));
 		close(s);
 		return(-1);
 	}
 
 	if (listen(s, actual_queue_len) < 0) {
-		lprintf(1, "citserver: Can't listen: %s\n", strerror(errno));
+		lprintf(CTDL_EMERG, "citserver: Can't listen: %s\n", strerror(errno));
 		close(s);
 		return(-1);
 	}
@@ -371,7 +337,7 @@ int ig_uds_server(char *sockpath, int queue_len)
 
 	i = unlink(sockpath);
 	if (i != 0) if (errno != ENOENT) {
-		lprintf(1, "citserver: can't unlink %s: %s\n",
+		lprintf(CTDL_EMERG, "citserver: can't unlink %s: %s\n",
 			sockpath, strerror(errno));
 		return(-1);
 	}
@@ -382,19 +348,19 @@ int ig_uds_server(char *sockpath, int queue_len)
 
 	s = socket(AF_UNIX, SOCK_STREAM, 0);
 	if (s < 0) {
-		lprintf(1, "citserver: Can't create a socket: %s\n",
+		lprintf(CTDL_EMERG, "citserver: Can't create a socket: %s\n",
 			strerror(errno));
 		return(-1);
 	}
 
 	if (bind(s, (struct sockaddr *)&addr, sizeof(addr)) < 0) {
-		lprintf(1, "citserver: Can't bind: %s\n",
+		lprintf(CTDL_EMERG, "citserver: Can't bind: %s\n",
 			strerror(errno));
 		return(-1);
 	}
 
 	if (listen(s, actual_queue_len) < 0) {
-		lprintf(1, "citserver: Can't listen: %s\n", strerror(errno));
+		lprintf(CTDL_EMERG, "citserver: Can't listen: %s\n", strerror(errno));
 		return(-1);
 	}
 
@@ -429,9 +395,9 @@ INLINE struct CitContext *MyContext(void) {
 struct CitContext *CreateNewContext(void) {
 	struct CitContext *me, *ptr;
 
-	me = (struct CitContext *) mallok(sizeof(struct CitContext));
+	me = (struct CitContext *) malloc(sizeof(struct CitContext));
 	if (me == NULL) {
-		lprintf(1, "citserver: can't allocate memory!!\n");
+		lprintf(CTDL_ALERT, "citserver: can't allocate memory!!\n");
 		return NULL;
 	}
 	memset(me, 0, sizeof(struct CitContext));
@@ -492,7 +458,7 @@ void buffer_output(void) {
 	if (CC->buffering == 0) {
 		CC->buffering = 1;
 		CC->buffer_len = 0;
-		CC->output_buffer = mallok(SIZ);
+		CC->output_buffer = malloc(SIZ);
 	}
 }
 
@@ -503,7 +469,7 @@ void unbuffer_output(void) {
 	if (CC->buffering == 1) {
 		CC->buffering = 0;
 		client_write(CC->output_buffer, CC->buffer_len);
-		phree(CC->output_buffer);
+		free(CC->output_buffer);
 		CC->output_buffer = NULL;
 		CC->buffer_len = 0;
 	}
@@ -537,7 +503,7 @@ void client_write(char *buf, int nbytes)
 	if (CC->buffering) {
 		old_buffer_len = CC->buffer_len;
 		CC->buffer_len += nbytes;
-		CC->output_buffer = reallok(CC->output_buffer, CC->buffer_len);
+		CC->output_buffer = realloc(CC->output_buffer, CC->buffer_len);
 		memcpy(&CC->output_buffer[old_buffer_len], buf, nbytes);
 		return;
 	}
@@ -555,7 +521,7 @@ void client_write(char *buf, int nbytes)
 		retval = write(sock, &buf[bytes_written],
 			nbytes - bytes_written);
 		if (retval < 1) {
-			lprintf(2, "client_write() failed: %s\n",
+			lprintf(CTDL_ERR, "client_write() failed: %s\n",
 				strerror(errno));
 			if (sock == CC->client_socket) CC->kill_me = 1;
 			return;
@@ -618,7 +584,7 @@ int client_read_to(char *buf, int bytes, int timeout)
 
 		rlen = read(CC->client_socket, &buf[len], bytes-len);
 		if (rlen<1) {
-			lprintf(2, "client_read() failed: %s\n",
+			lprintf(CTDL_ERR, "client_read() failed: %s\n",
 				strerror(errno));
 			CC->kill_me = 1;
 			return(-1);
@@ -686,11 +652,11 @@ void sysdep_master_cleanup(void) {
 	    serviceptr = serviceptr->next ) {
 
 		if (serviceptr->tcp_port > 0)
-			lprintf(3, "Closing listener on port %d\n",
+			lprintf(CTDL_INFO, "Closing listener on port %d\n",
 				serviceptr->tcp_port);
 
 		if (serviceptr->sockpath != NULL)
-			lprintf(3, "Closing listener on '%s'\n",
+			lprintf(CTDL_INFO, "Closing listener on '%s'\n",
 				serviceptr->sockpath);
 
 		close(serviceptr->msock);
@@ -773,15 +739,15 @@ void create_worker(void) {
 	struct worker_node *n;
 	pthread_attr_t attr;
 
-	n = mallok(sizeof(struct worker_node));
+	n = malloc(sizeof(struct worker_node));
 	if (n == NULL) {
-		lprintf(1, "can't allocate worker_node, exiting\n");
+		lprintf(CTDL_EMERG, "can't allocate worker_node, exiting\n");
 		time_to_die = -1;
 		return;
 	}
 
 	if ((ret = pthread_attr_init(&attr))) {
-		lprintf(1, "pthread_attr_init: %s\n", strerror(ret));
+		lprintf(CTDL_EMERG, "pthread_attr_init: %s\n", strerror(ret));
 		time_to_die = -1;
 		return;
 	}
@@ -789,7 +755,7 @@ void create_worker(void) {
 	/* we seem to need something bigger than FreeBSD's default 64k stack */
 
 	if ((ret = pthread_attr_setstacksize(&attr, 128 * 1024))) {
-		lprintf(1, "pthread_attr_setstacksize: %s\n", strerror(ret));
+		lprintf(CTDL_EMERG, "pthread_attr_setstacksize: %s\n", strerror(ret));
 		time_to_die = -1;
 		return;
 	}
@@ -797,7 +763,7 @@ void create_worker(void) {
 	if ((ret = pthread_create(&n->tid, &attr, worker_thread, NULL) != 0))
 	{
 
-		lprintf(1, "Can't create worker thread: %s\n",
+		lprintf(CTDL_ALERT, "Can't create worker thread: %s\n",
 			strerror(ret));
 	}
 
@@ -838,7 +804,7 @@ void dead_session_purge(void) {
 		 * section, so we have to do it like this.
 		 */	
 		if (rem != NULL) {
-			lprintf(9, "Purging session %d\n", rem->cs_pid);
+			lprintf(CTDL_DEBUG, "Purging session %d\n", rem->cs_pid);
 			RemoveContext(rem);
 		}
 
@@ -875,7 +841,7 @@ void dead_session_purge(void) {
 			if ((*node)->tid == self) {
 				tmp = *node;
 				*node = (*node)->next;
-				phree(tmp);
+				free(tmp);
 				break;
 			}
 
@@ -929,25 +895,25 @@ void init_master_fdset(void) {
 	struct ServiceFunctionHook *serviceptr;
 	int m;
 
-	lprintf(9, "Initializing master fdset\n");
+	lprintf(CTDL_DEBUG, "Initializing master fdset\n");
 
 	FD_ZERO(&masterfds);
 	masterhighest = 0;
 
-	lprintf(9, "Will listen on rescan pipe %d\n", rescan[0]);
+	lprintf(CTDL_DEBUG, "Will listen on rescan pipe %d\n", rescan[0]);
 	FD_SET(rescan[0], &masterfds);
 	if (rescan[0] > masterhighest) masterhighest = rescan[0];
 
 	for (serviceptr = ServiceHookTable; serviceptr != NULL;
 	    serviceptr = serviceptr->next ) {
 		m = serviceptr->msock;
-		lprintf(9, "Will listen on master socket %d\n", m);
+		lprintf(CTDL_DEBUG, "Will listen on master socket %d\n", m);
 		FD_SET(m, &masterfds);
 		if (m > masterhighest) {
 			masterhighest = m;
 		}
 	}
-	lprintf(9, "masterhighest = %d\n", masterhighest);
+	lprintf(CTDL_DEBUG, "masterhighest = %d\n", masterhighest);
 }
 
 
@@ -967,6 +933,8 @@ void *worker_thread(void *arg) {
 	int i;
 	char junk;
 	int highest;
+	/* This is synchronized below; it helps implement round robin mode */
+	static struct CitContext* next_session = NULL;
 	struct CitContext *ptr;
 	struct CitContext *bind_me = NULL;
 	fd_set readfds;
@@ -1028,7 +996,7 @@ SETUP_FD:	memcpy(&readfds, &masterfds, sizeof masterfds);
 		 */
 		if (retval < 0) {
 			if (errno != EINTR) {
-				lprintf(9, "Exiting (%s)\n", strerror(errno));
+				lprintf(CTDL_EMERG, "Exiting (%s)\n", strerror(errno));
 				time_to_die = 1;
 			} else if (!time_to_die)
 				goto do_select;
@@ -1043,11 +1011,12 @@ SETUP_FD:	memcpy(&readfds, &masterfds, sizeof masterfds);
 			if (FD_ISSET(serviceptr->msock, &readfds)) {
 				ssock = accept(serviceptr->msock, NULL, 0);
 				if (ssock < 0) {
-					lprintf(2, "citserver: accept(): %s\n",
+					lprintf(CTDL_CRIT,
+						"citserver: accept(): %s\n",
 						strerror(errno));
 				}
 				else {
-					lprintf(7, "citserver: "
+					lprintf(CTDL_NOTICE,
 						"New client socket %d\n",
 						ssock);
 
@@ -1101,7 +1070,16 @@ SETUP_FD:	memcpy(&readfds, &masterfds, sizeof masterfds);
 		else {
 			bind_me = NULL;
 			begin_critical_section(S_SESSION_TABLE);
-			for (ptr = ContextList;
+			/*
+			 * We start where we left off.  If we get to the end
+			 * we'll start from the beginning again, then give up
+			 * if we still don't find anything.  This ensures
+			 * that all contexts get a more-or-less equal chance
+			 * to run. And yes, I did add a goto to the code. -IO
+			 */
+find_session:		if (next_session == NULL)
+				next_session = ContextList;
+			for (ptr = next_session;
 			    ( (ptr != NULL) && (bind_me == NULL) );
 			    ptr = ptr->next) {
 				if ( (FD_ISSET(ptr->client_socket, &readfds))
@@ -1114,6 +1092,13 @@ SETUP_FD:	memcpy(&readfds, &masterfds, sizeof masterfds);
 				 * letting anyone else touch the context list.
 				 */
 				bind_me->state = CON_EXECUTING;
+				next_session = bind_me->next;
+			} else if (next_session == ContextList) {
+				next_session = NULL;
+			}
+			if (bind_me == NULL && next_session != NULL) {
+				next_session = NULL;
+				goto find_session;
 			}
 
 			end_critical_section(S_SESSION_TABLE);
@@ -1141,3 +1126,154 @@ SETUP_FD:	memcpy(&readfds, &masterfds, sizeof masterfds);
 	--num_threads;
 	return NULL;
 }
+
+
+
+
+/*
+ * SyslogFacility()
+ * Translate text facility name to syslog.h defined value.
+ */
+int SyslogFacility(char *name)
+{
+	int i;
+	struct
+	{
+		int facility;
+		char *name;
+	}   facTbl[] =
+	{
+		{   LOG_KERN,   "kern"		},
+		{   LOG_USER,   "user"		},
+		{   LOG_MAIL,   "mail"		},
+		{   LOG_DAEMON, "daemon"	},
+		{   LOG_AUTH,   "auth"		},
+		{   LOG_SYSLOG, "syslog"	},
+		{   LOG_LPR,	"lpr"		},
+		{   LOG_NEWS,   "news"		},
+		{   LOG_UUCP,   "uucp"		},
+		{   LOG_LOCAL0, "local0"	},
+		{   LOG_LOCAL1, "local1"	},
+		{   LOG_LOCAL2, "local2"	},
+		{   LOG_LOCAL3, "local3"	},
+		{   LOG_LOCAL4, "local4"	},
+		{   LOG_LOCAL5, "local5"	},
+		{   LOG_LOCAL6, "local6"	},
+		{   LOG_LOCAL7, "local7"	},
+		{   0,		  NULL		}
+	};
+	for(i = 0; facTbl[i].name != NULL; i++) {
+		if(!strcasecmp(name, facTbl[i].name))
+			return facTbl[i].facility;
+	}
+	return -1;
+}
+
+
+/********** MEM CHEQQER ***********/
+
+#ifdef DEBUG_MEMORY_LEAKS
+
+#undef malloc
+#undef realloc
+#undef strdup
+#undef free
+
+void *tracked_malloc(size_t size, char *file, int line) {
+	struct igheap *thisheap;
+	void *block;
+
+	block = malloc(size);
+	if (block == NULL) return(block);
+
+	thisheap = malloc(sizeof(struct igheap));
+	if (thisheap == NULL) {
+		free(block);
+		return(NULL);
+	}
+
+	thisheap->block = block;
+	strcpy(thisheap->file, file);
+	thisheap->line = line;
+	
+	begin_critical_section(S_DEBUGMEMLEAKS);
+	thisheap->next = igheap;
+	igheap = thisheap;
+	end_critical_section(S_DEBUGMEMLEAKS);
+
+	return(block);
+}
+
+
+void *tracked_realloc(void *ptr, size_t size, char *file, int line) {
+	struct igheap *thisheap;
+	void *block;
+
+	block = realloc(ptr, size);
+	if (block == NULL) return(block);
+
+	thisheap = malloc(sizeof(struct igheap));
+	if (thisheap == NULL) {
+		free(block);
+		return(NULL);
+	}
+
+	thisheap->block = block;
+	strcpy(thisheap->file, file);
+	thisheap->line = line;
+	
+	begin_critical_section(S_DEBUGMEMLEAKS);
+	thisheap->next = igheap;
+	igheap = thisheap;
+	end_critical_section(S_DEBUGMEMLEAKS);
+
+	return(block);
+}
+
+
+
+void tracked_free(void *ptr) {
+	struct igheap *thisheap;
+	struct igheap *trash;
+
+	free(ptr);
+
+	if (igheap == NULL) return;
+	begin_critical_section(S_DEBUGMEMLEAKS);
+	for (thisheap = igheap; thisheap != NULL; thisheap = thisheap->next) {
+		if (thisheap->next != NULL) {
+			if (thisheap->next->block == ptr) {
+				trash = thisheap->next;
+				thisheap->next = thisheap->next->next;
+				free(trash);
+			}
+		}
+	}
+	if (igheap->block == ptr) {
+		trash = igheap;
+		igheap = igheap->next;
+		free(trash);
+	}
+	end_critical_section(S_DEBUGMEMLEAKS);
+}
+
+char *tracked_strdup(const char *s, char *file, int line) {
+	char *ptr;
+
+	if (s == NULL) return(NULL);
+	ptr = tracked_malloc(strlen(s) + 1, file, line);
+	if (ptr == NULL) return(NULL);
+	strncpy(ptr, s, strlen(s));
+	return(ptr);
+}
+
+void dump_heap(void) {
+	struct igheap *thisheap;
+
+	for (thisheap = igheap; thisheap != NULL; thisheap = thisheap->next) {
+		lprintf(CTDL_CRIT, "UNFREED: %30s : %d\n",
+			thisheap->file, thisheap->line);
+	}
+}
+
+#endif /*  DEBUG_MEMORY_LEAKS */
