@@ -1,5 +1,5 @@
 /* 
- * $Id: user_ops.c,v 613.0 2003/12/15 16:37:04 ajc Exp $
+ * $Id: user_ops.c,v 620.0 2004/03/27 02:33:34 ajc Exp $
  *
  * Server functions which perform operations on user objects.
  *
@@ -33,7 +33,6 @@
 #endif
 
 #include <string.h>
-#include <syslog.h>
 #include <limits.h>
 #ifndef ENABLE_CHKPWD
 #include "auth.h"
@@ -353,6 +352,7 @@ int CtdlLoginExistingUser(char *trythisname)
 	char username[SIZ];
 	int found_user;
 	struct recptypes *valid = NULL;
+	struct passwd *p = NULL;
 
 	if (trythisname == NULL) return login_not_found;
 	safestrncpy(username, trythisname, sizeof username);
@@ -375,10 +375,24 @@ int CtdlLoginExistingUser(char *trythisname)
 				found_user = getuser(&CC->user,
 						valid->recp_local);
 			}
-			phree(valid);
+			free(valid);
 		}
 	}
 
+#ifdef ENABLE_AUTOLOGIN
+	/* If we haven't found the account yet, and the supplied name
+	 * is a login name on the underlying host system, create the
+	 * account.
+	 */
+	if (found_user != 0) {
+		p = (struct passwd *) getpwnam(username);
+		if (p != NULL) {
+			create_user(username, 0);
+			found_user = getuser(&CC->user, username);
+		}
+	}
+#endif /* ENABLE_AUTOLOGIN */
+
 	/* Did we find something? */
 	if (found_user == 0) {
 		if (((CC->nologin)) && (CC->user.axlevel < 6)) {
@@ -408,7 +422,7 @@ void cmd_user(char *cmdbuf)
 	a = CtdlLoginExistingUser(username);
 	switch (a) {
 	case login_already_logged_in:
-		cprintf("%d Already logged in.\n", ERROR);
+		cprintf("%d Already logged in.\n", ERROR + ALREADY_LOGGED_IN);
 		return;
 	case login_too_many_users:
 		cprintf("%d %s: "
@@ -422,9 +436,10 @@ void cmd_user(char *cmdbuf)
 			MORE_DATA, CC->curr_user);
 		return;
 	case login_not_found:
-		cprintf("%d %s not found.\n", ERROR, username);
+		cprintf("%d %s not found.\n", ERROR + NO_SUCH_USER, username);
 		return;
-		cprintf("%d Internal error\n", ERROR);
+	default:
+		cprintf("%d Internal error\n", ERROR + INTERNAL_ERROR);
 	}
 }
 
@@ -437,8 +452,7 @@ void session_startup(void)
 {
 	int i;
 
-	syslog(LOG_NOTICE, "Session %d: %s logged in",
-	       CC->cs_pid, CC->curr_user);
+	lprintf(CTDL_NOTICE, "<%s> logged in\n", CC->curr_user);
 
 	lgetuser(&CC->user, CC->curr_user);
 	++(CC->user.timescalled);
@@ -551,13 +565,13 @@ static int validpw(uid_t uid, const char *pass)
 	char buf[24];
 
 	if (pipe(pipev)) {
-		lprintf(1, "pipe failed (%s): denying autologin access for "
+		lprintf(CTDL_ERR, "pipe failed (%s): denying autologin access for "
 			"uid %ld\n", strerror(errno), (long)uid);
 		return 0;
 	}
 	switch (pid = fork()) {
 	case -1:
-		lprintf(1, "fork failed (%s): denying autologin access for "
+		lprintf(CTDL_ERR, "fork failed (%s): denying autologin access for "
 			"uid %ld\n", strerror(errno), (long)uid);
 		close(pipev[0]);
 		close(pipev[1]);
@@ -585,7 +599,7 @@ static int validpw(uid_t uid, const char *pass)
 
 	while (waitpid(pid, &status, 0) == -1)
 		if (errno != EINTR) {
-			lprintf(1, "waitpid failed (%s): denying autologin "
+			lprintf(CTDL_ERR, "waitpid failed (%s): denying autologin "
 				"access for uid %ld\n",
 				strerror(errno), (long)uid);
 			return 0;
@@ -609,19 +623,19 @@ int CtdlTryPassword(char *password)
 	int code;
 
 	if ((CC->logged_in)) {
-		lprintf(5, "CtdlTryPassword: already logged in\n");
+		lprintf(CTDL_WARNING, "CtdlTryPassword: already logged in\n");
 		return pass_already_logged_in;
 	}
 	if (!strcmp(CC->curr_user, NLI)) {
-		lprintf(5, "CtdlTryPassword: no user selected\n");
+		lprintf(CTDL_WARNING, "CtdlTryPassword: no user selected\n");
 		return pass_no_user;
 	}
 	if (getuser(&CC->user, CC->curr_user)) {
-		lprintf(5, "CtdlTryPassword: internal error\n");
+		lprintf(CTDL_ERR, "CtdlTryPassword: internal error\n");
 		return pass_internal_error;
 	}
 	if (password == NULL) {
-		lprintf(5, "CtdlTryPassword: NULL password string supplied\n");
+		lprintf(CTDL_INFO, "CtdlTryPassword: NULL password string supplied\n");
 		return pass_wrong_password;
 	}
 	code = (-1);
@@ -658,7 +672,7 @@ int CtdlTryPassword(char *password)
 		do_login();
 		return pass_ok;
 	} else {
-		lprintf(3, "Bad password specified for <%s>\n", CC->curr_user);
+		lprintf(CTDL_WARNING, "Bad password specified for <%s>\n", CC->curr_user);
 		return pass_wrong_password;
 	}
 }
@@ -674,14 +688,14 @@ void cmd_pass(char *buf)
 
 	switch (a) {
 	case pass_already_logged_in:
-		cprintf("%d Already logged in.\n", ERROR);
+		cprintf("%d Already logged in.\n", ERROR + ALREADY_LOGGED_IN);
 		return;
 	case pass_no_user:
 		cprintf("%d You must send a name with USER first.\n",
-			ERROR);
+			ERROR + USERNAME_REQUIRED);
 		return;
 	case pass_wrong_password:
-		cprintf("%d Wrong password.\n", ERROR);
+		cprintf("%d Wrong password.\n", ERROR + PASSWORD_REQUIRED);
 		return;
 	case pass_ok:
 		logged_in_response();
@@ -707,7 +721,7 @@ int purge_user(char pname[])
 	makeuserkey(usernamekey, pname);
 
 	if (getuser(&usbuf, pname) != 0) {
-		lprintf(5, "Cannot purge user <%s> - not found\n", pname);
+		lprintf(CTDL_ERR, "Cannot purge user <%s> - not found\n", pname);
 		return (ERROR + NO_SUCH_USER);
 	}
 	/* Don't delete a user who is currently logged in.  Instead, just
@@ -723,15 +737,15 @@ int purge_user(char pname[])
 	}
 	end_critical_section(S_SESSION_TABLE);
 	if (user_is_logged_in == 1) {
-		lprintf(5, "User <%s> is logged in; not deleting.\n", pname);
+		lprintf(CTDL_WARNING, "User <%s> is logged in; not deleting.\n", pname);
 		usbuf.axlevel = 0;
 		putuser(&usbuf);
 		return (1);
 	}
-	lprintf(5, "Deleting user <%s>\n", pname);
+	lprintf(CTDL_NOTICE, "Deleting user <%s>\n", pname);
 
 	/* Perform any purge functions registered by server extensions */
-	PerformUserHooks(usbuf.fullname, usbuf.usernum, EVT_PURGEUSER);
+	PerformUserHooks(&usbuf, EVT_PURGEUSER);
 
 	/* delete any existing user/room relationships */
 	cdb_delete(CDB_VISIT, &usbuf.usernum, sizeof(long));
@@ -827,8 +841,11 @@ int create_user(char *newusername, int become_user)
                 lputroom(&qrbuf);
         }
 
+	/* Perform any create functions registered by server extensions */
+	PerformUserHooks(&usbuf, EVT_NEWUSER);
+
 	/* Everything below this line can be bypassed if administratively
-	   creating a user, instead of doing self-service account creation
+	 * creating a user, instead of doing self-service account creation
 	 */
 
 	if (become_user) {
@@ -843,7 +860,7 @@ int create_user(char *newusername, int become_user)
 		}
 	}
 
-	lprintf(3, "New user <%s> created\n", username);
+	lprintf(CTDL_NOTICE, "New user <%s> created\n", username);
 	return (0);
 }
 
@@ -860,12 +877,12 @@ void cmd_newu(char *cmdbuf)
 
 	if (config.c_disable_newu) {
 		cprintf("%d Self-service user account creation "
-			"is disabled on this system.\n", ERROR);
+			"is disabled on this system.\n", ERROR + NOT_HERE);
 		return;
 	}
 
 	if (CC->logged_in) {
-		cprintf("%d Already logged in.\n", ERROR);
+		cprintf("%d Already logged in.\n", ERROR + ALREADY_LOGGED_IN);
 		return;
 	}
 	if (CC->nologin) {
@@ -878,14 +895,14 @@ void cmd_newu(char *cmdbuf)
 	strproc(username);
 
 	if (strlen(username) == 0) {
-		cprintf("%d You must supply a user name.\n", ERROR);
+		cprintf("%d You must supply a user name.\n", ERROR + USERNAME_REQUIRED);
 		return;
 	}
 
 	if ((!strcasecmp(username, "bbs")) ||
 	    (!strcasecmp(username, "new")) ||
 	    (!strcasecmp(username, "."))) {
-		cprintf("%d '%s' is an invalid login name.\n", ERROR, username);
+		cprintf("%d '%s' is an invalid login name.\n", ERROR + ILLEGAL_VALUE, username);
 		return;
 	}
 
@@ -903,7 +920,7 @@ void cmd_newu(char *cmdbuf)
 			ERROR + INTERNAL_ERROR);
 		return;
 	} else {
-		cprintf("%d unknown error\n", ERROR);
+		cprintf("%d unknown error\n", ERROR + INTERNAL_ERROR);
 	}
 }
 
@@ -918,7 +935,7 @@ void cmd_setp(char *new_pw)
 		return;
 	}
 	if ( (CC->user.uid != BBSUID) && (CC->user.uid != (-1)) ) {
-		cprintf("%d Not allowed.  Use the 'passwd' command.\n", ERROR);
+		cprintf("%d Not allowed.  Use the 'passwd' command.\n", ERROR + NOT_HERE);
 		return;
 	}
 	strproc(new_pw);
@@ -930,7 +947,7 @@ void cmd_setp(char *new_pw)
 	safestrncpy(CC->user.password, new_pw, sizeof(CC->user.password));
 	lputuser(&CC->user);
 	cprintf("%d Password changed.\n", CIT_OK);
-	lprintf(3, "Password changed for user <%s>\n", CC->curr_user);
+	lprintf(CTDL_INFO, "Password changed for user <%s>\n", CC->curr_user);
 	PerformSessionHooks(EVT_SETPASS);
 }
 
@@ -957,7 +974,7 @@ void cmd_creu(char *cmdbuf)
 	strproc(password);
 
 	if (strlen(username) == 0) {
-		cprintf("%d You must supply a user name.\n", ERROR);
+		cprintf("%d You must supply a user name.\n", ERROR + USERNAME_REQUIRED);
 		return;
 	}
 
@@ -978,7 +995,7 @@ void cmd_creu(char *cmdbuf)
 			ERROR + ALREADY_EXISTS, username);
 		return;
 	} else {
-		cprintf("%d An error occured creating the user account.\n", ERROR);
+		cprintf("%d An error occured creating the user account.\n", ERROR + INTERNAL_ERROR);
 	}
 }
 
@@ -1011,7 +1028,7 @@ void cmd_setu(char *new_parms)
 		return;
 
 	if (num_parms(new_parms) < 3) {
-		cprintf("%d Usage error.\n", ERROR);
+		cprintf("%d Usage error.\n", ERROR + ILLEGAL_VALUE);
 		return;
 	}
 	lgetuser(&CC->user, CC->curr_user);
@@ -1071,7 +1088,7 @@ void cmd_seen(char *argbuf) {
 	}
 
 	if (num_parms(argbuf) != 2) {
-		cprintf("%d Invalid parameters\n", ERROR);
+		cprintf("%d Invalid parameters\n", ERROR + ILLEGAL_VALUE);
 		return;
 	}
 
@@ -1095,16 +1112,48 @@ void cmd_gtsn(char *argbuf) {
 }
 
 
+/*
+ * API function for cmd_invt_kick() and anything else that needs to
+ * invite or kick out a user to/from a room.
+ * 
+ * Set iuser to the name of the user, and op to 1=invite or 0=kick
+ */
+int CtdlInvtKick(char *iuser, int op) {
+	struct ctdluser USscratch;
+	struct visit vbuf;
+	char bbb[SIZ];
+
+	if (getuser(&USscratch, iuser) != 0) {
+		return(1);
+	}
+
+	CtdlGetRelationship(&vbuf, &USscratch, &CC->room);
+	if (op == 1) {
+		vbuf.v_flags = vbuf.v_flags & ~V_FORGET & ~V_LOCKOUT;
+		vbuf.v_flags = vbuf.v_flags | V_ACCESS;
+	}
+	if (op == 0) {
+		vbuf.v_flags = vbuf.v_flags & ~V_ACCESS;
+		vbuf.v_flags = vbuf.v_flags | V_FORGET | V_LOCKOUT;
+	}
+	CtdlSetRelationship(&vbuf, &USscratch, &CC->room);
+
+	/* post a message in Aide> saying what we just did */
+	snprintf(bbb, sizeof bbb, "%s %s %s> by %s\n",
+		iuser,
+		((op == 1) ? "invited to" : "kicked out of"),
+		CC->room.QRname,
+		CC->user.fullname);
+	aide_message(bbb);
+
+	return(0);
+}
+
 
 /*
  * INVT and KICK commands
  */
-void cmd_invt_kick(char *iuser, int op)
-			/* user name */
-{				/* 1 = invite, 0 = kick out */
-	struct ctdluser USscratch;
-	char bbb[SIZ];
-	struct visit vbuf;
+void cmd_invt_kick(char *iuser, int op) {
 
 	/*
 	 * These commands are only allowed by aides, room aides,
@@ -1127,31 +1176,10 @@ void cmd_invt_kick(char *iuser, int op)
 		return;
 	}
 
-	if (lgetuser(&USscratch, iuser) != 0) {
-		cprintf("%d No such user.\n", ERROR);
+	if (CtdlInvtKick(iuser, op) != 0) {
+		cprintf("%d No such user.\n", ERROR + NO_SUCH_USER);
 		return;
 	}
-	CtdlGetRelationship(&vbuf, &USscratch, &CC->room);
-
-	if (op == 1) {
-		vbuf.v_flags = vbuf.v_flags & ~V_FORGET & ~V_LOCKOUT;
-		vbuf.v_flags = vbuf.v_flags | V_ACCESS;
-	}
-	if (op == 0) {
-		vbuf.v_flags = vbuf.v_flags & ~V_ACCESS;
-		vbuf.v_flags = vbuf.v_flags | V_FORGET | V_LOCKOUT;
-	}
-	CtdlSetRelationship(&vbuf, &USscratch, &CC->room);
-
-	lputuser(&USscratch);
-
-	/* post a message in Aide> saying what we just did */
-	snprintf(bbb, sizeof bbb, "%s %s %s> by %s\n",
-		iuser,
-		((op == 1) ? "invited to" : "kicked out of"),
-		CC->room.QRname,
-		CC->user.fullname);
-	aide_message(bbb);
 
 	cprintf("%d %s %s %s.\n",
 		CIT_OK, iuser,
@@ -1204,7 +1232,7 @@ void cmd_forg(void)
 		cprintf("%d Ok\n", CIT_OK);
 	}
 	else {
-		cprintf("%d You may not forget this room.\n", ERROR);
+		cprintf("%d You may not forget this room.\n", ERROR + NOT_HERE);
 	}
 }
 
@@ -1549,7 +1577,7 @@ int InitialMailCheck()
         cdbfr = cdb_fetch(CDB_MSGLISTS, &mailbox.QRnumber, sizeof(long));
 
         if (cdbfr != NULL) {
-                msglist = mallok(cdbfr->len);
+                msglist = malloc(cdbfr->len);
                 memcpy(msglist, cdbfr->ptr, cdbfr->len);
                 num_msgs = cdbfr->len / sizeof(long);
                 cdb_free(cdbfr);
@@ -1563,7 +1591,7 @@ int InitialMailCheck()
                         }
                 }
         if (msglist != NULL)
-                phree(msglist);
+                free(msglist);
 
         return (num_newmsgs);
 }
